/*
   ----------------------------------------------------------------------------
   BusinessLayer.cc
	Programmer: Ken Mayer
   Date......: May, 2015
   Notes.....: Business Layer for sample Invoice system for
	            The dBASE Book Plus.
					
					The idea is to put any data validation code here.
					
					As this code does not understand the data layer, 
					the programmer must pass information to the individual
					methods, which will return a value of true or false,
					denoting if the data is valid. 
					
					The User Interface layer will pass the information to
					the Business Layer, and if everything comes out as 
					valid, then it will pass the data to the Data Layer
					(in the .DMD in this application).

   Usage:      set procedure to BusinessLayer.cc
	            bOK = InvoiceDateValid( form.InvoiceDate.value )
               close procedure BusinessLayer.cc

   Functions:  InvoiceDateValid( <dDate> )
	
	----------------------------------------------------------------------------				
*/

function InvoiceDateValid( dDate )
	// Validate an Invoice Date field:
	local bOK
	bOK = true

	if dDate > date() // Invoice in the future?
		msgbox( "The date '"+dDate+"' is in the future. "+;
				  "This is not acceptable.",;
				  "Invalid Date", 16 )
		bOK := false
	endif
	
	if dDate < date() // Invoice date in the past?
		// ask user if this is intended/okay --
		// they might be updating the record due to
		// an error when the record was created:
		bOK = msgbox( "The date '"+dDate+"' is in the past. "+;
						  "If this is okay, click 'OK', otherwise "+;
						  "click 'Cancel'.",;
						  "Date May Be Invalid", 32+1 ) == 1 // 1 = OK, 2 = Cancel
	endif

return bOK
// End of function: InvoiceDateValid


// ----------------------------------------------------------------------------
// End of file: BusinessLayer.cc
