/*
   ----------------------------------------------------------------------------
	CreateDatabase.prg
   A sample dBL program from 
   The dBASE Book Plus by Kenneth J. Mayer. 
   Program copyright 2015. 
   Permission is granted for use as a learning tool for dBASE Plus.
   Sample from: Chapter 4
   
   Updated July, 2016 -- somewhere a bug crept in that does not allow
      saving "" to a character field, but " " works. This was causing
      problems in the Customers table. Don't know if this is the most
      recent version of dBASE, or if it is Windows 10, or ...
      Changed the code for this.
	
   A program to show the basic use of the following Objects:
	   ADODatabase
		ADOQuery
		ADORowset
		ADOFields
		ADOException
		ADOError
		ADOErrorArray
		
	Relies on:
	   ADOSeqValue.cc (handle autoincrement fields)
		Only if uncommented, however. 
		
	   Code has been added to use Firebird Generators/Sequences for
	   each table for the primary key fields. Hence, the ADOSeqValue.cc 
		references are not required.
		
		***********************************************************************
		Issue with using a trigger is that the value is created, but
		you cannot access it. By removing the trigger, you SHOULD
		be able to create the next value automatically, and then
		return the number, which you can then use in your code
		as needed ... Mervyn Bick has pointed this out ...
		
		The interesting thing is a bug discovered while trying to figure
		out why my code works and Mervyn's didn't. The bug is in
		dBASE Plus 9.51 (Mervyn was working with dBASE Plus 8.x at
		the time). The sample code for the use of the Gen_ID() function
		in Firebird, the documentation suggests using a metadata table
		called RDB$DATABASE. However, the bug in dBASE has to deal with
		the table name, and as it turns out, the generator values are
		not stored in that table. 
		
		The standard command in the documentation is:
		    select Gen_ID( <sequencename>,1) from RDB$DATABASE
		
		Another oddity was discovered when Andre Knappstein pointed out
		I could just use an existing table, and it should work. He suggested
		the Customers table. While testing, I discovered that if I used the
		Customers table (which when done here will have 6 rows in it), the
		generator function above (Gen_ID) will increment by 6, hence
		my testing gave me values 1, 7, 13, 19, etc. 
		
		For this example to work, the code here uses a dummy table named
		MyGen (see the code below), with one row in it. It increments
		by one, everything works as expected. 
		***********************************************************************
		
	This code connects to the sample database created in Firebird,
	using the Connection Alias "BookSample_ADO", removes existing
	tables, and Sequences and re-creates them, showing the use 
	of the executeSQL() method of the ADODatabase, 	populating the 
	tables with some sample data. You should have set up the
	database in Chapter 2. 
	----------------------------------------------------------------------------
*/

// Empty the output pane of the Command Window
clear
close databases

/*
	   -------------------------------------------------------------------------
      Connect to database
	   -------------------------------------------------------------------------
   */
try // whole program is wrapped in one large try/catch/endtry

	d = new ADODatabase()
	d.databaseName := "BookSample_ADO"
	// This is only necessary if the connection does not include
	// the username/password, OR if you wish to log in as a different
	// user than the default:
	d.loginString  := "SYSDBA/masterkey"
	d.active       := true
	? "Database is active!"
   ? // blank line
	
   /*
	   -------------------------------------------------------------------------
      Invoices table
	   -------------------------------------------------------------------------
   */   
	local cBuildTable, cCmd

   // if the table exists, remove it, and create new:
	if d.tableExists( "Invoices" )
		d.dropTable( "Invoices" )
	endif

	// remove the Sequence (Generator) if it exists
	try
	   d.executeSQL( "drop sequence InvNum_Seq" )
   catch ( ADOException e )
	   // if there is an error, it is probably that
		// the sequence doesn't exist:
	   ? e.errors[1].code, e.errors[1].message
	endtry

   // use a character variable    
	cBuildTable = [create table Invoices (]+;
	                           [ InvoiceNumber bigint,]+;
										[ InvoiceDate   date,]+;
										[ CustomerNumber bigint, ]+;
										[ Primary Key (InvoiceNumber ) )]
										
   // hopefully build the table:										
   d.executeSQL( cBuildTable )
   ? "Invoices Table Created successfully"

   // add the sequence (generator):
	try
		cCmd = "CREATE SEQUENCE InvNum_Seq"
		d.executeSQL( cCmd )
		? "Sequence (Generator) 'InvNum_Seq' created"

   catch ( ADOException e )
	   // if there is an error, it is probably that
		// the sequence doesn't exist:
	   ? e.errors[1].code, e.errors[1].message
      ? "Problem creating Sequence"
      cancel		
	endtry

   /*
	   -------------------------------------------------------------------------
      Customers table
	   -------------------------------------------------------------------------
   */
	
   ? // blank line in output pane of Command Window

   // if the table exists, remove it, and create new:
	if d.tableExists( "Customers" )
		d.dropTable( "Customers" )
	endif
	
   // remove the Sequence (Generator) if it exists
	try
	   d.executeSQL( "drop sequence CustNum_Seq" )
   catch ( ADOException e )
	   // if there is an error, it is probably that
		// the sequence doesn't exist:
	   ? e.errors[1].code, e.errors[1].message
	endtry
	
   // use a character variable    
	cBuildTable = [create table Customers (]+;
	                           [ CustomerNumber bigint,]+;
	                           [ BusinessName   char(30),]+;	
	                           [ Address1       char(25),]+;
										[ Address2       char(25),]+;
										[ City           char(30),]+;
										[ State          char(2),]+;
										[ PostalCode     char(10),]+;
										[ EmailAddress   char(60),]+;
										[ Primary Key (CustomerNumber ) )]
										
   // hopefully build the table:										
   d.executeSQL( cBuildTable )
   ? "Customers Table Created successfully"
	
	// add the sequence (generator):
	try
		cCmd = "CREATE SEQUENCE CustNum_Seq"
		d.executeSQL( cCmd )
		? "Sequence (Generator) 'CustNum_Seq' created"

   catch ( ADOException e )
	   // if there is an error, it is probably that
		// the sequence doesn't exist:
	   ? e.errors[1].code, e.errors[1].message
      ? "Problem creating Sequence"
      cancel		
	endtry
	
   /*
	   -------------------------------------------------------------------------
      LineItems table
	   -------------------------------------------------------------------------
   */

   ? // blank line in output pane of Command Window
	
   // if the table exists, remove it, and create new:
	if d.tableExists( "LineItems" )
		d.dropTable( "LineItems" )
	endif
	
   // remove the Sequence (Generator) if it exists
	try
	   d.executeSQL( "drop sequence LINum_Seq" )
   catch ( ADOException e )
	   // if there is an error, it is probably that
		// the sequence doesn't exist:
	   ? e.errors[1].code, e.errors[1].message
	endtry
	
   // use a character variable    
	cBuildTable = [create table LineItems (]+;
	                           [ LineItemNumber  bigint,]+;
	                           [ InvoiceNumber   bigint,]+;	
	                           [ ItemNumber      bigint,]+;
										[ Quantity        integer,]+;
										[ CostPerItem     numeric(6,2),]+;
										[ Primary Key (LineItemNumber ) )]
										
   // hopefully build the table:										
   d.executeSQL( cBuildTable )
   ? "LineItems Table Created successfully"

   // add the sequence (generator):
	try
		cCmd = "CREATE SEQUENCE LINum_Seq"
		d.executeSQL( cCmd )
		? "Sequence (Generator) 'LINum_Seq' created"
		
   catch ( ADOException e )
	   // if there is an error, it is probably that
		// the sequence doesn't exist:
	   ? e.errors[1].code, e.errors[1].message
      ? "Problem creating Sequence"
      cancel		
	endtry
	
   /*
	   -------------------------------------------------------------------------
      Inventory table
	   -------------------------------------------------------------------------
   */
   ? // blank line in Command Window
	
   // if the table exists, remove it, and create new:
	if d.tableExists( "Inventory" )
		d.dropTable( "Inventory" )
	endif
	
   // remove the Sequence (Generator):
	try
	   d.executeSQL( "drop sequence ItemNum_Seq" )
   catch ( ADOException e )
	   // if there is an error, it is probably that
		// the sequence doesn't exist:
	   ? e.errors[1].code, e.errors[1].message
	endtry
	
   // use a character variable    
	cBuildTable = [create table Inventory (]+;
	                           [ ItemNumber  bigint,]+;
	                           [ ItemDescription char(100),]+;
										[ CostPerItem     numeric(6,2),]+;
										[ Primary Key (ItemNumber ) )]
										
   // hopefully build the table:										
   d.executeSQL( cBuildTable )
   ? "Inventory Table Created successfully"

   // add the sequence (generator):
	try
		cCmd = "CREATE SEQUENCE ItemNum_Seq"
		d.executeSQL( cCmd )
		? "Sequence (Generator) 'ItemNum_Seq' created"

   catch ( ADOException e )
	   // if there is an error, it is probably that
		// the sequence doesn't exist:
	   ? e.errors[1].code, e.errors[1].message
      ? "Problem creating Sequence"
      cancel		
	endtry
	
	/*
	   ------------------------------------------------------------------------
		Table: MyGen 
		Dummy table used for the express purpose of Sequences (generators)
		in Firebird.
		------------------------------------------------------------------------
	*/

   ? // blank line in Command Window
	
   // if the table exists, remove it, and create new:
	if d.tableExists( "MyGen" )
		d.dropTable( "MyGen" )
	endif
	
   // use a character variable    
	cBuildTable = [create table MyGen ( DummyField  bigint )]
										
   // hopefully build the table:										
   d.executeSQL( cBuildTable )
   ? "MyGen Table Created successfully"
	
	// populate this table with a single record:
	qMyGen = new ADOQuery()
	qMyGen.database := d
	qMyGen.sql      := "insert into MyGen ( DummyField ) values (1)"
	qMyGen.active   := true
	qMyGen.active   := false
	release object qMyGen
	qMyGen := null

   /*
	   ------------------------------------------------------------------------
		Connect to a table using the ADOQuery object and adding data.
		Invoices table first:
		------------------------------------------------------------------------
	*/
	? // blank line in Command Window
   qInvoices = new ADOQuery()
	qInvoices.database := d

   // Adding invoices here, if you are not using 
	// Sequences, then uncomment the
	// references to nRows++ and comment out the 
	// ones that use the GenNextKey function:
	local nRows
	//nRows = 0
	
	// first row:
	//nRows++
	nRows = GetNextKey( "InvNum_Seq" )
	qInvoices.sql := "insert into Invoices ( InvoiceNumber, InvoiceDate, CustomerNumber ) ;
	                                values ( :InvNum, :InvDate, :CustNum )"
   qInvoices.parameters["InvNum"].value  := nRows
	qInvoices.parameters["InvDate"].value := {12/23/2014}
	qInvoices.parameters["CustNum"].value := 1
   qInvoices.active := true // activate and execute the command
	qInvoices.active := false // deactivate
	
	// next row:
	//nRows++
	nRows = GetNextKey( "InvNum_Seq" )	
	qInvoices.parameters["InvNum"].value  := nRows
	qInvoices.parameters["InvDate"].value := {12/27/2014}
	qInvoices.parameters["CustNum"].value := 4
   qInvoices.active := true // activate and execute the command
	qInvoices.active := false // deactivate

	// next row:
	//nRows++
	nRows = GetNextKey( "InvNum_Seq" )	
	qInvoices.parameters["InvNum"].value  := nRows
	qInvoices.parameters["InvDate"].value := {12/27/2014}
	qInvoices.parameters["CustNum"].value := 2
   qInvoices.active := true // activate and execute the command
	qInvoices.active := false // deactivate

	// next row:
	//nRows++
	nRows = GetNextKey( "InvNum_Seq" )
	qInvoices.parameters["InvNum"].value  := nRows
	qInvoices.parameters["InvDate"].value := {01/04/2015}
	qInvoices.parameters["CustNum"].value := 2
   qInvoices.active := true // activate and execute the command
	qInvoices.active := false // deactivate

	// next row:
	//nRows++
	nRows = GetNextKey( "InvNum_Seq" )
	qInvoices.parameters["InvNum"].value  := nRows
	qInvoices.parameters["InvDate"].value := {01/04/2015}
	qInvoices.parameters["CustNum"].value := 3
   qInvoices.active := true // activate and execute the command
	qInvoices.active := false // deactivate

	// next row:
	//nRows++
	nRows = GetNextKey( "InvNum_Seq" )
	qInvoices.parameters["InvNum"].value  := nRows
	qInvoices.parameters["InvDate"].value := {01/05/2015}
	qInvoices.parameters["CustNum"].value := 6
   qInvoices.active := true // activate and execute the command
	qInvoices.active := false // deactivate

	// next row:
	//nRows++
	nRows = GetNextKey( "InvNum_Seq" )
	qInvoices.parameters["InvNum"].value  := nRows
	qInvoices.parameters["InvDate"].value := {03/15/2015}
	qInvoices.parameters["CustNum"].value := 5
   qInvoices.active := true // activate and execute the command
	qInvoices.active := false // deactivate

	// next row:
	//nRows++
	nRows = GetNextKey( "InvNum_Seq" )
	qInvoices.parameters["InvNum"].value  := nRows
	qInvoices.parameters["InvDate"].value := {04/15/2015}
	qInvoices.parameters["CustNum"].value := 3
   qInvoices.active := true // activate and execute the command
	qInvoices.active := false // deactivate
	
   // next row:
	//nRows++
	nRows = GetNextKey( "InvNum_Seq" )
	qInvoices.parameters["InvNum"].value  := nRows
	qInvoices.parameters["InvDate"].value := {05/05/2015}
	qInvoices.parameters["CustNum"].value := 3
   qInvoices.active := true // activate and execute the command
	qInvoices.active := false // deactivate
	
	// next row:
	//nRows++
	nRows = GetNextKey( "InvNum_Seq" )
	qInvoices.parameters["InvNum"].value  := nRows
	qInvoices.parameters["InvDate"].value := {05/08/2015}
	qInvoices.parameters["CustNum"].value := 4
   qInvoices.active := true // activate and execute the command
	qInvoices.active := false // deactivate

   qInvoices.sql := "select * from Invoices"
	qInvoices.active := true
	nInvoices = qInvoices.rowset.count()
   ? "Invoice Table -- Rows added: "+nInvoices
	qInvoices.active := false

	/*
		------------------------------------------------------------------------
	   Customers Table -- add data -- need 6 customers
		------------------------------------------------------------------------
	*/
   qCustomers = new ADOQuery()
	qCustomers.database := d

	//nRows = 0
	
	// first row:
	//nRows++
	nRows = GetNextKey( "CustNum_Seq" )
	qCustomers.sql := "insert into Customers (CustomerNumber, BusinessName, Address1, Address2, ;
                                             City, State, PostalCode, EmailAddress );
                                     values ( :CustNum, :BusName, :Addr1, :Addr2, ;
												          :City, :State, :Postal, :Email )"
   qCustomers.parameters[ "CustNum" ].value  := nRows
   // here's where it gets tricky, if the character value can be > 20,
	// we should specify the length for the parameters, but in theory,
	// just this once:
	// Business name could be longer than 20:
   qCustomers.parameters[ "BusName"].type    := "Char"	
	qCustomers.parameters[ "BusName" ].length := 30
	qCustomers.parameters[ "BusName" ].value  := "A Very Fine Business"
	// Address1, Address2, City could be longer than 20:
	qCustomers.parameters[ "Addr1"].type      := "Char"
	qCustomers.parameters[ "Addr1"].length    := 25
	qCustomers.parameters[ "Addr1"].value     := "111 Main Street"
   qCustomers.parameters[ "Addr2"].type      := "Char"
	qCustomers.parameters[ "Addr2"].length    := 25
	qCustomers.parameters[ "Addr2"].value     := " " // need a space here, "" doesn't work
	qCustomers.parameters[ "City"].type       := "Char"
	qCustomers.parameters[ "City"].length     := 30
	qCustomers.parameters[ "City"].value      := "Anytown"
	qCustomers.parameters[ "State"].value     := "CA"
	qCustomers.parameters[ "Postal"].value    := "99881-1234"
	// Email can be up to 60 characters in length:
	qCustomers.parameters[ "Email"].type      := "Char"
	qCustomers.parameters[ "Email"].length    := 60
	qCustomers.parameters[ "Email"].value     := "AVFB@FineBusiness.com"
	qCustomers.active := true
	qCustomers.active := false

	// next row:
	//nRows++
   nRows = GetNextKey( "CustNum_Seq" )
   qCustomers.parameters[ "CustNum" ].value  := nRows
   qCustomers.parameters[ "BusName" ].value  := "Fred's Widget Shop"
   qCustomers.parameters[ "Addr1"].value     := "12345 Business Row"
   qCustomers.parameters[ "Addr2"].value     := "Number 678"
   qCustomers.parameters[ "City"].value      := "Anytown"
   qCustomers.parameters[ "State"].value     := "CA"
   qCustomers.parameters[ "Postal"].value    := "99881-1234"
   qCustomers.parameters[ "Email"].value     := "Fred@WidgetsRUs.com"
	qCustomers.active := true
	qCustomers.active := false

	// next row:
	//nRows++
   nRows = GetNextKey( "CustNum_Seq" )
   qCustomers.parameters[ "CustNum" ].value  := nRows
   qCustomers.parameters[ "BusName" ].value  := "Joe's Machine Parts"
   qCustomers.parameters[ "Addr1"].value     := "543 Some Street"
   qCustomers.parameters[ "Addr2"].value     := " "
   qCustomers.parameters[ "City"].value      := "Another Town"
   qCustomers.parameters[ "State"].value     := "CA"
   qCustomers.parameters[ "Postal"].value    := "99771-2345"
   qCustomers.parameters[ "Email"].value     := "Joe@JoesMachineParts.net"
	qCustomers.active := true
	qCustomers.active := false

	// next row:
	//nRows++
   nRows = GetNextKey( "CustNum_Seq" )
   qCustomers.parameters[ "CustNum" ].value  := nRows
   qCustomers.parameters[ "BusName" ].value  := "Best Mechanic In Town"
   qCustomers.parameters[ "Addr1"].value     := "P.O. Box 555"
   qCustomers.parameters[ "Addr2"].value     := " "
   qCustomers.parameters[ "City"].value      := "Somewhere"
   qCustomers.parameters[ "State"].value     := "CA"
   qCustomers.parameters[ "Postal"].value    := "99661-6789"
   qCustomers.parameters[ "Email"].value     := "Best@Mechanic.com"
	qCustomers.active := true
	qCustomers.active := false

	// next row:
	//nRows++
   nRows = GetNextKey( "CustNum_Seq" )
   qCustomers.parameters[ "CustNum" ].value  := nRows
   qCustomers.parameters[ "BusName" ].value  := "Jane's Widget Shop"
   qCustomers.parameters[ "Addr1"].value     := "5378 Side Street"
   qCustomers.parameters[ "Addr2"].value     := " "
   qCustomers.parameters[ "City"].value      := "Anywhere"
   qCustomers.parameters[ "State"].value     := "CA"
   qCustomers.parameters[ "Postal"].value    := "99551-6712"
   qCustomers.parameters[ "Email"].value     := "Jane@WeAreWidgets.com"
   qCustomers.active := true
	qCustomers.active := false

	// next row:
	//nRows++
   nRows = GetNextKey( "CustNum_Seq" )
   qCustomers.parameters[ "CustNum" ].value  := nRows
   qCustomers.parameters[ "BusName" ].value  := "Machine Parts, Inc."
   qCustomers.parameters[ "Addr1"].value     := "9999 N. Main"
   qCustomers.parameters[ "Addr2"].value     := " "
   qCustomers.parameters[ "City"].value      := "Somewhere"
   qCustomers.parameters[ "State"].value     := "CA"
   qCustomers.parameters[ "Postal"].value    := "99662-6789"
   qCustomers.parameters[ "Email"].value     := "Sales@MachinePartsInc.com"
   qCustomers.active := true
	qCustomers.active := false

   qCustomers.sql := "select * from Customers"
	qCustomers.active := true
	nCustomers = qCustomers.rowset.count()
   ? "Customers Table -- Rows added: "+nCustomers
	qCustomers.active := false

   /*
	   -------------------------------------------------------------------------
      Inventory Table -- we're keeping the inventory *REALLY* simple
	   for this company, we only have five items ...
		------------------------------------------------------------------------
   */		
	qInv = new ADOQuery()
	qInv.database     := d
   nRows := 0
   
   qInv.SQL := ( "insert into Inventory ( ItemNumber, ItemDescription, ;
	                                       CostPerItem );
	                              values (:ItemNum, :ItemDesc, :Cost )" )	
   // Inventory Item 1
   //nRows++
   nRows = GetNextKey( "ItemNum_Seq" )
   qInv.parameters["ItemNum"].value   := nRows
	qInv.parameters["ItemDesc"].type   := "Char"
	qInv.parameters["ItemDesc"].length := 100
	qInv.parameters["ItemDesc"].value  := '1/2" Widget'
   qInv.parameters["Cost"].value      := 1.50
	qInv.active := true
	qInv.active := false

   // Next item:
	//nRows++
   nRows = GetNextKey( "ItemNum_Seq" )
   qInv.parameters["ItemNum"].value   := nRows
	qInv.parameters["ItemDesc"].value  := '3/4" Widget'
   qInv.parameters["Cost"].value      := 1.65
	qInv.active := true
	qInv.active := false

   // Next item:
	//nRows++
   nRows = GetNextKey( "ItemNum_Seq" )
   qInv.parameters["ItemNum"].value   := nRows
	qInv.parameters["ItemDesc"].value  := '5" Gangly Wrench'
   qInv.parameters["Cost"].value      := 5.45
	qInv.active := true
	qInv.active := false

   // Next item:
	//nRows++
   nRows = GetNextKey( "ItemNum_Seq" )
   qInv.parameters["ItemNum"].value   := nRows
	qInv.parameters["ItemDesc"].value  := '10 Pack 1/2" Widgets'
   qInv.parameters["Cost"].value      := 10.00
	qInv.active := true
	qInv.active := false

   // Next item:
	//nRows++
   nRows = GetNextKey( "ItemNum_Seq" )
   qInv.parameters["ItemNum"].value   := nRows
	qInv.parameters["ItemDesc"].value  := '10 Pack 3/4" Widgets'
   qInv.parameters["Cost"].value      := 12.00
	qInv.active := true
	qInv.active := false

   qInv.sql := "select * from Inventory"
	qInv.active := true
	nInv = qInv.rowset.count()
   ? "Inventory Table -- Rows added: "+nInv
	qInv.active := false

   /*
		------------------------------------------------------------------------
      LineItems -- this will be tricky. There has to be at least
	   one LineItem per invoice, there are 14 invoices ... 
	   There should be multiple items for some of the Invoices.
	   ------------------------------------------------------------------------
   */
   qLI = new ADOQuery()
	qLI.database     := d
	
	nRows = 0
	
	qLI.SQL := ( "insert into LineItems ( LineItemNumber, InvoiceNumber, ;
	                                      ItemNumber, Quantity, CostPerItem );
	                             values (:LINum, :InvNum, :ItemNum, :Quan, :Cost )" )
   // Invoice 1, LineItem 1:
	//nRows++										  
   nRows = GetNextKey( "LINum_Seq" )
   qLI.parameters["LINum"].value     := nRows
	qLI.parameters["InvNum"].value    := 1
	qLI.parameters["ItemNum"].value   := 1
	qLI.parameters["Quan"].value      := 3
   qLI.parameters["Cost"].value      := 1.50 
	qLI.active := true
	qLI.active := false

	// Invoice 1, LineItem 2:
	//nRows++
   nRows = GetNextKey( "LINum_Seq" )
	qLI.parameters["LINum"].value     := nRows
	qLI.parameters["InvNum"].value    := 1
	qLI.parameters["ItemNum"].value   := 3
	qLI.parameters["Quan"].value      := 5
   qLI.parameters["Cost"].value      := 5.45
	qLI.active := true
	qLI.active := false

	// Invoice 2, LineItem 1:
	//nRows++
   nRows = GetNextKey( "LINum_Seq" )
	qLI.parameters["LINum"].value     := nRows
	qLI.parameters["InvNum"].value    := 2
	qLI.parameters["ItemNum"].value   := 2
	qLI.parameters["Quan"].value      := 10
   qLI.parameters["Cost"].value      := 1.65
   qLI.active := true
	qLI.active := false

	// Invoice 3, LineItem 1:
	//nRows++
   nRows = GetNextKey( "LINum_Seq" )
	qLI.parameters["LINum"].value     := nRows
	qLI.parameters["InvNum"].value    := 3
	qLI.parameters["ItemNum"].value   := 5
	qLI.parameters["Quan"].value      := 3
   qLI.parameters["Cost"].value      := 12.00
   qLI.active := true
	qLI.active := false

   // Invoice 3, LineItem 2:
	//nRows++
   nRows = GetNextKey( "LINum_Seq" )
	qLI.parameters["LINum"].value     := nRows
	qLI.parameters["InvNum"].value    := 3
	qLI.parameters["ItemNum"].value   := 4
	qLI.parameters["Quan"].value      := 5
   qLI.parameters["Cost"].value      := 10.00
   qLI.active := true
	qLI.active := false
	
	// Invoice 3, LineItem 3:
	//nRows++
   nRows = GetNextKey( "LINum_Seq" )
	qLI.parameters["LINum"].value     := nRows
	qLI.parameters["InvNum"].value    := 3
	qLI.parameters["ItemNum"].value   := 2
	qLI.parameters["Quan"].value      := 10
   qLI.parameters["Cost"].value      := 1.65
	qLI.active := true
	qLI.active := false

	
   // Invoice 3, LineItem 4:
	//nRows++
   nRows = GetNextKey( "LINum_Seq" )
	qLI.parameters["LINum"].value     := nRows
	qLI.parameters["InvNum"].value    := 3
	qLI.parameters["ItemNum"].value   := 1
	qLI.parameters["Quan"].value      := 6
   qLI.parameters["Cost"].value      := 1.50
	qLI.active := true
	qLI.active := false
	
	// Invoice 4, LineItem 1:
	//nRows++
   nRows = GetNextKey( "LINum_Seq" )
	qLI.parameters["LINum"].value     := nRows
	qLI.parameters["InvNum"].value    := 4
	qLI.parameters["ItemNum"].value   := 2
	qLI.parameters["Quan"].value      := 5
   qLI.parameters["Cost"].value      := 1.65
	qLI.active := true
	qLI.active := false

	// Invoice 4, LineItem 2:
	//nRows++
   nRows = GetNextKey( "LINum_Seq" )
	qLI.parameters["LINum"].value     := nRows
	qLI.parameters["InvNum"].value    := 4
	qLI.parameters["ItemNum"].value   := 4
	qLI.parameters["Quan"].value      := 5
   qLI.parameters["Cost"].value      := 10.00
	qLI.active := true
	qLI.active := false

	// Invoice 5, LineItem 1:
	//nRows++
   nRows = GetNextKey( "LINum_Seq" )
	qLI.parameters["LINum"].value     := nRows
	qLI.parameters["InvNum"].value    := 5
	qLI.parameters["ItemNum"].value   := 1
	qLI.parameters["Quan"].value      := 15
   qLI.parameters["Cost"].value      := 1.50
	qLI.active := true
	qLI.active := false

	// Invoice 5, LineItem 2:
	//nRows++
   nRows = GetNextKey( "LINum_Seq" )
	qLI.parameters["LINum"].value     := nRows
	qLI.parameters["InvNum"].value    := 5
	qLI.parameters["ItemNum"].value   := 2
	qLI.parameters["Quan"].value      := 6
   qLI.parameters["Cost"].value      := 1.65
	qLI.active := true
	qLI.active := false

   // Invoice 6, LineItem 1:
	//nRows++
   nRows = GetNextKey( "LINum_Seq" )
	qLI.parameters["LINum"].value     := nRows
	qLI.parameters["InvNum"].value    := 6
	qLI.parameters["ItemNum"].value   := 1
	qLI.parameters["Quan"].value      := 20
   qLI.parameters["Cost"].value      := 1.50
	qLI.active := true
	qLI.active := false

   // Invoice 7, LineItem 1:
	//nRows++
   nRows = GetNextKey( "LINum_Seq" )
	qLI.parameters["LINum"].value     := nRows
	qLI.parameters["InvNum"].value    := 7
	qLI.parameters["ItemNum"].value   := 2
	qLI.parameters["Quan"].value      := 5
   qLI.parameters["Cost"].value      := 1.65
	qLI.active := true
	qLI.active := false
	
   // Invoice 7, LineItem 2:
	//nRows++
   nRows = GetNextKey( "LINum_Seq" )
	qLI.parameters["LINum"].value     := nRows
	qLI.parameters["InvNum"].value    := 7
	qLI.parameters["ItemNum"].value   := 3
	qLI.parameters["Quan"].value      := 2
   qLI.parameters["Cost"].value      := 5.45
	qLI.active := true
	qLI.active := false
	
	// Invoice 8, LineItem 1:
	//nRows++
   nRows = GetNextKey( "LINum_Seq" )
	qLI.parameters["LINum"].value     := nRows
	qLI.parameters["InvNum"].value    := 8
	qLI.parameters["ItemNum"].value   := 1
	qLI.parameters["Quan"].value      := 15
   qLI.parameters["Cost"].value      := 1.50
	qLI.active := true
	qLI.active := false
	
	// Invoice 8, LineItem 2:
	//nRows++
   nRows = GetNextKey( "LINum_Seq" )
	qLI.parameters["LINum"].value     := nRows
	qLI.parameters["InvNum"].value    := 8
	qLI.parameters["ItemNum"].value   := 2
	qLI.parameters["Quan"].value      := 12
   qLI.parameters["Cost"].value      := 1.65
	qLI.active := true
	qLI.active := false
	
	// Invoice 9, LineItem 1:
	//nRows++
   nRows = GetNextKey( "LINum_Seq" )
	qLI.parameters["LINum"].value     := nRows
	qLI.parameters["InvNum"].value    := 9
	qLI.parameters["ItemNum"].value   := 4
	qLI.parameters["Quan"].value      := 5
   qLI.parameters["Cost"].value      := 10.00
	qLI.active := true
	qLI.active := false
	
	// Invoice 9, LineItem 2:
	//nRows++
   nRows = GetNextKey( "LINum_Seq" )
	qLI.parameters["LINum"].value     := nRows
	qLI.parameters["InvNum"].value    := 9
	qLI.parameters["ItemNum"].value   := 5
	qLI.parameters["Quan"].value      := 5
   qLI.parameters["Cost"].value      := 12.00
	qLI.active := true
	qLI.active := false
	
	// Invoice 10, LineItem 1:
	//nRows++
   nRows = GetNextKey( "LINum_Seq" )
//? nRows	
	qLI.parameters["LINum"].value     := nRows
	qLI.parameters["InvNum"].value    := 10
	qLI.parameters["ItemNum"].value   := 3
	qLI.parameters["Quan"].value      := 3
   qLI.parameters["Cost"].value      := 5.45
	qLI.active := true
	qLI.active := false
	
	qLI.sql := "select * from LineItems"
	qLI.active := true
	nLI = qLI.rowset.count()
   ? "LineItems Table -- Rows added: "+nLI
	qLI.active := false
	
/*
   // UNCOMMENT THIS CODE ONLY IF YOU ARE *NOT* USING SEQUENCES:
   // If the SEQValue table exists, delete it so we can start over:
	if d.tableExists( "SEQValue" )
	   d.executeSQL( "drop table SeqValue" )
   endif

	// use ADOSeqValue.cc class to set up the autoincrement
	// fields ...
	set procedure to ADOSeqValue.cc
	oSeq = new SeqValue( "BookSample_ADO", "sysdba/masterkey" )
	oSeq.Increment( "Invoices", "InvoiceNumber" )
	oSeq.Reset( "Invoices", "InvoiceNumber", nInvoices )
	oSeq.Increment( "Customers", "CustomerNumber" )
	oSeq.Reset( "Customers", "CustomerNumber", nCustomers )
	oSeq.Increment( "LineItems", "LineItemNumber" )
	oSeq.Reset( "LineItems", "LineItemNumber", nLI )
	oSeq.release()
	close procedure ADOSeqValue.cc
*/

catch( ADOException e )
   
	// the error object itself:
	? "Code:     "+e.code
	? "Filename: "+e.filename
	? "LineNo:   "+e.lineNo
	? "Message:  "+e.message

   local aError, i
	aError = e.errors

	for i = 1 to aError.size
	   ? // blank line
	   ? "message:    "+ aError[i].message
      ? "code:       "+ aError[i].code
		? "source:     "+ aError[i].source
      ? "nativeCode: "+ aError[i].nativeCode
      ? "sqlCode:    "+ aError[i].sqlCode
   next
		
	return
endtry // whole thing wrapped in one big try/catch/endtry 

// Testing done, close and release data objects
qInvoices.active := false
release object qInvoices
qInvoices = null

qCustomers.active := false
release object qCustomers
qCustomers = null

qLI.active := false
release object qLI
qLI = null

qInv.active := false
release object qInv
qInv = null

d.active := false
release object d
d = null

/*
   ************************************************************************
   For this code to work you need to remove the
	triggers from the tables, and where the commented
	code for "nRows++" sits, you would replace it with:
	   nRows = GetNextKey( name_of_Sequence )
		  // for example:
		nRows = GetNextKey( "InvNum_Seq" ) // Invoice table
   ************************************************************************		
*/
function GetNextKey( cGenName )
   private dGen, qGen, nVal, cCmd
	dGen = new ADODatabase()
	dGen.databaseName := "BookSample_ADO_RO"
	dGen.loginString  := "SYSDBA/masterkey"
	dGen.active       := true
	
	qGen = new ADOQuery()
	qGen.database    := dGen
	cCmd = "Select Gen_ID("+cGenName+",1) from MyGen" 
	qGen.sql          = cCmd
	qGen.requestLive := false
	qGen.active      := true
	qGen.rowset.first()
	
	// next key value:
	nVal = qGen.rowset.fields[1].value 
	
   // deactivate:	
	qGen.active := false
	dGen.active := false
return nVal