/*
   ----------------------------------------------------------------------------
   GetReport.prg
   Sample code from The dBASE Reports Book, second edition
   Chapter 10
   Copyright  2016, Kenneth J. Mayer
   
   Program code to obtain a date range (using the GetDates.wfm form
   in this folder), set up an associative array used to pass parameters
   to a report, and using the Preview.wfm form in the dUFLP.
   
   This may be used for other purposes, as needed. 
   ----------------------------------------------------------------------------
*/
private oForm, dStart, dEnd, cFilter

// open the dialog:
set procedure to GetDates.wfm
oForm = new GetDatesForm()

// check the value returned by the form:
if oForm.readModal()
   // when the user closes the form:
   dStart = oForm.dStart
   dEnd = oForm.dEnd
   // create the filter string:
   cFilter = ""
   // if we have a start date:
   if not empty( dStart )
      cFilter := [Born =>']+dStart+[']
   endif // not empty( dStart )

   // if we have an end date:
   if not empty( dEnd )
      // if we already have something in the filter:
      if not empty( cFilter )
         cFilter += [ and ]
      endif // not empty( cFilter )
      cFilter += [Born <= ']+dEnd+[']
   endif // not empty( dEnd )

   // create an associative array:
   aParams = new AssocArray()
   // add the filter to it:
   aParams["filter"] = cFilter
   // Next, we need to set up the Preview form:
   set procedure to :dUFLP:Preview.wfm additive
   oPreview = new PreviewForm()
   // set the params array:
   oPreview.params = aParams
   // set the Modal property:
   oPreview.bModal = true
   // set the report filename:
   oPreview.Viewer.fileName := "PreviewSample1.rep"
   // open it:
   oPreview.readModal()
   // cleanup - once the Preview form closes:
   oPreview.release()
   close procedure :dUFLP:Preview.wfm
endif // oForm.readModal()

// clean up memory:
oForm.release()
close procedure GetDates.wfm
