/*
   ----------------------------------------------------------------------------
   Filename.....: OD.prg
   Purpose......: Display debug messages using Microsoft's
                  DebugView program -- to use this you must
                  download the program and install it from
                    https://technet.microsoft.com/en-us/library/bb896647.aspx
                  The program must be running or this will
                  not work.
   Programmer...: Jan Hoelterling
   Date.........: August 13, 2015
   Note.........: Posted in the beta newsgroup, this routine was created
                  to help debug an app, without using msgbox() calls
                  throughout the code -- there are frustrations with
                  msgbox() calls:
                     1) The msgbox() can interfere with the code (especially
                        the interface -- forms and such);
                     2) You have to click through a series of them ...
                  This program (OD.PRG) needs to be in the same folder as
                  your application, or you would need to store it in a 
                  code library, and use a source code alias:
                      set procedure to :MyLibrary:od
                      
                  Additionally, Jan has code that will write the same
                  information to a text file. To do this, you would
                  need to set two custom properties of the _app object:
                  
                      _app.Log2File = true
                      _app.LogFile  = "MyLogfile.txt"
                      
                  This file will be created in the same folder as the
                  executable, and will contain the same messages displayed
                  in the DebugView program.
   Written for..: dBASE Plus
   Dependencies.: DegugView.exe from Microsoft
   Example:.....: OD( "Display this message" )
   ----------------------------------------------------------------------------
*/

Function OD
   parameter instring
   if type("OutputDebugString") # "FP"
      extern cVOID OutputDebugString( CPTR ) kernel32 ;
                  from "OutputDebugStringA"
   endif
   OutputDebugString(ToUC(Instring))
   
   // You can also log to an output file (see header above):
   if type("_app.Log2File") == "L" and _app.Log2File
      Logit(_app.LogFile,Time()+" : "+Instring)
   endif
return

Function ToUC
   parameter c
   LOCAL cTemp, x

   cTemp = Replicate(Chr(0), ((Len(c) + 1) / 2) + ;
                             ((Len(c) + 1) % 2))
   For x = 1 To Len(c)
      cTemp.SetByte(x - 1, Asc(SubStr(c, x)))
   EndFor
RETURN cTemp 

function logit
    parameter cLogFile,cString

   local oLog
   oLog = new file()
   if oLog.Exists(cLogFile)
      oLog.open(cLogFile,"A")
   else
       oLog.create(cLogFile,"W")
   endif

    if empty(cString)
       oLog.WriteLn("")
   else
       oLog.writeln(date()+" "+time()+" "+getenv("Username")+": "+cString)
   endif
    oLog.close()
return 