; -----------------------------------------------------------------------------------------
; Invoice_Manual.iss 
; A sample Inno Setup Script from 
; The dBASE Book Plus by Kenneth J. Mayer. 
; Program copyright 2015. 
; Permission is granted for use as a learning tool for dBASE Plus.
;
; Sample from: Chapter 15
; This Script is designed to deploy an ADO application, both for a Server and 
; for a Client.
;
; Server should get:
;   * Firebird Server *
;   * Install ODBC Drivers for Firebird *
;   * ODBC Connections *
;   * Data file for Firebird database
;   * Application
;   * INI file with ODBC Connections set up
;
;  Client should get:
;   * Install ODBC Drivers for Firebird *
;   * ODBC Connections *
;   * Application
;   * INI file with ODBC Connections set up
;
; However, the items marked with a * at the end are ones that we cannot
; automate, so we have instructions instead.
; -----------------------------------------------------------------------------------------

[Setup]
SourceDir=C:\Kens dBASE Book Plus\Plus 1e Source Code\Chapter 15\ADO Example
SetupIconFile=C:\Kens dBASE Book Plus\Plus 1e Source Code\Chapter 15\ADO Example\form_green.ico
AppId=Invoice
AppName=ADO Invoice Example
AppVersion=1.00
AppVerName=ADO Invoice Example Version 1.00
AppCopyright=Copyright (c) 2015 Ken Mayer
DefaultDirName={pf}\Invoice
DefaultGroupName=Invoice
PrivilegesRequired=admin
ShowLanguageDialog=false
; Display this file after the install is
; complete:
InfoAfterFile=AfterInstall.txt

; Define types as Server and Client:
[Types]
Name: Server; Description: Server Install -- Data and program files on network drive
Name: Client; Description: Client Install -- Local workstation, installs program only, sets up references for data, ODBC, etc.

; Components:
[Components]
Name: Server; Types: Server; Description: Server Install
Name: Client; Types: Client; Description: Client/Workstation Install

[Dirs]
Name: {app}\Data; Components: Server; Flags: UninsAlwaysUninstall
Name: {app}\Documentation; Flags: UninsAlwaysUninstall

[Files]
; Images/Icons
Source: about.ico; DestDir: {app}; Flags: IgnoreVersion
Source: gear.png; DestDir: {app}; Flags: IgnoreVersion
Source: node.png; DestDir: {app}; Flags: IgnoreVersion
Source: form_green.ico; DestDir: {app}; Flags: IgnoreVersion

; Executable -- compiled with NO BDE and EMBED RUNTIME flags:
Source: Invoice.exe; DestDir: {app}; Flags: IgnoreVersion
; dBASE Runtime files:
Source: C:\Program Files (x86)\dBASE\Plus10\Runtime\PlusRun.exe.dll; DestDir: {app}; Flags: IgnoreVersion
Source: C:\Program Files (x86)\dBASE\Plus10\Runtime\PlusR_EN.dll; DestDir: {app}; Flags: IgnoreVersion
Source: C:\Program Files (x86)\dBASE\Plus10\Runtime\plusrun.exe.manifest; DestDir: {app}; Flags: IgnoreVersion

; Documentation:
Source: AfterInstall.txt; DestDir: {app}; Flags: IgnoreVersion
Source: Documentation\Firebird ODBC for Invoice.pdf; DestDir: {app}\Documentation; Components: Server; Flags: IgnoreVersion
Source: Documentation\Firebird Server for Invoice.pdf; DestDir: {app}\Documentation; Flags: IgnoreVersion

; Database
Source: C:\Kens dBASE Book Plus\Data\BOOKSAMPLE.FDB; DestDir: {app}\Data; Components: Server; Flags: IgnoreVersion

[INI]
; "Standard" entries in the ini file:
Filename: {app}\Invoice.ini; Section: Toolbars; Flags: UninsDeleteSection
Filename: {app}\Invoice.ini; Section: Toolbars; Key: Standard; String: 0
Filename: {app}\Invoice.ini; Section: Toolbars; Key: StandardFloat; String: 0
Filename: {app}\Invoice.ini; Section: Toolbars; Key: StandardStyle; String: 0
Filename: {app}\Invoice.ini; Section: Desktop; Flags: UninsDeleteSection
Filename: {app}\Invoice.ini; Section: Desktop; Key: Maximized; String: 0
Filename: {app}\Invoice.ini; Section: Desktop; Key: StatusBar; String: 0
Filename: {app}\Invoice.ini; Section: CommandWindow; Flags: UninsDeleteSection
Filename: {app}\Invoice.ini; Section: CommandWindow; Key: Open; String: 0
Filename: {app}\Invoice.ini; Section: DataEngine; Flags: UninsDeleteSection
Filename: {app}\Invoice.ini; Section: DataEngine; Key: DefaultEngine; String: None
Filename: {app}\Invoice.ini; Section: AppTheme; Flags: UninsDeleteSection
Filename: {app}\Invoice.ini; Section: AppTheme; Key: Type; String: 0
Filename: {app}\Invoice.ini; Section: Tools; Flags: UninsDeleteSection
Filename: {app}\Invoice.ini; Section: Tools; Key: LargeButtons; String: 1

; Add other keys needed, including the ADO Connection Strings (without password)
Filename: {app}\Invoice.ini; Section: Connections; Flags: UninsDeleteSection
Filename: {app}\Invoice.ini; Section: Connections; Key: 0; String: BookSample_ADO
Filename: {app}\Invoice.ini; Section: Connections; Key: 1; String: BookSample_ADO_RO
Filename: {app}\Invoice.ini; Section: BookSample_ADO; Flags: UninsDeleteSection
Filename: {app}\Invoice.ini; Section: BookSample_ADO; Key: Provider; String: MSDASQL.1
Filename: {app}\Invoice.ini; Section: BookSample_ADO; Key: Persist Security Info; String: False
Filename: {app}\Invoice.ini; Section: BookSample_ADO; Key: Data Source; String: BookSample
Filename: {app}\Invoice.ini; Section: BookSample_ADO; Key: Mode; String: ReadWrite
Filename: {app}\Invoice.ini; Section: BookSample_ADO_RO; Flags: UninsDeleteSection
Filename: {app}\Invoice.ini; Section: BookSample_ADO_RO; Key: Provider; String: MSDASQL.1
Filename: {app}\Invoice.ini; Section: BookSample_ADO_RO; Key: Persist Security Info; String: False
Filename: {app}\Invoice.ini; Section: BookSample_ADO_RO; Key: Data Source; String: BookSample_RO
Filename: {app}\Invoice.ini; Section: BookSample_ADO_RO; Key: Mode; String: Read

[UninstallDelete]
Type: files; Name: Filename: {app}\Invoice.ini
 
; ---------------------------------------------------------------------------------------------
; Code to set up/create pages so we can get the path to the network where the
; application is/should be installed ... This code is based on code provided ages ago for 
; Ken Mayer by Jonny Kwekkeboom for another application.
[Code]
var 
   NetDirPage: TInputDirWizardPage; 
   cNetDirDefault: String;

procedure InitializeWizard_NetDir;
var cTitle, cLabel, cDesc: String;
begin
  { Create the pages }

  cNetDirDefault := '{app}';
  cTitle         := 'Select Directory on Network Drive';
  cLabel         := 'Choose directory on the Network Drive where application files are installed.';
  cDesc          := 'Choose the folder on the network drive that has the data, most likely F:\Invoice';
  NetDirPage     := CreateInputDirPage( wpSelectDir, cTitle, cLabel, cDesc, False, '' );
  NetDirPage.add( '' );
  NetDirPage.Values[0] := '';
end;

function GetNetDir( Param: String): String;
begin
   Result := NetDirPage.Values[0];
end;

procedure CurPageChanged( CurPageID: Integer);
begin
   case CurPageID of
        NetDirPage.ID: 
        begin
             NetDirPage.Values[0];
             ExpandConstant( cNetDirDefault );
             if NetDirPage.Values[0] = '' then
                NetDirPage.Values[0] := 
                   ExpandConstant( cNetDirDefault );
             end;
        end;
 end;

 procedure InitializeWizard();
 begin
    InitializeWizard_NetDir;
 end;
