/*
   ------------------------------------------------------------
   Program...: InvoiceApp.cc
   Programmer: Ken Mayer
   Date......: June, 2015

   Invoice Application Class for Sample Application 
	"Invoice" for
	             The dBASE Book Plus
					 Chapters 4-6

   Copied from other sources and split up to appropriate
	code ...
	  
   Copyright 2015
   Ken Mayer
	Permission is granted for use for your own applications
	as long as credit is given where due.
   ------------------------------------------------------------ 
*/
#DEFINE APPVERSION "1.00"
class InvoiceApp 

   do setup // any setup necessary

   // assign the instance of this class
   // as an object reference off the _app.Framewin object:
   _app.FrameWin.App = this 

   // set the menu:
   // modification from Jim Sare:
   new InvoiceMenu( _app.framewin, "Root" )
   _app.framewin.root.release()
   new InvoiceMenu( _app.framewin, "Root" )

   // this function handles opening the application:
   function open
      // disallow DEO:
      _allowDEOExeOverride = false

      // deal with semaphore for application -- disallow
      // running it twice ...
      set procedure to :dUFLP:appflag.cc additive
      _app.Flag = new AppFlag()
      if not _app.Flag.open( "OandA" )
         if "RUNTIME" $ upper(version(0))
            quit
         endif
         RETURN
      endif

      /* 
         disable the 'x' for _app ... otherwise
         selecting that will not run the shutdown code ...
         (_app.framewin does not have a canClose, etc.)
      */
      sysClose( _app.Framewin, false ) // disable

      // set the ability for other products to still function
      // while this one is processing, and for the application
      // to be able to communicate better (update message 
      // forms, etc.):
      _app.allowYieldOnMsg := true

      /*
          Deal with initialization file:
      */
      set procedure to :duflp:ini.cc additive
      try
         if not "runtime" $ lower( version(1) )
			   cAppIniPath = set("directory")+"\Invoice.Ini"
            _app.Ini = new ini( set("directory")+"\Invoice.Ini")
         else // runtime, meaning the executable:
			   cAppIniPath = _app.IniFile
            _app.Ini = new ini( _app.IniFile )
			endif
//msgbox( "INI File: "+cAppIniPath )
      catch( exception E )
         msgbox( "Error: "+e.code+" -- "+e.message+chr(13)+;
                  e.fileName+" -- Line: "+e.lineNo, "Invoice Error", 16 )
      endtry

      // this event should be being fired when the 
      // menu is initialized, but it's not. So, 
      // we force it here ...
      _app.framewin.root.onInitMenu()

      // set Shell
      _app.Version          = APPVERSION // set here
      _app.Speedbar         := false // turn it off
      _app.StatusBar        := false // same
      #IF __version__ => 9
         _app.tabBar         := false // same -- dBASE Plus 9.5	
		#ENDIF
      _app.FrameWin.OldText  = _app.FrameWin.Text  // save old text
      _app.FrameWin.text    := "Ken's Widgets and Gadgets, LLC - Invoices -- "+;
                              "Version "+_app.Version

      // deal with size/position on desktop:
      // use a special class for this in the dUFLP:
      set proc to :dUFLP:AppFrameSize.cc additive
      oFrameWin = new appFrameSize()
      // path to .ini file ...?
      if not "runtime" $ lower( version( 1 ) )
         oFrameWin.userINI := set("dire")+"\Invoice.ini"
      else
         oFrameWin.userINI := _app.iniFile
      endif
      oFrameWin.readIni()

      shell( false, true )
      // if this is here, in the compiled app, then it works,
      // however, if it is taken out and we rely on the setting
      // above, it gets set back to true ... (known bug)
      //_app.StatusBar := false

   return

   // shutdown
   function close
      close forms
      cSection = "Desktop"
      cEntry   = "Maximized"
      _app.Ini.setValue( cSection, cEntry, str( _app.Framewin.WindowState ) )

      // save to INI file:
      _app.frameSize.save()
      _app.frameSize.release()

      // reset _app values
      with (_app)
         // reset ...:
         framewin.text := framewin.OldText
         speedbar      := true
         statusBar     := true
         #IF __version__ => 9			
            tabBar	  := true // 9.5
			#ENDIF
         app           := null
      endwith

      // close the semaphore ...
      _app.Flag.Close()

      // reset:
      _app.allowYieldOnMsg := false

      shell( true, true )
      // suggested by Jim Sare
      _app.framewin.root.release()

      // issue 'quit' only if we're in RUNTIME, otherwise
      // testing gets really annoying ...
      // If you don't issue the 'quit', then the shell
      // gets left on screen, which is weird, to say the least.
      if "RUNTIME" $ upper( version(1) )
         quit
      else
         sysClose( _app.Framewin, true ) // enable close button
      endif
   return
endclass

/* 
   ---------------------------------------------------------------
   Enables or disables the "close" item on the system menu as well 
   as the Windows X in the titlebar.

      Syntax:    sysClose( oForm, lEnabled )
      Example:   sysClose( _app.FrameWin, false ) // disable
                 sysClose( _app.FrameWin, true )  // ensable

   Posted by Gary White in the VdBASE newsgroups
   7/3/1998

   Returns the previous state of the menu item.
   ---------------------------------------------------------------
*/
function sysClose
parameters oForm, lEnabled
local hMenu, lRetVal
#define MF_ENABLED      0x00000000
#define MF_GRAYED       0x00000001
#define SC_CLOSE        0xF060

   if argCount() # 2 or ;
         type("oForm.hWnd") # "N" ;
         or type("lEnabled") # "L"
      msgbox( "Correct syntax is:  sysClose( oForm, lEnabled )", ;
         "Error", 16 )
      return
   endif
   if empty( oForm.hWnd )
      msgbox( "Form is not open", "Error", 16 )
      return
   endif

   if type( "GetSystemMenu" ) # "FP"
     extern cint GetSystemMenu( cHandle, cInt ) USER32
   endif
   if type( "EnableMenuItem" ) # "FP"
      extern CLOGICAL EnableMenuItem( cHandle, cInt, cInt) user32
   endif
	hMenu = GetSystemMenu( oForm.hWnd, 0 )
   if lEnabled
   	lRetVal = EnableMenuItem( hMenu, SC_CLOSE, MF_ENABLED )
   else
   	lRetVal = EnableMenuItem( hMenu, SC_CLOSE, MF_GRAYED )
	endif
return not lRetVal

