/*
   ----------------------------------------------------------------------------
   UILayer.CFM
   A sample dBL program from 
   The dBASE Book Plus by Kenneth J. Mayer. 
   Program copyright 2015. 
   Permission is granted for use as a learning tool for dBASE Plus.
   Sample from: Chapters 5 & 6
        
   A custom form designed to be the base form for the sample Invoice
        application.
        
   Relies on:
            DataLayer.dmd -- the Data Module containing both the read and write
                                  connections to the data, and the specific 
                                  read-only and write-only queries needed 
                                  to communicate with the data.
            SQLFormControls.cc -- a set of custom classes designed specifically
                                 to be flexible and work with the data. 
            SQLCustomButtons.cc -- custom pushbuttons designed to deal with
                                                     standard table navigation, etc.
   ----------------------------------------------------------------------------
*/
** END HEADER -- do not remove this line
class UILayerCForm of FORM custom
   set procedure to mseekerado.cc additive
   set procedure to SQLFormControls.cc additive
   set procedure to SQLCustomButtons.cc additive
   with (this)
      onNavigate = class::FORM_ONNAVIGATE
      open = class::FORM_OPEN
      readModal = class::FORM_READMODAL
      saveRecord = class::SAVERECORD
      abandonRecord = class::ABANDONRECORD
      metric = 6	// Pixels
      height = 290.0
      left = 169.0
      top = 7.0
      width = 596.0
      text = "UI Layer - Custom Form"
   endwith

   this.TITLETEXT = new TEXT(this)
   with (this.TITLETEXT)
      height = 34.0
      left = 0.0
      top = 0.0
      width = 594.0
      anchor = 2	// Top
      colorNormal = "Blue/BtnFace"
      alignVertical = 1	// Middle
      alignHorizontal = 1	// Center
      fontSize = 16.0
      fontBold = true
      text = "UI Layer - Custom Form"
      borderStyle = 1	// Raised
      pageno = 0
   endwith

   this.TOPBUTTON1 = new TOPBUTTON(this)
   with (this.TOPBUTTON1)
      height = 24.0
      left = 0.0
      top = 34.0
      width = 64.0
   endwith

   this.PREVIOUSBUTTON1 = new PREVIOUSBUTTON(this)
   with (this.PREVIOUSBUTTON1)
      height = 24.0
      left = 65.0
      top = 34.0
      width = 64.0
   endwith

   this.NEXTBUTTON1 = new NEXTBUTTON(this)
   with (this.NEXTBUTTON1)
      height = 24.0
      left = 130.0
      top = 34.0
      width = 64.0
   endwith

   this.BOTTOMBUTTON1 = new BOTTOMBUTTON(this)
   with (this.BOTTOMBUTTON1)
      height = 24.0
      left = 195.0
      top = 34.0
      width = 64.0
   endwith

   this.APPENDBUTTON1 = new APPENDBUTTON(this)
   with (this.APPENDBUTTON1)
      height = 24.0
      left = 265.0
      top = 34.0
      width = 64.0
   endwith

   this.SAVEBUTTON1 = new SAVEBUTTON(this)
   with (this.SAVEBUTTON1)
      height = 24.0
      left = 329.0
      top = 34.0
      width = 64.0
   endwith

   this.ABANDONBUTTON1 = new ABANDONBUTTON(this)
   with (this.ABANDONBUTTON1)
      height = 24.0
      left = 393.0
      top = 34.0
      width = 64.0
   endwith

   this.DELETEBUTTON1 = new DELETEBUTTON(this)
   with (this.DELETEBUTTON1)
      height = 24.0
      left = 457.0
      top = 34.0
      width = 64.0
   endwith

   this.CLOSEBUTTON1 = new CLOSEBUTTON(this)
   with (this.CLOSEBUTTON1)
      height = 24.0
      left = 528.0
      top = 34.0
      width = 64.0
   endwith

   this.SQLTEXTLABEL1 = new SQLTEXTLABEL(this)
   with (this.SQLTEXTLABEL1)
      height = 22.0
      left = 66.0
      top = 41.0
      width = 125.0
      text = "Search for XXX:"
      pageno = 2
   endwith

   this.MSEEKERADO1 = new MSEEKERADO(this)
   with (this.MSEEKERADO1)
      height = 22.0
      left = 199.0
      top = 41.0
      width = 253.0
      pageno = 2
   endwith

   this.TABBOX1 = new TABBOX(this)
   with (this.TABBOX1)
      onSelChange = class::TABBOX1_ONSELCHANGE
      id = 113
      height = 22.0
      left = 0.0
      top = 266.0
      width = 594.0
      colorNormal = "Blue"
      colorHighLight = "Blue/BtnFace"
      dataSource = 'ARRAY {"Detail","Find Record"}'
   endwith

   this.SEEKERGRID = new GRID(this)
   with (this.SEEKERGRID)
      bgColor = "White"
      cellHeight = 22.0
      rowSelect = true
      colorRowSelect = "WindowText/0x80ffff"
      allowRowSizing = false
      allowColumnSizing = false
      allowColumnMoving = false
      allowEditing = false
      allowAddRows = false
      vScrollBar = 1	// On
      pageno = 2
      height = 190.0
      left = 8.0
      top = 71.0
      width = 582.0
   endwith


   function form_open()
      class::init()
   return UILAYERCFORM::open()
   
	function form_readModal()
      class::init()      
   return UILAYERCFORM::readModal()
   
   function TABBOX1_onSelChange()
      // Tabbox changes the page number on the form:
		form.pageNo := this.curSel
		if this.curSel == 1
		   if type( "form.FirstControl" ) == "O" // object
			   form.FirstControl.setFocus()
			endif
		else
		   form.rowset.first()
			form.MSeekerADO1.setFocus()
		endif
   return
	
   function form_onNavigate(nWorkArea)
	   // we have navigated in the table:
      class::LoadData()
   return
	
	function init
	   // code that executes before the form opens
		// set some custom properties:
		form.append   = false
		form.modified = false

   	// make sure Tabbox is on the correct
		// page:
		form.pageNo         := 1
		form.Tabbox1.curSel := 1
		
		// first thing we need is to set up the DataModule:
		set procedure to DataLayer.dmd
		form.DataLayerDataModule1 = new DataLayerDataModule()

      // Load the data -- it is being called from its own
		// method, because we will need it again when we
		// navigate, etc.:
		class::LoadData()
	return
	
   function LoadData
	   // For this to work, we need specific information.
		// There needs to be a custom property of the form
		// that contains the primary key for the table
		// if it doesn't exist, we must assume the first
		// row of the table, which is not necessarily a value 
		// of 1. We are also assuming an array has been created
		// for the form that has object references to the various
		// controls that are to be "linked" to the table.
		local nControls, nFields, cTable, oQuery, bOK
		
		// if the array is undefined, there is nothing
		// for us to do:
		if type( "form.aSQLControls" ) == "U"
		   msgbox( "Cannot load data, form is missing aSQLControls array.",;
			        "LoadData Error", 16 )
		   return
		endif
		
		// Need to loop through the array:
		for nControls = 1 to form.aSQLControls.size
			// We need to look at the first control to see what
			// the tableName is:
			cTable = form.aSQLControls[ nControls ].tableName
			// if the developer has made an error, rather than
			// crashing the software, move on to the next 
			// control:
			if empty( cTable ) or ;
			   empty( form.aSQLControls[ nControls ].fieldName )
			   loop
			endif
			
         // call FindQuery method with the name of the table
			oQuery = class::FindQuery( cTable )
			if oQuery == null
				return // dealt with errors in FindQuery method
         endif
			
			// if we found this, we have the table/query, let's try
			// to find the field
			bOK = false

         for nFields = 1 to oQuery.rowset.fields.size
			
			    if oQuery.rowset.fields[ nFields ].fieldName.toUpperCase() == ;
                form.aSQLControls[ nControls ].fieldName.toUpperCase()

                // found it:
					 bOK := true
					 
					 // load the value property from the table:
                form.aSQLControls[ nControls ].value := ;
                    oQuery.rowset.fields[ nFields ].value
             endif // attempt to find field
				 
             // not found, assign null value ...				 
			    if not bOK
				    form.aSQLControls[ nControls ].value := null
				 endif
				 
				 // set some custom properties:
				 form.aSQLControls[ nControls ].newValue := null
				 form.aSQLControls[ nControls ].oldValue := ;
				    form.aSQLControls[ nControls ].value
				 form.aSQLControls[ nControls ].modified := false
				 form.aSQLControls[ nControls ].error    := false
				// set colors (constants in SQLFormControls do not carry over):
				form.aSQLControls[ nControls ].colorNormal    := "WindowText/Window"
				form.aSQLControls[ nControls ].colorHighlight := "WindowText/0x80ffff"
			next // nFields
		next // nControls
		
      // done loading the data, set custom form properties
      //	to false:
		form.append   := false
      form.modified := false
	return
	// End of method: LoadData
	
	function FindQuery( cTable )
	   // Method to find a table in the Data Module
		// returns either 'false' if not found, or
		// object reference for query
	
	   // if the data module does not have the custom
		// array needed, we also can't do anything:
		if type( "form.DataLayerDataModule1.aROQuery" ) # "A"
		   msgbox( "Cannot load data, Data Module is missing aROQuery array.",;
			        "FindQuery Error", 16 )
         return false
		endif
		
		private aQueries, qQuery
      qQuery = null
		
		// check array to see if tablename matches
		// the name property of the query:
      aQueries = form.DataLayerDataModule1.aROQuery
      for nQuery = 1 to aQueries.size
         if aQueries[ nQuery ].name.toUpperCase() ==;
				    cTable.rightTrim().toUpperCase()
				 qQuery = aQueries[ nQuery ]
			 endif
		next
		
	   if qQuery == null
		   msgbox( "Table '"+cTable+"': No query "+;
			        "found using this tablename ...", ;
					  "FindQuery Error", 16 )
		endif	

      // We have what we need, return it to the calling point:
	return qQuery
	// End of method: FindQuery
	
	function SaveRecord
	   /* 
		   This method will:
		   a) check for the form's append property, if it does not
		     exist, create it;
		   b) if true, we will call the AddRow method of the
		      Data Module, otherwise we are updating/saving changes
		      to an existing record, we will call the UpdateRow method,
      */				
		if type( "form.append" ) == "U"
		   form.append = false
		endif

		// need an array (and other variables) for this:
		private aSave, nSave, nControls, cTable, oQuery
		local cPrimary, nPrimary
		aSave = new array(1,2)
		nSave = 0
	
		// if we are appending:
		if form.append

         // Before we load the array, we need to ensure that
			// we have the primary key information ... and we
			// will need it to return to this row once saved:
			nSave = 1
			// Primary Key should always be first field in table:
			cPrimary = form.rowset.fields[1].fieldName
			aSave[ 1, 1] = cPrimary
  		   cSeq = form.rowset.Sequence
			aSave[ 1, 2 ] = GetNextKey( cSeq )
			nPrimary = aSave[ 1, 2 ]

 			// need table name:
			cTable = form.aSQLControls[ 1 ].tableName			

		   // load array from form.aSQLControls array:
		   for nControls = 1 to form.aSQLControls.size
			    // don't process this one
			    if not form.aSQLControls[ nControls ].includeInQuery
				    loop
				 endif
				 
      	    nSave++
		       aSave.grow( 1 ) // add row to array
             // add to array:
				 aSave[ nSave, 1 ] = form.aSQLControls[ nControls ].fieldName
			    aSave[ nSave, 2 ] = form.aSQLControls[ nControls ].value
			next // nControls
			// call code to actually save this row
		   form.DataLayerDataModule1.AddRow( cTable, aSave )
	   else // saving a changed record:
		   // get name of table
         cTable   = form.aSQLControls[ 1 ].tableName
			// load array from form.aSQLControls array:
		   for nControls = 1 to form.aSQLControls.size
			    if not form.aSQLControls[ nControls ].includeInQuery
				    loop
				 endif
			    // Add only fields that have been modified (which should
				 // avoid the primary key)
			    if form.aSQLControls[ nControls ].modified
				    nSave++
				    if nSave > 1
				       aSave.grow( 1 ) // add row to array
					 endif // nSave > 1
					 aSave[ nSave, 1 ] = form.aSQLControls[ nControls ].fieldName
					 aSave[ nSave, 2 ] = form.aSQLControls[ nControls ].value
				endif // readOnly
			next // nControls
			// deal with empty array:
			if aSave.size/2 == 1
			   if aSave[1,1] == false
				   form.modified := false
				   return
				endif // false
			endif // size
				
			// we also need the primary key name and value:
			local cPrimary, nPrimary
			cPrimary = form.rowset.fields[1].fieldName
			nPrimary = form.rowset.fields[1].value
			// and pass the data to the Data Module:
		   form.DataLayerDataModule1.UpdateRow( cTable, cPrimary, nPrimary, aSave )
		endif // form.append

		// refresh the query display updated version 
		// of record in form.
      oQuery = class::FindQuery( cTable )
		oQuery.requery()
      // navigate to the row just edited/appended:
      cFind = cPrimary+"="+nPrimary
		try
		   oQuery.rowset.applyLocate( cFind )
		catch( ADOexception e )
		   ? e.code, e.message
			// Navigate to first row
			oQuery.rowset.first()
      endtry

		// load the data, and away we go
		class::LoadData()
		class::ResetControls()
	return
	// End of method: SaveRecord
	
	function AbandonRecord
	   /*
		   This method will abandon changes to an existing record on the form,
			or if the user was adding a new record, abandon the changes
			and go back to the previous record.
		*/
		if type( "form.append" ) == "U"
		   form.append = false
		endif

      if form.append
		   if not form.rowset.next(-1) // previous record -- if we error
			   form.rowset.first()      // first record
				// form_onNavigate event handler calls LoadData method
			endif // navigate 
      else
   		// If here, user was not appending, but editing, re-load the
			// current row's data from the table:
	   	class::LoadData()
		endif // form.append

      // reset appearance of controls and some of the 
		// custom properties:
		class::ResetControls()
	return
	// End of method: AbandonRecord
	
	function AppendRecord
      /*
		   This will act as a record buffer, all we really need to
			do is clear out the controls:
		*/
		// if the array is undefined, there is nothing
		// for us to do:
		if type( "form.aSQLControls" ) == "U"
		   msgbox( "Cannot check data types, form is missing aSQLControls array.",;
			        "AppendRecord Error", 16 )
		   return
		endif

		private nControls
		for nControls = 1 to form.aSQLControls.size
			 // assign properties
          form.aSQLControls[ nControls ].value    := ;
			    form.aSQLControls[ nControls ].defaultValue
			 form.aSQLControls[ nControls ].oldValue := ;
			    form.aSQLControls[ nControls ].defaultValue
			 form.aSQLControls[ nControls ].newValue := null
			 form.aSQLControls[ nControls ].modified := false
			 form.aSQLControls[ nControls ].error    := false
			 // set colors (constants in SQLFormControls do not carry over):
			 form.aSQLControls[ nControls ].colorNormal    := "WindowText/Window"
			 form.aSQLControls[ nControls ].colorHighlight := "WindowText/0x80ffff"
		next // nControls

		// set the form's append property
		form.append   := true
		// reset the form's modified property (just to be sure)
		form.modified := false
	return
	// End of method: AppendRecord
	
	function DeleteRecord( bCascade, cChildTable, cForeignField )
	   /*
		   In a way this is the more complicated method of some of these,
			because we need to determine if we're dealing with a cascade
			delete ... and if we are, we have to pass more information
			to the Data Module's code than we would if we were *not*
			doing the cascade delete.
		*/
      local cTable, cPrimary, nPrimary
		// we must assume that the first field in the
		// table is the primary key -- it makes it easier ...
		cTable   = form.aSQLControls[ 1 ].tableName
		cPrimary = form.rowset.fields[1].fieldName
		nPrimary = form.rowset.fields[1].value

		// get primary key for row we want to display AFTER the delete ...
		// normally we back up one row, so if we were on the fifth row
		// we would, after deleting, display the fourth. But if we're 
		// deleting the FIRST row ... or the table is empty ...
		private oQuery, nNewPrimary
		oQuery = class::FindQuery( cTable )
		if oQuery.rowset.atFirst() // at first row
			oQuery.rowset.next() // go to row AFTER current one
		else	
			oQuery.rowset.next( -1 ) // otherwise go to row BEFORE this one
		endif	
		nNewPrimary = oQuery.rowset.fields[ 1 ].value

      // and pass the data to the Data Module:
		if not bCascade
		   form.DataLayerDataModule1.DeleteRow( cTable, cPrimary, nPrimary )
		else
		   if empty( cChildTable ) or empty( cForeignField )
			   msgbox( "Cannot perform cascade delete, check values of parameters.",;
			        "DeleteRecord Error", 16 )
				return
			else
			   form.DataLayerDataModule1.DeleteRow( cTable, cPrimary, nPrimary, ;
                                                 bCascade, cChildTable, ;
																 cForeignField )
			endif // empty( ... )
		endif // not bCascade
		
		// need to refresh the query, and deal with which row
		// to display ... 
		oQuery.requery()
		if oQuery.rowset.count() > 0
		   cFind = cPrimary+"="+nNewPrimary
			try
				oQuery.rowset.applyLocate( cFind )
			catch( ADOexception e )
				? e.code, e.message
			endtry
         // load the data, and away we go
		   class::LoadData()
		endif
		class::ResetControls()		
	return
   // End of method: DeleteRecord
	
	function ResetControls
		// we need to loop through the controls 
		// and reset specific properties:
		for nControls = 1 to form.aSQLControls.size
		   if not form.aSQLControls[ nControls ].readOnly
				form.aSQLControls[ nControls ].modified := false
				form.aSQLControls[ nControls ].error    := false
				form.aSQLControls[ nControls ].newValue := ;
					 form.aSQLControls[ nControls ].value
				form.aSQLControls[ nControls ].oldValue := ;
					 form.aSQLControls[ nControls ].value
				// set colors (constants in SQLFormControls do not carry over):
				form.aSQLControls[ nControls ].colorNormal    := "WindowText/Window"
				form.aSQLControls[ nControls ].colorHighlight := "WindowText/0x80ffff"
			endif // not readonly
		next // nControls
	return
	// End of method: ResetControls

endclass

function GetNextKey( cGenName )
   // This function accepts the name of a
   // Sequence (or Generator), increments
   // the value, and returns it, so that
   // you can use it when saving a new
   // record, also reference the value
   // as needed. Original code by
   // Mervyn Bick, 2015
   private dGen, qGen, nVal, cCmd
   dGen = new ADODatabase()
   dGen.databaseName := "BookSample_ADO_RO"
   dGen.loginString  := "SYSDBA/masterkey"
   dGen.active       := true
        
   qGen = new ADOQuery()
   qGen.database    := dGen
   cCmd = "Select Gen_ID("+cGenName+",1) from MyGen" 
   qGen.sql          = cCmd
   qGen.requestLive := false
   qGen.active      := true
   qGen.rowset.first()
        
   // next key value:
   nVal = qGen.rowset.fields[1].value 
        
   // deactivate:        
   qGen.active := false
   dGen.active := false
return nVal
