/****************************************************************************

   Programmers....: Marc Van den Berghen, Roland Wingerter, Rich Assaf,
	                 Tom Gleaton, Mervyn Bick, Ken Mayer

   Version........: 0.8 (2015/06/09)

   Notes..........: This seeker extends basic Seeker/zSeeker functionality

                    Additional features:
                    - Up/Down arrow keys can be used to navigate in the rowset
                    - PgUp and PgDn will skip by user-defined nSkipRecords
                    - Home and End keys go first() and last() respectively
                    - PgUp/PgDn/Home/End can be ignored by the paintbox
                    - seeker can optionally react to the Enter key 
                      (just assign your own method).
                    - optionally execute code if seeker doesn't find a match
                    -  Features from zSeeker (By Peter Rorlick, Feb 1998)
                     *  Does a SEEK on Backspace or Del
                     *  Yellow background when in focus.
                     *  Choice to enable/disable the exact match functionality 
							
							** Addition of mNumSeekerADO below, this class allows
							   using the control to search on numeric values.

   Written for ...: dBase 32bit ( dbase 8.3 --> )
     
   History.......:  0.1 (Jan 12, 2007) 
   					  Marc VdB posted Seekerform.wfm in dbase.programming 

                    0.2 (Jan 12, 2007)
                    R. Wingerter made cc form Marc's code 

                    0.3 (Jan 13, 2007) 
						  New custom property onEnterAction replaces 
                    properties bSelectOnEnter and cFieldName.

                    0.5 (Feb 24, 2007) - Rich Assaf
                    Extended paintbox to respond to Home/End/PgUp/PgDn.
                    Eliminated dependencies on zSeeker and Seeker
                    Added NoMatch functionality if the seek fails
                    Corrected typo:  re_estate() to re_state()
						  
	*** Change Notes **	  1/26/14  Tom Gleaton
                    Created mSeekerADO from mSeeker
                    All changes made by me are in function oSeek
                    Removed use SQL Order By
                    Added use of the ADO rowset's Sort field  property
						  
						  ******************************************************************************
						  12/24/14 Note:
						  Getting Memory error on SOME Computers- See line 271 Below
						  
						  Also, Is it possible to ignore the "enter" key when the seeker field has focus?
						  pressing enter key will cause a "Data Type mismatch error".

                    Added on 2015/05/31.  This problem has been addressed in version 0.7
						  ******************************************************************************
						  
							Marc made the following revision using Callback
							0.6 (Oct 03, 2014) 
						  adopted for use with the new dbase Callback functionality.
						  no need for the messenger paintbox
						  This works only in dbase 8.3 and later

                    ******************************************************************************

                    2015/05/31  Version 0.7  Mervyn Bick  
                    Removed spurious RETURN in function wndproc. See note in wndproc



   Dependencies.:   None

   Usage.......:    This class can be used in the same way as Seeker.cc.
						  In the form designer, make sure that the rowset you wish
						  to use this control with has the indexName property set to
						  an index tag that is character, and created with the UPPER()
						  function, so that it is indexed on the upper-case of any
						  characters in the field(s) index on.

						  Drag the SEEKER control to the design surface. When the form
						  is run, entering text in the entryfield will cause an
						  incremental search to occur, using the index tag mentioned
						  above.              	
						  NOTE: do not attempt to datalink this custom entryfield, 
						  first, it is "protected" so that you cannot access this
						  property in the designer, and second, the code will not function
						  properly if datalinked to a field in the table.

                    If you want the control to react to the Enter key,
                    simply assign a method to this.onEnterAction in 
                    an overridden onOpen event. (cf. sample form).  



**************************************************************************/

CLASS mSeekerADO( oContainer ) of entryfield( oContainer ) custom

   ** don't datalink the entryfield
	protect datalink 

   ** Stock Properties
   this.BorderStyle 		:= 7
   this.ColorHighlight 	:= 'N/RG+'  
   this.ColorNormal		:= 'N/W+'
   this.FontSize 			:= 10

   ** Custom Properties
   this.onEnterAction 	= NULL  // Method to be executed when Enter is pressed
   this.NoMatch         = NULL  // Code to be executed if value not found (see onLostFocus)
	this.nSkipRecords    = 5    // For use with PgUp/PgDn - set low for demo
   this.lEnablePgUpDn   = true
   this.lEnableHomeEnd  = true
   this.lRestrictedSeek = false  // set true for zSeeker behavior
   this.rowset				= Null   // default is form.rowset

	

   protect seekRowset

   ** private properties
   this.lCUAEnter = ( SET( "CUAENTER"  ) == "ON" )
   this.lFound    = true

	function onOpen

   	** If no Rowset is specified, look for a form.rowset,
      ** else keep the explicitly designated rowset.

      if this.rowset == Null
      	if type("form.rowset") = "O"
         	this.rowset = form.rowset
         endif
      endif

		** Initialize the Callback-Functon
		#define GWL_WNDPROC   (-4)
   	// we need three API-functions

      extern CLONG GetWindowLongA(CHANDLE, CINT) user32
      extern CLONG SetWindowLongA(CHANDLE, CINT, CLONG) user32
      extern CLONG CallWindowProcA(CPTR, CHANDLE, CUINT, CUINT, CUINT) user32

		// first we store the original WP for later use

      this.oldproc = GetWindowLongA(this.hwnd, GWL_WNDPROC)

		//we declare a Callback-function, that has the same form as a WP

      CALLBACK CLONG wndproc(CHANDLE, CUINT, CUINT, CUINT) OBJECT THIS
	
		//we need a pointer to that new WP

      this.newproc = GetCallAddress(this.wndproc)

		//now tell the control to use the new WP

      SetWindowLongA(this.hwnd, GWL_WNDPROC, this.newproc)
      this.lcuae    		= ( SET( "CUAENTER"  ) == "ON" )
      set cuaenter on
   return



   FUNCTION onGotFocus
    if form.rowset == null       // Xbase settings
      this.lNear    = ( SET( "NEAR"  ) == "ON" )
      this.lExact   = ( SET( "EXACT" ) == "ON" )
      this.lRefresh = form.RefreshAlways
      set near on                // To do incremental matching
      set exact off              // To return FOUND() true on substring match
      form.RefreshAlways = true
      this.seek     = this.xSeek // Use method with Xbase SEEK()
    else
      // For OODML, use findKeyNearest(), which always does a
      // partial key match (like EXACT OFF)
      this.seek     = this.oSeek
      // Assign protected object reference to either form's rowset
      // or control's rowset
      this.seekRowset = iif( this.rowset == null, form.rowset, this.rowset )
      // ----------------------------------------------
      // FOR SQL VERSION:
      // save current setting 
      this.oldLocateOptions = this.seekRowset.locateOptions
      // change to 3
      this.seekRowset.locateOptions := 3
      // ----------------------------------------------
    endif
    this.lFound  = true
    this.nMaxPos = 0

   function key( nChar, nPosition )
  	   if nchar = 13
   	 	return false
	   endif

   	if nextkey() # 0
      	return ( nChar # 255 )     
    	endif
      
      if nChar == 255
			if not empty( this.value )
         	this.lFound = this.oseek()
        		if not this.lFound and this.lRestrictedSeek
          		?? chr( 7 )  // beep
          		keyb "{End}" + "{Backspace}"   
        		endif
      	else
         	this.Rowset.first()
      	endif
      	nchar =0
    	else
      	if ((nChar > 31 .AND. nChar < 255) .OR. nChar = 7 .OR. nChar = 8 )  
         	keyboard chr(255)   
      	endif
    	endif
   return nChar

   function onLostFocus
     ** Execute the NoMatch() function if defined and needed
      if type("this.NoMatch") $ "CB|FP" 
      	if NOT this.lFound
            this.lFound := true  // reset
      		this.NoMatch(this.Value)
      	endif
      endif
   return

	function oSeek
   	local lRet, bBookMark, cSORTString
    	bBookMark = this.Rowset.BookMark()

    	 lRet = true

       // 1/26/14  Tom G - Removed check for SQL Order By
       //          Added check for the ADO rowset's Sort field  property
		 cSORTString = this.rowset.sort
		 if empty(cSORTString)
          msgbox("You Must set the 'Sort' property for the ADO rowset.","Information: mSeekerADO.cc",64)
          lRet = false
       endif
       if lRet
          lRet = this.seekRowset.ApplyLocate( cSORTString+" Like '"+this.normalizedValue()+"%'")
       endif
       if ( this.seekRowset.endOfSet )
         this.seekRowset.next( -1 )
       endif
    return lRet

   function wndproc(hwnd,umsg,wparam,lparam)
		// Here we check special keys...
      
   	local nSkip
		
		// is the incoming message a key-up event ?
		// WM_KEYUP = 257

      if umsg = 257
	
      do case 
       	case wparam =13
         	** If event handler exists, execute it.
         	if type("this.onEnterAction") $ "CB|FP"
            	this.onEnterAction()
         	endif
        	  //	return  // Removed. This prevents executing CallWindowProcA which
                       // is normally executed by the return from the function. MB
         case wparam =40 or wparam =38
            this.SkipRecs( -(39-wparam ) )
         case (wparam =33 AND this.lEnablePgUpDn)  // PgUp key, if enabled
            this.SkipRecs( -(this.nSkipRecords) )
         case (wparam =34 AND this.lEnablePgUpDn)  // PgDn key, if enabled
            this.SkipRecs( (this.nSkipRecords) )
         case (wparam =35 AND this.lEnableHomeEnd) // End key, if enabled
            this.Rowset.last()
         case (wparam =36 AND this.lEnableHomeEnd) // Home key, if enabled
            this.Rowset.first()
      endcase
		endif
	
	// 12/24/14 On Some Win7 64 bit machines get error on following line: "Memory Access Violation"	
	return CallWindowProcA(this.oldproc, hwnd, uMsg, wParam, lParam)

   function SkipRecs( nSkip )
   	if not this.Rowset.endofset // some records must exist
         this.Rowset.next(nSkip)
         if this.Rowset.endofset
            if nSkip > 0 // advanced past the last record
					this.Rowset.last()
            else
              	this.Rowset.first()
            endif
         endif
      endif
   return

    function cleanup()
   	if this.lcuae
      	set cuaenter on
      else 
      	set cuaenter off
      endif
      this.rowset = null
   return

   function normalizedValue()
  	return ""+this.value.rightTrim().toUpperCase()

endclass

class mNumSeekerADO( oParent ) of mSeekerADO( oParent ) custom
   // deal with numeric values instead of character
function oSeek
   	local lRet, bBookMark, cSORTString
    	bBookMark = this.Rowset.BookMark()

    	 lRet = true

       // 1/26/14  Tom G - Removed check for SQL Order By
       //          Added check for the ADO rowset's Sort field  property
		 cSORTString = this.rowset.sort
		 if empty(cSORTString)
          msgbox("You Must set the 'Sort' property for the ADO rowset.","Information: mSeekerADO.cc",64)
          lRet = false
       endif
       if lRet
          lRet = this.seekRowset.ApplyLocate( cSORTString+" = "+this.normalizedValue())
       endif
       if ( this.seekRowset.endOfSet )
         this.seekRowset.next( -1 )
       endif
    return lRet	
	
   function NormalizedValue
   return val( this.Value )
endclass