/*
   ----------------------------------------------------------------------------
   MyHTMLReport.prg
   Sample code from The dBASE Reports Book, second edition
   Chapter 23
   Copyright  2016, Kenneth J. Mayer
   
   A program to generate an HTML Report of the Household Expenses
   table.
   
   This may be used for other purposes, as needed. 
   ----------------------------------------------------------------------------
*/
// open the custom class file with our HTML code in it:
set procedure to MyHTML.cc additive
oHTML = new MyHTML()

// Data Objects
oData = new database()
oData.databaseName := "DBASEREPORTS"
oData.active       := true
oHE = new query() // "HE" = Household Expenses
oHE.database := oData
oHE.sql      := "select * from Household"
oHE.active   := true

// some shortcuts to the rowset and fields array:
rHE = oHE.rowset
fHE = rHE.fields

// index:
rHE.indexName := "EntryDate"

// go to the first row of the table:
rHE.first()

// Create the file object we need:
oRep = new file()
oRep.create( "HouseholdExpenses2.htm" )

// start with the HTML tag:
oRep.puts( [<html>] )

// Output the HTML Header section:
oRep.puts( oHTML.Header( "Household Expenses Report" ) )

// put the body tag in manually for now:
oRep.puts( oHTML.cTab+[<body>] )

// any headings on the report itself should go here
oRep.puts( oHTML.MainHeader() )
oRep.puts( oHTML.SecondaryHeader( "Household Expenses" ) )
oRep.puts( oHTML.DateText() )

// the "body of the report" will go here ...
oRep.puts( oHTML.cTab2+[<p>] ) // add a blank line into the report

// center the report in the browser window:
oRep.puts( oHTML.cTab2+[<center>] )

// output the beginning of the HTML table:
oRep.puts( oHTML.cTab3+[<table>] )

// add some column headings:
oRep.puts( oHTML.cTab4+[<tr align="left" style="font-family:Arial; font-size: 10pt;">] )
oRep.puts( oHTML.cTab5+[<th>Date</th>] )
oRep.puts( oHTML.cTab5+[<th>&nbsp;&nbsp;&nbsp;</th>] )
oRep.puts( oHTML.cTab5+[<th>Category</th>] )
oRep.puts( oHTML.cTab5+[<th>&nbsp;&nbsp;&nbsp;</th>] )
oRep.puts( oHTML.cTab5+[<th align="right">Amount</th>] )
oRep.puts( oHTML.cTab5+[<th>&nbsp;&nbsp;&nbsp;</th>] )
oRep.puts( oHTML.cTab5+[<th>Note</th>] )
oRep.puts( oHTML.cTab4+[</tr>] )

// we need to loop through the table here:
do while not rHE.endOfSet
   // we start with a table row tag:
   oRep.puts( oHTML.cTab4+[<tr valign="top" style="font-family:Arial; font-size: 10pt;">] )
   // here's where we put the data that is to be
   // displayed for the report - this is with the table data tags:
   oRep.puts( oHTML.cTab5+[<td>]+fHE["EntryDate"].value+[</td>] )
   oRep.puts( oHTML.cTab5+[<td>&nbsp;&nbsp;&nbsp;</td>] )
   oRep.puts( oHTML.cTab5+[<td>]+fHE["Category"].value+[</td>] )
   oRep.puts( oHTML.cTab5+[<td>&nbsp;&nbsp;&nbsp;</td>] )
   // right align the Amount, and format it:
   oRep.puts( oHTML.cTab5+[<td align="right">]+;
              transform( fHE["Amount"].value, "9,999.99" )+;
              [</td>] )
   oRep.puts( oHTML.cTab5+[<td>&nbsp;&nbsp;&nbsp;</td>] )
   oRep.puts( oHTML.cTab5+[<td>]+fHE["Note"].value+[</td>] )
   
   // and we end with an end table row tag:
   oRep.puts( oHTML.cTab4+[</tr>] )
   
   // next row in data table:
   rHE.next()
enddo // done looping through the table

// end of the HTML table tag:
oRep.puts( oHTML.cTab3+[</table>] )
// turn off the centering
oRep.puts( oHTML.cTab2+[</center>] )

// you could add something as a footer here
// if you wished ...

// ending body tag:
oRep.puts( oHTML.cTab+[</body>] )

// end with the closing HTML tag:
oRep.puts( [</html>] )

// close the file
oRep.close()

// release the file object
release object oRep
oRep = null

// clean up memory by closing the data objects:
oHE.active := false
release object oHE
oHE = null

oData.active := false
release object oData
oData = null
// done!
