// form metric must be 6 - pixels for this ...
/*
    InfoWindowButton.cc
    Ken Mayer
    April, 2006

    Notes.....: This class is a pushbutton that will drop 
                down right by the button an information
                window that can contain as much helpful
                text as needed ...

                This is based off the code used for the DateCalendar.cc
                also in the dUFLP ...

                Note that it lines up the right of the form
                with the right of the button -- this is so the
                window doesn't end up off the form, as at least
                on my forms this kind of thing is usually
                in the upper right corner ...

    Usage.....: set procedure to :dUFLP:InfoWindowButton.cc additive
                // in the form designer, place this control
                // on a form.
                 
                In the form's Init or other setup code (onOpen), set the
                text for the button:

                   form.InfoWindowButton1.text := "Some sample text"
                   // colors:
                   this.WindowColor = "0x80ffff"
                   this.EditorColor = "WindowText/0x80ffff"

                When the form runs, the value assigned to the 
                text property will be displayed. For line breaks,
                use <p> ... should be able to handle standard
                (basic) HTML as well. For <p> remember
                to put a space before the tag, or it will mess up
                your display.

    Dependencies:
                :dUFLP:Center32.prg
                :dUFLP:WinAttr32.cc
                :dUFLP:WinPosition.cc

*/
class InfoWindowButton( oParent ) of Pushbutton( oParent ) custom
   with (this)
      onOpen = class::IWB_ONOPEN
      onClick = class::IWB_onClick
      metric   := 6 // Pixels
      height   := 25.0
      width    := 26.0
      upbitmap := "resource #112"
   endwith

   // custom properties:
   this.dropped  = false  // whether the info form has "dropped" ...
   this.text     = ""     // text to display
   this.WindowColor = "Infobk"
   this.EditorColor = "InfoText/Infobk"

   function IWB_onOpen
      // determine if button should have SystemTheme property set:
      if __version__ = 2.6
         // for these toolbuttons, don't use the system theme ...
         this.systemTheme := false // for dBASE Plus 2.6
      endif

      // Defaults:
      this.dropped = false

      // Need WinPosition class to get positioning
      // information:
      set procedure to :dUFLP:WinPosition.cc additive
      oRef = new winPosition()
      oRef.get( int( this.hwnd ) )

      // Create instance of form, set top/left properties
      this.oWin = new InfoWindowForm()
      this.oWin.mdi = false
      this.oWin.top = oRef.bottom
         // line the right side of the form with the right 
         // side of the button
      this.oWin.left = oRef.left - (this.oWin.width - this.width )
      this.oWin.parent = this // set parent for form
      this.oWin.InfoEditor.value := this.text
      // Don't display text on button, assign
      // to the infoeditor:
      if not empty( this.text )
         this.text := ""
      endif

      // cleanup of winPosition
      oRef.release()
      oRef := null
      close procedure :dUFLP:WinPosition.cc

      // set colors
      this.oWin.colorNormal = this.WindowColor
      this.oWin.InfoEditor.colorNormal := this.EditorColor
   return

   function IWB_onClick
      /*
         This should display a form directly under the button
      */
      if not this.dropped
         this.dropped := true
         this.oWin.open()
      else
         this.dropped := false
         this.oWin.onLostFocus()
      endif

   return

endclass
/*
   This is the form that will be used to
   display when the user clicks the button as defined
   above ...
*/
class InfoWindowForm of FORM
   with (this)
      onLostFocus = class::FORM_ONLOSTFOCUS
      onOpen = class::FORM_ONOPEN
      onClose = class::FORM_ONCLOSE
      open = class::FORM_OPEN
      scaleFontBold = false
      metric = 6	// Pixels
      colorNormal = "Infobk"
      height = 165.0
      width = 342.0
      text = ""
      topMost = true
      smallTitle = true
      sizeable = false
      moveable = false
      sysMenu = false
      escExit = false
      maximize = false
      minimize = false
   endwith

   this.INFOEDITOR = new EDITOR(this)
   with (this.INFOEDITOR)
      height = 142.0
      left = 0.0
      top = 0.0
      width = 340.0
      modify = false
      borderStyle = 3 // None
      if smallFonts()
         fontSize = 10
      else
         fontSize = 8
      endif
   endwith

   this.OKBUTTON = new PUSHBUTTON(this)
   with (this.OKBUTTON)
      onClick = class::OKCLICKED
      height = 20.0
      left = 112.0
      top = 144.0
      width = 118.0
      text = "Done"
      if smallFonts()
         fontSize = 8
      else
         fontSize = 6
      endif
   endwith

   function form_open
      class::Init()
   return FORM::open()

   function Init
      /*
         Set up the anything? 
         Let's set the width and height of the
         editor, so if the form changes ...:
      */
      this.OKButton.top      := this.height - this.OKButton.height
      this.OKButton.width    := (this.width - this.OKButton.width)/2
      this.InfoEditor.width  := this.width
      this.InfoEditor.height := this.height - this.OKButton.height - 3
   return

   function form_onOpen
      // these have to be set after the form has opened
      set procedure to :dUFLP:WinAttr32.CC additive
      set procedure to :dUFLP:CENTER32   additive

      // deal with Window attributes here:
      form.oWinAttr = new WinAttr32( form )
      form.oWinAttr.set( "caption", false ) // titlebar
      form.oWinAttr.MinButton(false)
      form.oWinAttr.MaxButton(false)
      form.oWinAttr.DlgFrame(false)
      form.oWinAttr.Border(true) // single-line border around window
      form.oWinAttr.ThickFrame(false)
      form.oWinAttr.SysMenu(false)
   return

   function OKClicked
      form.close()
   return

   function form_onClose
      // close the winattr32 object:
      form.oWinAttr.close()
      form.oWinAttr := null
      if type( "form.parent" ) == "O"
         form.parent.dropped := false // form is closed ...
      endif
   return

   function form_onLostFocus
      // if the form loses focus, we need to 
      // close but don't assign a value ...
      form.close()
   return

endclass

Function Smallfonts
/*
   Programmer   : Romain Strieff [dBVIPS]
   Last Modified: 04/20/1999
   Rev. History.: 04/20/1999 -- Original
   Notes........: Returns True if Smallfonts are used
   .............: in Windows menus
   Written for..: Visual dBASE (32-bit versions)
   Usage .......: ? SmallFonts()
*/
   local dc,result
   extern CINT        GetDeviceCaps(cHandle, CINT) gdi32
   extern CINT        ReleaseDC(cHandle, cHandle) user32
   extern cHandle     GetDC(cHandle) user32
   #define LOGPIXELSX    88
   dc=getdc(0)
   result=GetDeviceCaps(dc,LOGPIXELSX)
   ReleaseDc(0,dc)
return result=96
// eof: SmallFonts()

