/*
   KenWebForm.cc
   A custom control designed to help the developer
   create a web application that uses the standard
   HTML form controls ...

   This class should be able to be used either with
   the WebClasses, or by itself ... 

   In order to help, it's been set up to be able to be used
   by itself ...

   The real point of this is to make it easier for a dBASE developer
   to generate an HTML FORM without having to know a lot about
   HTML Form tags ... 

   Updated: August 28, 2002 -- Modified to stream out lower-case
                               tags, and be sure that all tags that
                               should have a closing tag according
                               to XHTML specs does ...

   Methods:
      createFile -- creates the HTML document
      closeFile -- closes the HTML document -- don't forget to call this!

      streamHeader -- outputs the header at the top of the document
                      ( <head> tags and title and  ...)
      streamBody   -- outputs the body tag -- see documentation for
                      this method for details
      streamTitle  -- outputs a nice title at the top of the form
      streamDetail -- outputs individual lines of text/HTML to document
      streamFooter -- closes up (</BODY></html>)

      streamFormBegin -- steams out the <FORM> code with the
                         cgi definition you specify for what to do
                         with the form when it's submitted
      streamFormEnd   -- streams out the end tag for a form
      streamText      -- streams out an HTML entryfield
      streamPassword  -- streams out an HTML entryfield with a
                         password mask
      streamRadio     -- streams out an HTML radio button
      streamCheckbox  -- streams out an HTML checkbox
      streamReset     -- streams out an HTML "Reset" button
      streamSubmit    -- streams out an HTML "Submit" button
      streamTextArea  -- streams out an HTML Editor control
      streamSelectBegin -- streams out an HTML Combobox or Listbox
                           (see details in documentation for
                           this method)
      streamOption    -- stream out elements used in Select
      streamSelectEnd -- end tag for a select object
      streamFile      -- streams HTML "File" entryfield, which has
                         a BROWSE button, useful for finding a file
                         on a user's machine.

   Example:
      set procedure to KenWebForm.cc additive
      oMyForm = new KenWebForm()
      oMyForm.createFile( "TestHTMLForm.htm" )
      oMyForm.streamHeader("Test Form")
      oMyForm.streamBody()
      oMyForm.streamTitle( "Test HTML Form" )

      // the following intersperses some HTML and methods used here
      // specifically a table used to layout the form:
      oMyForm.streamFormBegin( "cgi-bin/MydBASEexe.exe" )
      oMyForm.streamDetail( "<table>" )
      // first row, first cell
      oMyForm.streamDetail( "<tr><TD ALIGN=RIGHT>" ) 
      oMyForm.streamDetail( "<B>Name:</B>" )
      oMyForm.streamDetail( "</td><td>" )
      oMyForm.streamText( "NameField", 20, "", 25 )
      oMyForm.streamDetail( "</td></tr>" )
      // next row, first cell
      oMyForm.streamDetail( "<tr><TD ALIGN=RIGHT>" )
      oMyForm.streamDetail( "EMailAddress: ")
      oMyForm.streamDetail( "</td><td>" )
      oMyForm.streamText( "EMail", 50, "", 50 )
      oMyForm.streamDetail( "</td></tr>" )
      // etc.
      // Submit and Reset buttons:
      oMyForm.streamDetail( "<tr><TD ALIGN=RIGHT>" )
      oMyForm.streamSubmit()
      oMyForm.streamDetail( "</td><td>" )
      oMyForm.streamReset()
      oMyForm.streamDetail( "</td></tr>" )
      oMyForm.streamDetail( "</table>" )

      // stream the footer out:
      oMyForm.streamFooter()
      // And close it ... and there you have an HTML file with
      // some form controls
      oMyForm.closeFile()
*/

class KenWebForm of object

   // Constructor code:

   // create default color and background image settings here
   // that will affect the whole HTML document and any document
   // using this instance of the class:
  
   // NOTE: colors must be in either text format (i.e., "BLUE" )
   //       or in hexidecimal (RGB) with the "#" in front of
   //       them ...

   // These are used by the streamBody() method if
   // no values are passed:
   this.BackgroundImage = "" // image that is displayed over the whole
                        // document
   this.BackgroundColor = "" // default background color of an HTML 
                        // document
   this.TextColor       = "BLACK" // default color of text on a page
   this.LinkColor       = "" // default link color (default is blue)
   this.VLinkColor      = "" // default Visited link color (one that the
                        // user has clicked on -- default is purple)
   this.ALinkColor      = "" // default color of link that is 'active' --
                        // mouse is over it and the left mouse button
                        // is down -- default is red(??)

   // These are used by the streamTitle() method if 
   // no values are passed:
   this.TitleBackColor  = "LIGHTBLUE" // Light blue
   this.TitleTextColor  = "WHITE"     // white

   function CreateFile( cFileName )
      /*
          creates a file object as a part of this class
          and assigns the name cFileName ...

          The file should have an HTM or HTML extension (myfile.htm)
      */
      this.fOut = new File()
      this.fOut.create( cFilename, "W" )
   // end of method: CreateFile()

   function CloseFile()
      /*
         Call this method at the end of your creation of the HTML
         document ...
      */
      this.fOut.close()
   // end of method: CloseFile()

   // -----------------------------------------------------------------
   // Streaming of HTML Documents -- I am modifying some of the 
   // original code in WebClass.cc so I have to copy the code
   // itself to here. In addition, I am giving a bit more flexibility
   // in the appearance and such ... this allows you the ability
   // to customize a bit easier ...
   // -----------------------------------------------------------------

   Function StreamHeader(Title)
      cTitle = iif(empty(Title),'Response Page',Title)

      with (this.fOut)
         puts('<html>')
         puts('')
         puts('<head>')
         puts('	<meta http-equiv="Content-Type" content="text/html">')
         puts('	<title> '+cTitle+'</title>')
         puts('</head>')
         puts('')
      endwith

   return true
   // end of method: streamHeader

   function streamBody(cBackImage, cBackColor, cTextColor, cLinkColor, cVLinkColor, cALinkColor)
      /*
         This streams out the HTML body tag used at the top
         of an HTML document ... however, note that you can 
         pass the colors that you want to display in your
         HTML document ... all settings for a page get set here.
         Note that the colors must use either standard named
         colors (e.g., "BLUE", "LIGHTBLUE", etc.) or standard
         hexidecimal colors with the "#" in front of them (e.g.,
         "#999999" which is white) 

         If the color parameters are left out, the default is a
         white page with black text, etc.

         Parameters:
            cBackImage -- background image used on the form
            cBackColor -- background color (under the image, if any)
            cTextColor -- text color -- this is important for text
                          on top of a background color ...
            cLinkColor -- default color of links
            cVLinkColor -- Viewed links (once that have been clicked on
                           by a user)
            cAlinkColor -- Active link (if the cursor is over the link
                           and the mouse button is pressed ...)
      */

      // note that you can set defaults in the constructor code:
      cBackImage  = iif(empty(cBackImage),  this.BackgroundImage, cBackImage )
      cBackColor  = iif(empty(cBackColor),  this.BackgroundColor, cBackColor )
      cTextColor  = iif(empty(cTextColor),  this.TextColor,       cTextColor )
      cLinkColor  = iif(empty(cLinkColor),  this.LinkColor,       cLinkColor )
      cVLinkColor = iif(empty(cVLinkColor), this.VLinkColor,      cVLinkColor )
      cALinkColor = iif(empty(cALinkColor), this.ALinkColor,      cALinkColor )

      cOut = [<body]
      if not empty( cBackImage )
         cOut += [ background="]+cBackImage+["]
      endif
      if not empty( cBackColor )
         cOut += [ bgcolor="]+cBackColor+["]
      endif
      if not empty( cTextColor )
         cOut += [ text="]+cTextColor+["]
      endif
      if not empty( cLinkColor )
         cOut += [ link="]+cTextColor+["]
      endif
      if not empty( cVLinkColor )
         cOut += [ vlink="]+cVLinkColor+["]
      endif
      if not empty( cALinkColor )
         cOut += [ alink="]+cALinkColor+["]
      endif

      cOut+= ">" // close that tag
      this.streamDetail( cOut )
   // end of method: streamBody

   function streamTitle( cTitle, cBackColor, cTextColor )
      /*
          streamTitle is used to stream out a title that works
          like the one in the primary webclass for the sorryPage
          and errorPage methods ... it's simply done with
          the use of a table and setting the background for
          the cell in the table. However, this allows you to
          pass a parameter for the two colors (if you wish),
          and if you wish, you can set a default color here
      */
      // if no value passed, use default ...
      cBackColor = iif( empty( cBackColor ), this.TitleBackColor, cBackColor )
      cTextColor = iif( empty( cTextColor ), this.TitleTextColor, cTextColor )

      // Title:
      this.streamDetail([<table border="0" cellpadding="0" cellspacing="0" width="100%">])
      this.streamDetail([<tr align="center">])
      this.streamDetail([<td bgcolor="]+cBackColor+[">])
      this.streamDetail([<font color="]+cTextColor+[" size=6>]+cTitle+[</font>])
      this.streamDetail([</td>])
      this.streamDetail([</tr>])
      this.streamDetail([</table>])

      // Blank Line:
      this.streamDetail( [<p></p>] )
   // end of method: streamTitle      

   function streamDetail( cString )
      // cString is the string you want streamed out -- it can
      // contain HTML ...
      // NOTE: This should be called after the streamHeader()
      // and streamBody() methods are called ...
      this.fOut.puts( ""+cString )
   // end of method: streamDetail

   Function streamFooter
      with (this.fOut)
         this.fOut.puts(' ')
         this.fOut.puts('</body>')
         this.fOut.puts('</html>')
      endwith
   return true
   // end of method: streamFooter


   // streamFormBegin streams out the form tag
   function streamFormBegin( cCGI )
       /*
          The form tag needs to know what to do when
          the user submits it, hence the "cCGI" parameter ... 
          if this is left out, the form will not actually
          do anything if you click the submit button ... it will
          work, but ...

          Your CGI/Action ought to be something like:
             "/cgi-bin/myprog.exe" 
       */
      cOut = [<form method="POST" ]
      if pCount() => 1
         cOut += [action="]+cCGI+["]
      endif
      cOut += [>]

      this.streamDetail( cOut )
   // end of method: streamFormBegin

   // streamFormEnd streams out the end form tag
   function streamFormEnd
      this.streamDetail( "</form>" )
   // end of method: streamFormEnd

   // Form controls:
   function streamText( cName, nSize, cValue, nMaxLength )
      /*
         The 'text' control for HTML forms is
         the same as an entryfield ...
         Parameters:
           cName is the name -- if you want this to match a field,
                make sure you use *exact* case every time you 
                reference this, as the case is vital to the way 
                the web classes work (they are subclassed from 
                an Associative Array, and it is case sensitive).
           nSize is the width of the entryfield on the form --
                the size is character based ...
           cValue is the value you want to display -- could
                be the contents of the field
           nMaxLength is optional, and is the maximum amount
                of data you want to allow to be entered.
      */
      cOut = [<input type="TEXT" name="]+cName+["]
      if pCount() > 1
         cOut+= [ size=]+nSize
      endif
      if pCount() > 2
         cOut+= [ value="]+cValue+["]
      endif
      if pCount() > 3
         cOut+= [ maxlength=]+nMaxLength
      endif
      cOut+= [>]
      this.streamDetail( cOut )
   // end of method: streamText

   function streamPassword( cName, nSize, cValue, nMaxLength )
      /*
          the password control in HTML is exactly the same
          as the text control, except that a password mask
          is displayed for each character entered, rather
          than displaying the character.

          The parameters for this control are exactly the
          same as the text control -- see comments for
          this above.
      */
      cOut = [<input type="PASSWORD" name="]+cName+["]
      if pCount() > 1
         cOut+= [ size=]+nSize
      endif
      if pCount() > 2
         cOut+= [ value="]+cValue+["]
      endif
      if pCount() > 3
         cOut+= [ maxlength=]+nMaxLength
      endif
      cOut+= [>]
      this.streamDetail( cOut )
   // end of method: streamPassword

   function streamRadio( cName, cValue, cText, lChecked )
      /*
         radio controls in HTML use multiple controls
         just like in dBASE. In HTML they are grouped
         by the NAME property (cName parameter)
         The VALUE is returned for whichever radiobutton
         was checked when the HTML form was submitted ...

         Parameters:
             cName --    as text control
             cValue --   value returned for radiobutton selected
             cText --    text that appears to the right of the 
                         radiobutton
             lChecked -- is this radiobutton defaulting to 
                         a 'selected' state?
      */
      cOut = [<input type="RADIO" name="]+cName+["]
      cOut+= [ value="]+cValue+["]
      if pCount() > 3 and lChecked 
         cOut+= [ checked]
      endif
      cOut+= [>]
      if pCount() > 2
         cOut+= cText
      endif

      this.streamDetail( cOut )
   // end of method: streamRadio

   function streamCheckbox( cName, cValue, cText, lChecked )
      /*
          the Checkbox works similar to a dBASE checkbox,
          the 'value' is actually the text assigned to
          the checkbox. Note that the HTML checkbox 
          returns the value property, not a true or false
          property ... when checking your return values
          you would want to check for the value -- i.e.,
            if you pass a value property to this method
            of "A Test", and a name of "MyCheckbox", 
            if the checkbox is checked, the value returned is:
               oCGI["MyCheckbox"] = "A Test"
            OTHERWISE, the name is not returned at all -- 
            so your code would need to check for the existance
            of the key:
               if oCGI.isKey( "MyCheckbox" )
                  // it was checked and the value was passed on
               endif
          Parameters:
             cName --    as text control
             cValue --   value returned if checkbox was checked
             cText --    text that appears to the right of the 
                         Checkbox
             lChecked -- is this checkbox defaulting to a 'checked' 
                         state?
      */
      cOut = [<input type="CHECKBOX" name="]+cName+["]
      cOut+= [ value="]+cValue+["]
      if pCount() > 3 and lChecked 
         cOut+= [ checked]
      endif
      cOut+= [>]
      if pCount() > 2
         cOut+= cText
      endif

      this.streamDetail( cOut )
   // end of method: streamCheckbox

   function streamReset( cText )
      /*
         The reset button simply clears out the values in
         an HTML form ...

         Parameters: cText -- text displayed on the button
                              this is optional ...
      */
      cOut = [<input type="reset"]
      if pCount() => 1 // anything beyond 1 is ignored
         cOut+= [ value="]+cText+["]
      endif
      cOut+= [>]

      this.streamDetail( cOut )
   // end of method: streamReset

   function streamSubmit( cText )
      /*
         The submit button submits the contents of the form ...

         Parameters: cText -- text displayed on the button
                              this is optional ...
      */
      cOut = [<input type="submit"]
      if pCount() => 1 // anything beyond 1 is ignored
         cOut+= [ value="]+cText+["]
      endif
      cOut+= [>]

      this.streamDetail( cOut )
   // end of method: streamSubmit

   function streamTextArea( cName, nRows, nCols, cValue )
      /*
          A textArea control in HTML is like an editor control
          In HTML this one actualy has two tags -- the second
          is used after the default value ...
          

          Parameters:
              cText -- as other controls except the buttons
                       reset/submit
              nRows -- this is the number of rows to take up
                       on a form -- it does not limit the
                       input, just the display
              nCols -- same as nRows -- how many columns to use.
              cValue -- optional, this is a default value if
                       adding data, or the contents of the field
                       you want to display ...
      */
      cOut = [<textarea name="]+cName+["]
      if pCount() > 1
         cOut+= [ rows=]+nRows
      endif
      if pCount() > 2
         cOut+= [ cols=]+nCols
      endif
      cOut+= [>]
      this.streamDetail( cOut )
      if pCount() > 3
         this.streamDetail( cValue )
      endif
      cOut = "</textarea>"
      this.streamDetail( cOut )
   // end of method: streamTextArea

   // --------------------------------------------------
   // The select control is interesting and different
   // from all the other HTML controls ... takes
   // some extra work. For our purposes, it's easier
   // to break it out into parts, allowing you to define
   // your SELECT in as much detail as you need ...
   // ex:
   //     oCGI.streamSelectBegin( "MyList" )
   //     oCGI.streamOption( "Option1", "Display this", true )
   //     oCGI.streamOption( "Option2", "Display another" )
   //     oCGI.streamSelectEnd()
   // --------------------------------------------------
   function streamSelectBegin( cName, nSize, lMultiple )
      /*
          This method starts the Select object, but you must
          also call the streamOption method for each item
          you wish to appear in your SELECT, and when done
          you must call the streamSelectEnd() method to
          stuff the </SELECT> tag into the form ... (without
          that tag your form is hosed!)

          Parameters:
             cName -- see other controls
             nSize -- if size is zero, or omitted, 
                      you will get a combobox. If
                      the size is larger than that,
                      you will get a listbox ...
             lMultiple -- allow multiple selections?
                      NOTE: If you select this, you 
                      will, by definition, get a listbox,
                      not a combobox ...
      */
      cOut = [<select name="]+cName+["]
      if pCount() > 1
         if nSize > 1
            cOut+= [ size=]+nSize
         endif
      endif
      if pCount() > 2 and lMultiple
         cOut+= [ multiple]
      endif
      cOut+= [>]
      this.streamDetail( cOut )
   // end of method: streamSelect

   function streamOption( cValue, cText, lSelected )
      /*
          This method streams out the individual options that
          are needed for your select control.

          cValue -- is the value returned when the form is submitted
                    if this option is selected (required)
          cText  -- is the text displayed (it can be different from
                    the value) (required)
          lSelected -- is this selected when the form is displayed 
                    the first time?  (optional)
      */
      cOut = [<option value="]+cValue+["]
      if pCount() > 2 and lSelected
         cOut+= [ selected]
      endif
      cOut+= [>]
      this.streamDetail( cOut )

      this.streamDetail( cText )
   // end of method: streamOption

   function streamSelectEnd
      // a simple method, this just makes sure we have
      // the end tag for the select option -- without it,
      // we have a messed up form ...
      this.streamDetail( "</select>" )
   // end of method: streamSelectEnd()

   function streamFile( cName, nSize )
      /*
          streamFile streams out the HTML <input type="FILE" ...>
          tag, which gives an entryfield, and a "Browse" button,
          allowing a user to select a filename. This is quite
          useful when working with FTP or other methods of 
          uploading a file to a server.

          Parameters:
             cName -- the name of the name/value pair
             nSize -- width of the entryfield
      */ 
      cOut = [<input type="FILE" name="]+cName
      if pCount() > 1
         cOut+= [ size=]+nSize
      endif
      cOut += [">]
   // end of method: streamFile

endclass
