; MyFishInstall.iss
; A sample installer created for The dBASE Book, Chapter 28
; to install the "MyFish" application on a computer.
;
; This sample is for the Manual Install part of the chapter.
;
; This adds some options for customization that cannot be done
; using the Project Explorer, and calls the BDE/Runtime
; install in a different way.
;
; If you want to experiment with this script, or even just
; run it:
;    a) make sure that you have compiled the MyFish.exe 
;       from the Fish1.prj file in the folder this is in;
;    b) make sure the paths are correct throughout ...
;       for example, the C:\Users\ paths have my username
;       in them ("Ken") -- you would want to change them
;       appropriately and so on (I have tried to use
;       Inno Setup constants where possible to alleviate
;       the hard-coding of paths). Check each path ...

[Setup]
; These two entries following are required:
AppName=MyFish
AppVersion=1.0.0.0

; This is used on the Welcome page of the setup
; dialog, and in the Add/Remove Program entry
; in Windows:
AppVerName=MyFish 1.0.0.0

; The directive AppID is used for your uninstall
; program. It is probably a good idea to use it.
; The suggestion is to use the name of the program,
; similar to AppName as shown above:
AppID=MyFish

; Copyright information (the copyright symbol is
; created by using the alt key and the numeric
; keypad on the right side of your keyboard: alt+0169:
AppCopyright=Copyright  2013 Ken Mayer

; If you wish to allow your users to Select the path
; to install to (which is a default setting). You can
; find more on this directive in the Inno Setup help. The
; constant {pf} references "C:\Program Files (x86)", or
; "C:\Program Files" depending on the operating system.
; This path is what is referenced in the script by the
; Inno Setup constant {app}:
DefaultDirName={pf}\dBASEBook\MyFish

; If you expect your user may be installing updates,
; you can include this directive as well:
UsePreviousAppDir=yes

; This is the group in Windows, where a link to the
; executable and any other related files will be placed
DefaultGroupName=MyFish

; The following directive defaults to "Yes", the difficulty
; is that it can confuse your end-users as it shows all the
; components that will be installed. It is a good idea
; to not do that unless you have a fairly complex installation:
AlwaysShowComponentsList=no

; This tells Inno Setup where to find the source
; code for your application. Rather than having
; to continually reference it in this script,
; you can then refer to the files in the path
; without having to put the full path in. If
; a subfolder is used, you would just reference
; the subfolder followed by files (i.e., 
;     Main\*.wfm   )
SourceDir=C:\dBASEBook1_SourceCode\Chapter 28

; Output directory is where you want Inno Setup
; to place the generated install program (the
; Project Explorer uses a folder called "Output",
; I prefer having some control over my folder
; names ...):
OutputDir=InstallImage

; What is the name of the actual setup program?
OutputBaseFilename=MyFishSetup

; The two following directives are used in
; the install screen and the uninstall screen:
AppPublisher=Golden Stag Productions
AppPublisherURL=http://www.goldenstag.net/dBASEBook

; Appearance of the installer -- this section is not
; required, but is handy to personalize your install:
SetupIconFile=C:\Users\Ken\AppData\Local\dBASE\PLUS\Samples\Fish.ico
WizardImageBackColor=clGreen
WizardImageFile=Images\SmallStag.bmp
WizardImageStretch=no

; -- Text Files --
; License file - the license here was copied from
; http://www.gnu.org/licenses/#GPL - it is a generic
; General Public License ... for your own software
; you would obviously want to determine the license 
; you would need with any specifics ...
LicenseFile=TextFilesForInstall\license.txt
; Information to be displayed after license is
; accepted, but before installation starts.
InfoBeforeFile=TextFilesForInstall\pre-install.txt
; Information displayed after installation,
; but before any "readme.txt" type file.
InfoAfterFile=TextFilesForInstall\post-install.txt

; ----------------------------------------------------------
; ------------------ End of SETUP section ------------------
; ----------------------------------------------------------

[Dirs]
; If your application needs to store files in multiple
; folders, this is where you would define them. 
; For example, you might want to store the data
; in the C:\ProgramData folder, then any sub-folders
; you might need. The first location is:
;     C:\ProgramData\dBASEBook\MyFish\Tables   and should be read & execute only:
Name: "{commonappdata}\dBASEBook\MyFish\Tables"; Permissions: users-readexec;
; The second location is the real location of the data -- where
; the data can be modified:
;     C:\Users\username\AppData\Local\dBASEBook\MyFish\Tables
Name: "{localappdata}\dBASEBook\MyFish\Tables"; Permissions: everyone-full;

; -----------------------------------------------------
; Where the dBASE Plus Runtime Engine will go: "C:\Program Files (x86)" if on 
; 64-bit versions of Windows, or on 32-bit versions, "C:\Program Files"):
Name: "{pf}\dBASE\Plus\Runtime"; Permissions: everyone-readexec; Flags: uninsneveruninstall;

; -----------------------------------------------------
; Where the BDE will go:
Name: "{cf}\Borland\BDE"; Permissions: everyone-full; Flags: uninsneveruninstall;

; If you were going to do a DEO installation, you
; might want to have a folder stored either locally
; or on the network (if this is a network application).
; See information in The dBASE Book, Chapter 28, on
; setting up folders for a network application,
; and dealing with "type" of installation.

; ----------------------------------------------------------
; ------------------ End of DIRS section -------------------
; ----------------------------------------------------------

[Files]
; The executable itself, to the application folder:
Source: "MyFish.exe"; DestDir: "{app}"; Flags: ignoreversion

; INI File --
; if you have an existing .INI file with specific settings in it
; then you will want to copy this to the application folder.
Source: "MyFish.ini"; DestDir: "{app}"; Flags: ignoreversion

; Tables, indexes and .dbts to the application\data folder -- 
; note that you can use wildcards in some cases (i.e., *.dbf):
Source: "Tables\Fish.dbf"; DestDir: "{commonappdata}\dBASEBook\MyFish\Tables"; Flags: onlyifdoesntexist;
Source: "Tables\Fish.mdx"; DestDir: "{commonappdata}\dBASEBook\MyFish\Tables"; Flags: onlyifdoesntexist;
Source: "Tables\Fish.dbt"; DestDir: "{commonappdata}\dBASEBook\MyFish\Tables"; Flags: onlyifdoesntexist;
Source: "Tables\Fish.dbf"; DestDir: "{localappdata}\dBASEBook\MyFish\Tables"; Flags: onlyifdoesntexist;
Source: "Tables\Fish.mdx"; DestDir: "{localappdata}\dBASEBook\MyFish\Tables"; Flags: onlyifdoesntexist;
Source: "Tables\Fish.dbt"; DestDir: "{localappdata}\dBASEBook\MyFish\Tables"; Flags: onlyifdoesntexist;

; Add any other files that would need to be deployed, which might
; include such things as a Readme file, image files, DLL files 
; that may be required, etc. This would include putting files 
; on the network for DEO applications, or other (see details 
; in Ch. 28 of the book).

; BDE/Runtime Installer for dBASE Plus (install to Temporary folder):
Source: "C:\Program Files (x86)\dBASE\Plus\Runtime\PlusRuntime-b2215_EN.exe"; DestDir: "{tmp}"; Flags: ignoreversion deleteafterinstall

; -----------------------------------------------------
; Manifest file for runtime:
Source: "C:\Program Files (x86)\dBASE\PLUS\Runtime\plusrun.exe.manifest"; DestDir: "{pf}\dBASE\Plus\Runtime"
; Manifest file for BDE:
Source: "C:\Program Files (x86)\dBASE\PLUS\Runtime\bdeadmin.exe.manifest"; DestDir: "{cf}\Borland\BDE"
; Manifest file for application, must be in same folder as .EXE file:
Source: "MyFish.exe.manifest"; DestDir: "{app}";

; ----------------------------------------------------------
; ------------------ End of FILES section ------------------
; ----------------------------------------------------------

[INI]
; -----------------------------------------------------
; Update the .ini file for the User BDE Alias:
Filename: {app}\MyFish.ini; Section: "UserBDEAliases";
Filename: {app}\MyFish.ini; Section: "UserBDEAliases"; Key: "0"; String: "dBASEFish";
Filename: {app}\MyFish.ini; Section: "dBASEFish";
Filename: {app}\MyFish.ini; Section: "dBASEFish"; Key: "Driver"; String: "DBASE";
Filename: {app}\MyFish.ini; Section: "dBASEFish"; Key: "Options"; String: "PATH: ""{localappdata}\dBASEBook\MyFish\Tables""";

; ----------------------------------------------------------
; ------------------- End of INI section -------------------
; ----------------------------------------------------------

[Run]
; Silent install of runtime suppresses the runtime install display, and the installshield stuff ...
; PlusRuntime-bNNNN_en.exe -s -a Runtime=d:\dBASEPLUS\Runtime BDE=d:\dBASEPLUS\bde Silent -sr
; In order to get this to work properly, the double-quotes are doubled up in a few places
; if you look at the statement ... that embeds a single double-quote (as it were) in the
; parameters needed for the runtime and bde installs.
;
; It should be noted that this version with the silent flag does not appear to
; be very "silent" ... however, this runs without the error created by the Project
; Explorer version:
Filename: {tmp}\PlusRuntime-b2215_EN.exe; Parameters: "-s -a Runtime=""{pf}\dBASE\Plus\Runtime"" BDE=""{cf}\Borland\BDE"" Silent -sr"; StatusMsg: "Installing dBASE Runtime and BDE..."; Flags: runascurrentuser; WorkingDir: {tmp};

; ----------------------------------------------------------
; ------------------- End of RUN section -------------------
; ----------------------------------------------------------

; -----------------------------------------------------------------------------------------------------
[Registry]
; If you need to modify the registry, create a specific key, this is 
; where you can do it during installation.
; 
; Note: BDE Registry entries are being handled in the CODE section
; below. This is due to the way that Inno Setup runs the sections.
; The Registry section is executed BEFORE the Run section.
; As such, if you set registry entries for the BDE, then
; run the BDE/Runtime installer (as shown above), the default
; registry settings will be made, overwriting whatever you did
; here. 
;
; Instead, the code at the end of this script runs during
; the "PostInstall" phase of the script, and therefore
; after the BDE has been installed. This will update the
; registry keys as shown.

; ----------------------------------------------------------
; ---------------- End of REGISTRY section -----------------
; ----------------------------------------------------------

[Icons] 
; Application icon (goes in the "Group")
Name: "{group}\MyFish"; Filename: "{app}\MyFish.exe"; WorkingDir: "{app}"

; Desktop icons (place icon on the desktop):
Name: "{userdesktop}\MyFish"; Filename: "{app}\MyFish.exe"; WorkingDir: "{app}";

; ----------------------------------------------------------
; ------------------ End of ICONS section ------------------
; ----------------------------------------------------------

; -----------------------------------------------------------
; ----------------- End of MAIN Part of SCRIPT --------------
; -----------------------------------------------------------

;------------------------------------------------------------
; The code section uses a form of Pascal and must be the last
; part of the Script. For this install test
; we are going to deal with with Registry settings
; in a "PostInstall" setting ...
[Code]

procedure CurStepChanged(CurStep: TSetupStep);
begin
   if CurStep = ssPostInstall then begin

      // at the suggestion of Jonny Kwekkeboom, check to see if registry
      // keys exist -- if they don't user may have cancelled the
      // BDE/Runtime installation, which means the application
      // won't run.
      if RegKeyExists(HKLM, 'SOFTWARE\Borland\Database Engine\Settings\System\INIT' ) then begin

         // BDE Registry settings, these are all based on the appendices
         // of The dBASE Book, which are taken from recommendations from
         // a variety of developers:
       
         // The following are under the System\INIT structure:
         // LOCAL SHARE: TRUE
         RegWriteStringValue(HKEY_LOCAL_MACHINE, 'SOFTWARE\Borland\Database Engine\Settings\System\INIT', 'LOCAL SHARE', 'TRUE');
         // MAXFILEHANDLES: 4096
         RegWriteStringValue(HKEY_LOCAL_MACHINE, 'SOFTWARE\Borland\Database Engine\Settings\System\INIT', 'MAXFILEHANDLES', '4096');
         // MAXBUFSIZE: 8192
         RegWriteStringValue(HKEY_LOCAL_MACHINE, 'SOFTWARE\Borland\Database Engine\Settings\System\INIT', 'MAXBUFSIZE', '8192');
         // MINBUFSIZE: 4096
         RegWriteStringValue(HKEY_LOCAL_MACHINE, 'SOFTWARE\Borland\Database Engine\Settings\System\INIT', 'MINBUFSIZE', '4096');
         // MEMSIZE: 205
         RegWriteStringValue(HKEY_LOCAL_MACHINE, 'SOFTWARE\Borland\Database Engine\Settings\System\INIT', 'MEMSIZE', '205');
         // SHAREDMEMSIZE: 8192
         RegWriteStringValue(HKEY_LOCAL_MACHINE, 'SOFTWARE\Borland\Database Engine\Settings\System\INIT', 'SHAREDMEMSIZE', '8192');
         // LOW MEMORY USAGE LIMIT: 32
         RegWriteStringValue(HKEY_LOCAL_MACHINE, 'SOFTWARE\Borland\Database Engine\Settings\System\INIT', 'LOW MEMORY USAGE LIMIT', '32');
         // LANGDRIVER: DBWINWE0 (or 'WEuropean ANSI')
         RegWriteStringValue(HKEY_LOCAL_MACHINE, 'SOFTWARE\Borland\Database Engine\Settings\System\INIT', 'LANGDRIVER', 'DBWINWE0');

         // The following are under the DRIVERS\DBASE\INIT structure: 
         // LANGDRIVER: DBWINWE0 (or 'WEuropean ANSI')
         RegWriteStringValue(HKEY_LOCAL_MACHINE, 'SOFTWARE\Borland\Database Engine\Settings\DRIVERS\DBASE\INIT', 'LANGDRIVER', 'DBWINWE0');

         // The following are under the DRIVERS\DBASE\TABLE CREATE structure: 
         // LEVEL: 7
         RegWriteStringValue(HKEY_LOCAL_MACHINE, 'SOFTWARE\Borland\Database Engine\Settings\DRIVERS\DBASE\TABLE CREATE', 'LEVEL', '7');
         // MDX BLOCK SIZE: 1024
         RegWriteStringValue(HKEY_LOCAL_MACHINE, 'SOFTWARE\Borland\Database Engine\Settings\DRIVERS\DBASE\TABLE CREATE', 'MDX BLOCK SIZE', '1024');
         // MEMO FILE BLOCK SIZE: 1024
         RegWriteStringValue(HKEY_LOCAL_MACHINE, 'SOFTWARE\Borland\Database Engine\Settings\DRIVERS\DBASE\TABLE CREATE', 'MEMO FILE BLOCK SIZE', '1024');

      end else
         // there is a problem, Houston ... you will want to use appropriate wording
         // for your application, language, etc.:
         MsgBox( 'Database Engine Not Installed -- This application will not function properly without it. Please re-install and complete the BDE/Runtime installation.', mbError, MB_OK );
         // no 'end' statement necessary
   end;
end;
// ----------------------------------------------------------
// ------------------ End of CODE section ------------------
// ----------------------------------------------------------



