// Metric: 0 - Chars and 6 - Pixels only
// -- File: ENTRY.CC

/* 
   ----------------------------------------------------------------
    Description:
 
       This UI file is designed to give some optional entryfields
       which can be used either 'as is' or you can even subclass 
       them.
 
    Programmers: 
 
       Ken Mayer
       Romain Strieff
 
    History: 
 
       11/28/1996 -- First version of this custom class file.
       12/11/1997 -- Removed all but BigCursorEntryField and
                     made minor change necessary for VdBASE 7
       01/06/1998 -- Added Romain's POSNEG_ENTRYFIELD
       03/28/1998 -- Added Gary White's clock
       07/18/1998 -- Added QDateEntryfield
       10/04/1998 -- removed Gary's clock and left it in CLOCK.CC
       January 28, 1999 -- added "valid" event to the 
                     QDateEntryfield below ...
       Sept. 9, 1999 -- Added ProperEntryfield
       October, 1999 -- added CCCardEntryField
                        added LanguageEntryfield
       December 20, 1999 -- removed valid event from QDateEntryfield,
                     for Visual dBASE 7.5, as the bug this was
                     created for has been fixed.
       November 15, 2000 -- Fixed bug in ProperEntryfield ...

    Instantiation: 
 
       set procedure to entry.cc additive
       * create/modify a form, place an instance of one of
       * these entryfields on the form as a normal entryfield ...
 
    Classes:
       More details on each of these is contained in the commentary
       for each class ...
 
       BigCursorEntryField
         When this field gets focus, the cursor changes shape
         to a larger one.

       POSNEG_EntryField
         When datalinked to a numeric field, this will display
         different colors based on whether or not the value is
         positive or negative.

       QDATEENTRYFIELD
         A date entryfield that works like the Quicken dates --
         specific keystrokes (as described in the code) will
         manipulate the date ...

       ProperEntryField
         Allows user to turn on/off "proper casing" for entryfield,
         handles most proper casing automatically ...

       LongYear
         Forces entry of four-digit year

       CCardEntryField
         Uses code in stringex.cc for isValidCard -- note,
           this requires that the stringex.cc be deployed
           with your application

       LanguageEntryfield
          Loads/unloads different keyboard layouts based on
          language drivers ...
   ----------------------------------------------------------------
*/

CLASS BigCursorEntryfield(FormObj) OF ;
                                        ENTRYFIELD(FormObj) Custom
   /*
      -------------------------------------------------------------
       Description:
    
          When an instance of this entryfield gets focus, the 
          cursor changes shape to one that is a bit more obvious 
          than the default.
    
       Programmers: 
    
          ???????
    
       History: 
     
          11/28/1996 -- Very minor modifications for the dUFLP 
                        library.
          12/11/1997 -- Changed call to USER to USER32
    
       Instantiation: 
    
          set procedure to entry.cc additive
          * create/modify a form, place an instance of this 
            entryfield
          * on the form, and datalink as usual.
            set the datalink as noted in the header of this file 
            (under "NOTE"). 
      -------------------------------------------------------------
   */

   Protect OnOpen,OnGotFocus

   PROCEDURE OnOpen
      if type("CREATECARET")# "FP"
         extern CVOID CreateCaret ( CHANDLE,CHANDLE,CINT,CINT ) USER32
      endif
      if type("SHOWCARET")# "FP"
         extern CVOID ShowCaret ( CHANDLE ) USER32
      endif
   
   PROCEDURE OnGotFocus
      createcaret(this.hwnd,0,5,24)  
      showcaret(this.hwnd)
      
ENDCLASS
// -- End of Class: BigCursorEntryField

CLASS POSNEG_Entryfield( FormObj ) of ENTRYFIELD( FormObj ) Custom
   /*
      -------------------------------------------------------------
       Description:
    
          When datalinked to a numeric field, the color will change
          if the value is negative to red ...
    
       Programmer: 
    
          Romain Strieff
    
       History: 
    
          01/06/1998 -- First posted in the Visual dBASE Newsgroups
    
       Instantiation: 
    
          set procedure to entry.cc additive
          * create/modify a form, place an instance of this 
            entryfield
          * on the form, and datalink as usual.
            set the datalink as noted in the header of this file 
            (under "NOTE"). 
      -------------------------------------------------------------
   */

   with (this )
//      metric         := 0      // chars
      colorHighlight := ""
      borderStyle    := 7 // Client
      SelectAll      := true
      onOpen         := class::InstallColor
   endwith

   PROCEDURE InstallColor
      if type( 'this.datalink' ) = 'O';
         and empty( this.datalink.beforeGetValue );
         and type( 'this.datalink.value' ) = 'N'
            this.dataLink.ref = this
            this.dataLink.beforeGetValue = class::CheckColor
      endif

   PROCEDURE CheckColor
      this.ref.colorNormal = ;
         iif( this.value < 0, 'WindowText/R+', 'WindowText/0xffff80' )
      RETURN this.value

endclass
// -- End of Class: POSNEG_EntryField

/*
     --------------------------------------------------------------
     QDATEENTRYFIELD
     Author: Ken Mayer (sort of)
     Date..: January 28, 1999

     A simple entryfield that emulates the Quicken-style
     keystrokes to manipulate the date. 

     These keystrokes are: 
        T  = today's date
        +  = forward one day (next day)
        -  = backward one day (previous day)
        M  = First day of current month
        H  = Last day of current month
        Y  = First day of current year
        R  = Last day of current year

     Easy enough ... the code in QDATEENTRYFIELD_Key was lifted
     from online help.

     REQUIRES: dateex.cc from the dUFLP library, as it uses
     methods from that class (FDOM, LDOM, FDOY, LDOY).

   ** Note that this cc file uses Source Aliasing, specifically the
   ** dUFLP source Alias. If you do not know how to set up
   ** a Source Alias, see the instructions in WHATS.NEW at
   ** the top of the file, or in README.TXT.
   ** Any files referenced by :dUFLP:filename  must be
   ** included in an executable if you are building 
   ** one ... make sure that your project includes these
   ** files.
   **
   ** This particular program uses:
   **           dateex.cc

     --------------------------------------------------------------
*/


class QDATEENTRYFIELD(parentObj) of ENTRYFIELD(parentObj) custom
   with (this)
      key = class::QDATEENTRYFIELD_KEY
      height = 1
//      left = 6
//      top = 2.5
      width = 11
      metric = 0	// Chars
      value = {}  // blank date
   endwith

   function onDesignOpen( lFromPalette )
      if lFromPalette and form.metric == 6 // Pixels
         this.height := 22
         this.width  := 77
      endif
   return

   function OnOpen
      // a couple of important things need to be done here:
      if not file( "dateex.cc" )
         msgbox( "Need 'dateex.cc' from the dUFLP library! "+;
                 "Copy that custom file over or this entryfield "+;
                 "won't work!", "Can't do it!", 64 )
         return
      endif
      set procedure to :dUFLP:dateex.cc addi
      this.d = new dateex()
      this.lastKey = ""  // required custom property

   function QDATEENTRYFIELD_key( nChar )
      // Code lifted from the onLine help (found in "Do Case" entry):
      local c1
      c1 = upper( chr( nChar ))
      do case
         case c1 == "T"                // Today
              this.value := date()
         case c1 == "-" or c1 == "_"   // Previous day (minus one day)
              this.value--
         case c1 == "+" or c1 == "="   // Next day (plus one day)
              this.value++
         case c1 == "M"                // First day of the month
              this.value := ;
                 this.d.FDoM( iif( this.lastKey == "M", --this.value, this.value ) )
         case c1 == "H"                // Last day of the month
              this.value := ;
                 this.d.LDoM( iif( this.lastKey == "H", ++this.value, this.value ) )
         case c1 == "Y"                // First day of the year
              this.value := ;
                 this.d.FDoY( iif( this.lastKey == "Y", --this.value, this.value ) )
         case c1 == "R"                // Last day of the year
              this.value := ;
                 this.d.LDoY( iif( this.lastKey == "R", ++this.value, this.value ) )
         otherwise
              this.lastKey := null        // Clear stored keystroke
              return true                 // Handle key normally
      endcase
      this.lastKey := c1              // Store as property for comparison next time
   return false                       // Ignore key

endclass

/*
   ProperEntryField
   By Peter Rorlick, posted in the Visual dBASE Newsgroups
   Thanks to Edward Sanders for the idea.

   Minor modification by KM -- moved function ProperName into
   the class, to make it a method of the class, for proper
   encapsulation ...

   Marko Mihorko found a bug where I had not quite completed
   the above fix. Sorry folks. It's been fixed ... 
   (November, 2000)

   In addition, colors set as custom properties, rather than
   hard-coded ... allows single change by developer ...

*/
CLASS ProperEntryField( oCont ) of Entryfield( oCont ) CUSTOM

   with (this)
      onGotFocus = {; this.keyboard( "{Home}" )}
    	BorderStyle = 7
    	Height = 1
    	SelectAll = false
    	speedTip := 'Press <Alt>+<=> to turn auto-casing on/off'
   endwith

   function onDesignOpen( lFromPalette )
      if lFromPalette and form.metric == 6 // Pixels
         this.height := 22
         this.width  := 140
      endif
   return


   this.lAutoCasing = true
   // set color definitions here:
   this.colorH = "N/RG+"   // highlight color
   this.colorN = "N/W+"    // normal color
   this.colorO = "N/CYAN"  // auto-casing is off
   this.ColorHighlight := this.colorH
   this.ColorNormal    := this.colorN

   FUNCTION key( nChar, nPosition )
      if nextkey() # 0
         // Wait until there are no keystrokes pending, and...
         RETURN ( nChar # 255 )     // ...ignore keystroke if special key
      endif
      if nChar == 255
         if this.lAutoCasing
            this.value := this.ProperName(this.value)
         endif
         return false
      else
         if nChar = 61   // <Alt>=
            this.lAutoCasing = not this.lAutoCasing
            if this.lAutoCasing
               this.ColorHighlight := this.colorH
            else
               this.ColorHighlight := this.colorO 
            endif
          endif

          if (nChar > 31 and nChar < 255) or ;
             nChar = 7 or nChar = 8 or nChar = 127  // trap printable chars or Del or Backspace.
             keyboard chr(255)
          endif
      endif
   RETURN nChar

   func onLostFocus
     this.lAutoCasing = true
     this.ColorHighlight = this.colorH 
   return

   function ProperName( cName )
      // Purpose: 	Format names properly
      // Usage:		? ProperName("MICHAEL MACLEOD")
      //					(returns "Michael MacLeod")
      //	Contributing authors: Peter Rorlick, Dan Howard, Brian Kavanaugh.
      //
      local TempString, NewString, EvalChar, mCount, Separators, OldString, CapNext
      Separators = " /-'<>=*+."  
      // add final space to make && routine easier to manage
      OldString = ltrim(rtrim(cName)) + " "  
      NewString = ""
      TempString = ""
      CapNext = true
      for mCount = 1 to len(OldString)
          EvalChar = subs(OldString, mCount, 1)
          if EvalChar $ Separators   // end of word
             CapNext = true
             NewString += TempString + EvalChar
             TempString = ""
          elseif CapNext   // initial character of a word
             TempString = upper(EvalChar)
             CapNext = false
          else   // character w/i word
             evalchar = lower(evalchar)
             TempString += EvalChar
             if TempString = "Mc" or TempString = "Mac"
                CapNext = true
                NewString += TempString
                tempstring = ""
             endif
          endif
      next
   return NewString

ENDCLASS
// end of class ProperEntryField

CLASS LongYear( oCont ) OF ENTRYFIELD( oCont ) CUSTOM
/*
   LongYear entryfield -- posted on VdB newsgroups by
   Romain Strieff, this entryfield forces the user
   to enter a four-digit year (they can't enter a two digit
   year ...).

   Date: Sept. 22, 1999
*/
   this.value = {} // date
   this.validErrorMsg := "Please key-in the full year"

   function onGotFocus
      this.epoch = set("epoch")
      set epoch to 9900 // huge value 

   function onLostFocus
      set epoch to this.epoch

   function valid
      lRet = true
      if year(this.value) > 9000
         lRet = false
      endif

   return lRet

ENDCLASS
// end of class LongYear

CLASS CCardEntryfield( oContainer ) OF ENTRYFIELD( oContainer ) CUSTOM
/*
   CCardEntryfield
   Ken Mayer
   September 30, 1999

   This entryfield is designed to work with the isValidCard code
   in the stringex.cc, to determine if a credit card *may*
   be a valid number. It uses a scheme that appears to work,
   but there are no guarantees ...

   ** Note that this cc file uses Source Aliasing, specifically the
   ** dUFLP source Alias. If you do not know how to set up
   ** a Source Alias, see the instructions in WHATS.NEW at
   ** the top of the file, or in README.TXT.
   ** Any files referenced by :dUFLP:filename  must be
   ** included in an executable if you are building 
   ** one ... make sure that your project includes these
   ** files.
   **
   ** This particular program uses:
   **           stringex.cc
   **           setproc.prg

*/
   this.width    := 20
   this.speedTip := "Enter a valid credit card number"

   function onDesignOpen( lFromPalette )
      if lFromPalette and form.metric == 6 // Pixels
         this.height := 22
         this.width  := 140
      endif
   return

   function onLostFocus
      set procedure to :dUFLP:stringex.cc additive
      local s, oRef
      s = new stringEx( this.value )
      if not s.isValidCard()
         oRef = this
         // msgbox type: 5 = retry/cancel, add 16 for error image
         if msgbox( "Credit Card number does not appear to be valid. "+;
                    "Try again? (Use 'Cancel' to continue ...)",;
                    "Invalid Card", 21 ) == 4 // 4 = retry
            oRef.setFocus() // back to entryfield
         endif
      endif
      s.release()
      close procedure :dUFLP:stringex.cc

ENDCLASS
// end of class CCCardEntryfield

class LanguageEntryfield(f) of Entryfield(f) custom
/*
    LanguageEntryfield
    Posted by Romain Strieff, Visual dBASE Newsgroups
    October 27, 1999

    This entryfield loads or switches to a specific
    defined keyboard mapping.

    See documentation after this class on different
    mappings ...
*/
   #define HKL_PREV            0
   #define HKL_NEXT            1

   #define KLF_ACTIVATE        0x00000001
   #define KLF_SUBSTITUTE_OK   0x00000002
   #define KLF_UNLOADPREVIOUS  0x00000004
   #define KLF_REORDER         0x00000008
   #define KLF_REPLACELANG     0x00000010
   #define KLF_NOTELLSHELL     0x00000080
   if type("LoadKeyboardLayout") # "FP"
      extern cHandle LoadKeyboardLayout(cstring, cint) user32 ;
             from "LoadKeyboardLayoutA"
   endif

   if type("GetKeyboardLayout") # "FP"
      extern cHandle  GetKeyboardLayout(cint) user32
   endif

   if type("GetKeyboardLayoutName") # "FP"
      extern cLogical GetKeyboardLayoutName(cString) user32 ;
             from "GetKeyboardLayoutNameA"
   endif

   if type("ActivateKeyboardLayout") # "FP"
      extern cLogical ActivateKeyboardLayout(cHandle, cInt) user32
   endif

   if type("UnloadKeyboardLayout") # "FP"
      extern cLogical UnloadKeyboardLayout(cHandle) user32
   endif

   //store active keyboad layout on instantiation
   this.original_keyboard_handle= GetKeyboardLayout(0)

   //store a default driver
   this.KeyboardName= itoh(0x100c,8) //french suiss

   //load all open keyboard drivers, so that we don't
   //load them again or accidently unload them
   this.aKeyboards=this.GetKeyboardArray()

   function OnGotFocus
      //the needed KBlayout is already loaded
      if this.aKeyboards.scan(this.KeyboardName)>0
         this.lUnload=false //so don't unload later
    
         //check what's active now
         cBuffer=space(8)+chr(0) 
         GetKeyboardLayoutName(cBuffer)
         cCurrentKB=cBuffer
         //go through the loaded drivers until we 
         //find the right one
         do while not cCurrentKB==this.keyboardname
            ActivateKeyboardLayout(;
            HKL_NEXT,KLF_ACTIVATE+KLF_NOTELLSHELL)
            cBuffer=space(8)+chr(0) 
            GetKeyboardLayoutName(cBuffer)
            cCurrentKB=cBuffer
         enddo
      else //not loaded, so we load it
         this.lUnload=true
         this.NewKeyboardHandle=;
         LoadKeyboardLayout(this.KeyboardName,;
         KLF_ACTIVATE+KLF_NOTELLSHELL)
         ActivateKeyboardLayout(this.NewKeyboardHandle,0)
      endif
   return

   function onLostFocus
      //activate what we had before
      ActivateKeyboardLayout(this.original_keyboard_handle,0)
      if this.lUnload//we loaded one, so we unload it
         UnloadKeyboardLayout(this.NewKeyboardHandle)
      endif
   return

   Function GetKeyboardArray
      //this method loads all driver into a custom array
      local  aKeyboards,cBuffer,cFirstKB,cCurrentKB
      aKeyboards=new array() 
      cBuffer=space(8)+chr(0) 

      GetKeyboardLayoutName(cBuffer)
      cFirstKB=cBuffer
      cCurrentKB=space(8)+chr(0)
      do while not cCurrentKB==cFirstKB
         ActivateKeyboardLayout(;
         HKL_NEXT,KLF_ACTIVATE+KLF_NOTELLSHELL)

         cBuffer=space(8)+chr(0)
         GetKeyboardLayoutName(cBuffer)
         cCurrentKB=cBuffer

         aKeyboards.add(cBuffer)
       enddo
   Return aKeyboards

Endclass 
/*
   --------------------------------------------------------
   Documentation on language codes ...:

   Latin 1 Script

   Identifier	Language	Sublanguage/Locale	Language Code
   0x0406	Danish	   Danish                  DAN 
   0x0413	Dutch	      Dutch (Standard)        NLD 
   0x0813	Dutch	      Belgian (Flemish)       NLB 
   0x0409	English	   American                ENU 
   0x0809	English	   British                 ENG 
   0x0c09	English	   Australian              ENA 
   0x1009	English     Canadian                ENC 
   0x1409	English     New Zealand             ENZ 
   0x1809	English     Ireland                 ENI 
   0x040b	Finnish     Finnish                 FIN 
   0x040c	French      French (Standard)       FRA 
   0x080c	French      Belgian                 FRB 
   0x0c0c	French      Canadian                FRC 
   0x100c	French      Swiss                   FRS 
   0x0407	German      German (Standard)       DEU 
   0x0807	German      Swiss                   DES 
   0x0c07	German      Austrian                DEA 
   0x040f	Icelandic   Icelandic               ISL 
   0x0410	Italian     Italian (Standard)      ITA 
   0x0810	Italian     Swiss                   ITS 
   0x0414	Norwegian   Norwegian (Bokmal)      NOR 
   0x0814	Norwegian   Norwegian (Nynorsk)     NON 
   0x0416	Portuguese  Portuguese (Brazilian)  PTB 
   0x0816	Portuguese  Portuguese (Standard)   PTG 
   0x041D	Swedish     Swedish                 SVE 
   0x040a	Spanish     Spanish (Standard/Traditional) ESP 
   0x080a	Spanish     Mexican                 ESM 
   ------------------------------------------------------
*/
// End of class LanguageEntryfield


/*
   ----------------------------------------------------------------
   End of File: ENTRY.CC
   ----------------------------------------------------------------
*/
