/*

The program creates a text file LdId.txt containing a list of the Long Names,
the Short Names and the corresponding language ID's of the BDE language drivers 
installed on the computer.

Revision History:

June 7 1999. 		Romain Streiff,
						ldList.prg published in dBase,native-tables

May 11 2003.  		Ivar B. Jessen.
						Replacd automatic creation of level 5 test table with
                  manual creation in order to overcome some problems.
                  Code expanded to filter out invalid language driver IDs.
                  Published in dBase,native-tables

February 11 2008.	Ivar B. Jessen
						Reintroduced automatic creation of test table, now level 4.
						Basis for filtering improved.

*/

close database

local cA, cLongName, cShortName, nldID, nPr, ldID, f, t 

deleteFiles()

// Get the Short and Long names of the language driver set in the BDE

cA = _app.ldriver
cShortName = substr(cA, 1, at("(", cA) - 1)
cLongName  = substr(cA, at("(", cA) + 1, len(cA) - at("(", cA) - 1)

// Get the ID of the language driver set in the BDE

nldID = makeTable(cShortName)

set console off
set alternate to ldID.txt
set alternate on

? "List of language drivers ;
included in " + version(0.89) at 15
?
?  "No." at 15, "ID" at 20, "ID as" at 26, ;
"Long name of" at 35, "Short name of" at 60
? "hex" at 26, "language driver" at 35, ;
"language driver" at 60
? "--------------------------------------;
----------------------" at 15
?

// Insert load driver iD's from 0 to 255 in table dummy4 
// and get resulting language driver names

nPr = 0
for ldID = 0 to 255
   
	try
		f = new file()
		f.open("dummy4.dbf", "W")
		f.seek(29, 0)
		f.write(chr(ldID), 1)
		f.close()
		release object f; release f

		t = new tabledef()
		t.tablename = ("dummy4.dbf")
		t.load()

   	use dummy4
		if not ((ldID = 0 or ldID > htoi(""+nldID)) and t.language = cLongName)
			nPr++
	 		? ""+nPr at 15, ""+ldID at 20, "0x" + itoh(ldID, 2) at 26, ; 
	   	t.language at 35, ldriver() at 60
		endif
   	use

		release object t; release t

	catch(DBexception d)
//		for j = 1 to d.errors.size
//	   	with d.errors[j]
//	      	? nativeCode, message, code, context
//	      endwith
//	   endfor
   	loop

	catch(exception e)
//		? e.lineno, e.message
		loop
	endtry
next ldID

use
erase "dummy4.dbf"

set alternate off
set alternate to
set console on

modi comm ldID.txt


function makeTable(clDriver)
****************************

// Create dummy version 7 table to be used for
// getting the language driver ID

local cHex7, i, cStr, f7, n, fx, cHex4, fd

cHex7 = ""
for i = 1 to 8
	cHex7 += itoh(asc(substr(cLDriver, i, 1)))
next

cStr = ""
cStr += "046A0A14000000007500050000000000"
cStr += "00000000000000000000000000000000"
cStr +=  cHex7 + "0000000000000000"
cStr += "00000000000000000000000000000000"
cStr += "00000000494400000000000000000000"
cStr += "00000000000000000000000000000000"
cStr += "00000000490400000000000000000000"
cStr += "000000000D1A"

f7 = new file() 
f7.create("dummy7.dbf")
f7.open("dummy7.dbf","RW")

for n = 1 to len(cStr) step 2
	f7.write(chr(htoi(substr(cStr,n,2))))
next n

f7.close()
release object f7

use dummy7 excl
index on ID tag ID
use

// Read language driver ID from dummy7.mdx 

fx = new file()
fx.open("dummy7.mdx", "R")
fx.seek(0x01F, 0)
ldID = itoh(asc(fx.read(1)), 2)
fx.close()
drop table dummy7

// Create dummy version 4 table to be used in loop 
// for inserting language driver ID's from 0 to 255

cHex4 = ldID

cStr = ""
cStr += "036C02090000000041000C0000000000"
cStr += "00000000000000000000000000" + cHex4 + "0000"
cStr += "49440000000000000000004E00000000"
cStr += "0B000000000000000000000000000000"
cStr += "0D1A"

fd = new file() 
fd.create("dummy4.dbf")
fd.open("dummy4.dbf","W")
for n=1 to len(cStr) step 2
		fd.write(chr(htoi(substr(cStr,n,2))))
endfor
fd.close()
release object fd; release fd

return ldID

function deleteFiles
********************

if file("dummy0.dbf")
	erase "dummy0dbf"
endif
if file("dummy7.dbf")
	erase "dummy7.dbf"
endif
if file("dummy4.dbf")
	erase "dummy4.dbf"
endif

return
//-----
