/*
     mbMonthCombobox.cc

     Author:  Mervyn Bick
     Date..:  November, 2011
     Notes.:  This custom control creates a combobox to return the month number
              for a selected month and is based on an idea suggested by
              Ivar B Jessen.

              The month name is available in the control's value property as for
              a normal combobox.

              The month number is available in the control's nMonth property

     Usage.:  An overwritten event handler in the form needs to call the onChange
              event handler in the custom control before adding code to be
              actioned by a change in selection


             function MBMONTHCOMBOBOX1_onChange
                mbMonthCombobox::onChange()
                form.entryfield1.value = this.value
                form.entryfield2.value = this.nMonth
                return

             	
              Tested with dBase Plus v2.6.0.1 under Windows XP Professional, SP3

      Returns :  Null

*/

class MBMONTHCOMBOBOX(parentObj) of COMBOBOX(parentObj) custom
   with (this)
      onOpen = class::onOpen
      onChange = class::onChange
      width = 18
   endwith


   function onOpen
      this.aMonths = new array()
      this.asMonths = new assocArray()
      for n = 1 to 12
		   this.aMonths.add(Cmonth(new date(0, n - 1, 1)))
   	   this.asMonths[this.aMonths[n]] = n
      next n
      this.datasource = "array this.aMonths"
      this.value = "Select a month"
      this.nMonth = 0
      return

   function onChange
      this.nMonth = this.asMonths[this.value]
      return
endclass



