/*
  ----------------------------------------------------------------
   MiscWSH.prg

   This program will contain Windows Scripting Host scripts 
   as written by various folk that appears to be useful. 
   I will add whatever commentary to help in using it as 
   I stumble across it ...

   Much of this code you will want to copy and paste elsewhere
   for use in your own applications ...

   WARNINGS:
      * You should check the Microsoft Help Files online and elsewhere
        for specific functions, as this explains the various parameters
        used, and what kinds of pitfalls or limitations to look out for
        when applying a function or method.

        // a LONG URL -- copy and paste this into your browser:
        http://msdn.microsoft.com/library/default.asp?url=/library/en-us/script56/html/wsoriwindowsscripthost.asp

      * Some organizations disable WSH due to security issues,
        as viruses can also use WSH. You may not be able to rely on 
        WSH being available in a deployed application.

   August, 2004

   Updated: February, 2006

   Contains:

   CopyFolder()            -- copies the contents of one folder
                              to another, and will create the target
                              if needed.
   FolderSize()            -- returns amount of space used by all 
                              files in a folder.
   GetAllDrives()          -- returns an array with all disk drives ...
   GetAllDrivesType()      -- returns a two column array with all 
                              disk drives, and their type ...
   GetFolder()             -- gives a "GetDirectory" dialog that
                              is resizable, and allows you
                              to create a directory ...
   GetCPUInfo()            -- returns an array of CPU information
   GetDesktopPath()        -- returns path to desktop folder
   GetDriveSerialNumber()  -- returns serial number of disk drive
   GetFileSize()           -- returns size of a file
   GetNetAbsolutePath()    -- returns absolute path of a network
                              mapped drive, such as "S:" might
                              be mapped to \\server\netapp
   ListNetworkProperties() -- returns an array of network info
   MapNetDrive()           -- maps a network drive to a local 
                              drive ...
   OpenExplorerAt()        -- opens the Windows Explorer at
                              the path given.
   ---------------------------------------------------------------- 
*/

msgbox( "This program is not meant to be executed, "+;
        "but contains useful WSH Scripts.","Don't run this!", 16 )
return

function ListNetworkProperties
/*
    ListNetworkProperties()
    Rich Muller
    August, 2004

    Updates: Modified by Ken Mayer to return an array, a bit of
             documentation added ...

    This function will list the domain, computer name of the
    computer it's on, the user name of the person logged on,
    and any drives that are mapped. It returns the information
    in an array, so calling it:

    aNetProps = ListNetworkProperties()

    Then you can check the properties, or do what you might
    need with them:

    ? "Domain:   "+aNetProps[1]
    ? "Computer: "+aNetProps[2]
    ? "User:     "+aNetProps[3]
    ? "Drive Mappings: "
    if aNetProps.size < 4
       ? "  ** No mapped drives **"
    else
       for i = 4 to aNetProps.size
          ? "  Drive:  "+aNetProps[i]
       next
    endif
*/
   local oDrives, aProps, WshNetwork, i

   aProps = new Array()
   WshNetwork = new OleAutoClient("WScript.Network")
   // Domain:
   aProps.add( WshNetwork.UserDomain )
   // Computer Name:
   aProps.add( WshNetwork.ComputerName )
   // User Name:
   aProps.add( WshNetwork.UserName )

   oDrives = WshNetwork.EnumNetworkDrives()
   // get the drives:
   For i = 0 to oDrives.Count() - 1 Step 2
	   aProps.add( oDrives.Item(i) + " = " + oDrives.Item(i+1) )
   endfor
return aProps

function GetDesktopPath
/*
    GetDesktopPath()
    Michael Nuwer
    August, 2004

    Updates: Modified by Ken Mayer to work as a function

    This function returns the path to the Desktop folder
    on the computer. This can be used to place a file, shortcut,
    whatever on the desktop directly.

    Usage: cPath = GetDesktopPath()
*/
   local oShell
   oShell = new OleAutoClient( "WScript.Shell" )
return oShell.SpecialFolders.Item("Desktop")


function GetAllDrives
/*
   GetAllDrives()
   Rich Muller
   October, 2004

   This returns an array listing all available drives.

   Usage: a = getAllDrives()
          for i = 1 to a.size
              ? a[i]
          endfor
*/
   local a,xx, n
   a = new array()
   xx = new OleAutoClient("Scripting.FileSystemObject").drives
   for n = 0 to xx.count -1
       a.add(xx[n].path)
   endfor
return a

function getAllDrivesType
/*
   GetAllDrivesType()
   Rich Muller
   October, 2004

   This returns a two column array listing all available 
   drives, with the drive type in the second column. Uses
   function getDriveType below.

   Usage: a = getAllDrivesType()
          for i = 1 to a.size/2
              ? a[i,1]
              ?? "  Type:" + a[i,2]
          endfor
*/

   local a,xx, n
   xx = new OleAutoClient("Scripting.FileSystemObject").drives
   a = new array(xx.count,2)
   for n = 0 to xx.count -1
      a[n+1,1] = xx[n].path
      a[n+1,2] = getDriveType(xx[n].driveType)
   endfor
return a

/*
    getDriveType
    Rich Muller
    October, 2004

    Returns a string showing the disk drive type,
    based on a numeric value passed as a parameter
    from 0 to 5 (any other number is "Unknown")
*/
function getDriveType
   parameters lnDriveType
   do case
      case lnDriveType == 0
         lcDriveType = [Unknown]
      case lnDriveType == 1
         lcDriveType = [Removable]
      case lnDriveType == 2
         lcDriveType = [Fixed]
      case lnDriveType == 3
         lcDriveType = [Network]
      case lnDriveType == 4
         lcDriveType = [CD-ROM]
      case lnDriveType == 5
         lcDriveType = [RAM-disk]
      otherwise
         lcDriveType = [Unknown]
   endcase
return lcDriveType

/*
    FolderSize()
    Ivar B. Jessen posted the basics for this in
    the dBASE newsgroups, December, 2004. 

    Ken Mayer took it and turned it into a function ...

    This little routine returns the total size of
    the files in a folder -- including all subfolders. 

    Example:

    ? FolderSize( _dBWINHome )
*/
function FolderSize( cPath )

   private fso, nSize
   fso = new oleautoclient("Scripting.FileSystemObject")

   try
      nSize = fso.GetFolder( cPath ).size
   catch( Exception E )
      msgbox( "Error: "+e.code+" -- "+e.message+chr(13)+;
              "(Make sure path is correct!)",;
              "FolderSize error", 48 )
      nSize = 0
   endtry

return nSize

/*
    GetFileSize()
    Roland Wingerter
    January, 2005

    This little routine returns the size of a file, ignoring the
    issues that dBASE' File object has with files that are > 2gigabytes
    in size.

    Example:

    ? GetFileSize( "MyLargeFile.txt" )
*/
function GetFileSize( cFile )
return new oleautoclient("Scripting.FileSystemObject").getFile(cFile).size


/*
    GetCPUInfo()
    Rich Muller/Ken Mayer
    February, 2005

    This routine gets the Processor and other info, and returns
    an array with four properties.

    Usage: set proc to :dUFLP:miscwsh
           a = GetCPUInfo()
           ? a[1] // Processor Name and information
           ? a[2] // Vendor Identifier
           ? a[3] // CPU Identifier
           ? a[4] // Processor Speed (Mhz)
*/
function GetCPUInfo
   local x, aData, cKey
   aData = new array()
   x = new OleAutoClient("WScript.Shell")
   cKey = "HKEY_LOCAL_MACHINE\HARDWARE\DESCRIPTION\System\CentralProcessor\0\ProcessorNameString"
   aData.add( x.RegRead(cKey).leftTrim() )
   cKey = "HKEY_LOCAL_MACHINE\HARDWARE\DESCRIPTION\System\CentralProcessor\0\VendorIdentifier"
   aData.add( x.RegRead(cKey).leftTrim() )
   cKey = "HKEY_LOCAL_MACHINE\HARDWARE\DESCRIPTION\System\CentralProcessor\0\Identifier"
   aData.add( x.RegRead(cKey).leftTrim() )
   cKey = "HKEY_LOCAL_MACHINE\HARDWARE\DESCRIPTION\System\CentralProcessor\0\~MHz"
   aData.add( int(x.RegRead(cKey))+" MHz" )
return aData

/*
    OpenExplorerAt()
    Ivar B. Jessen
    March, 2005

    This routine takes the path passed to it, and opens the Windows
    Explorer at that path:

       OpenExplorerAt( _dbwinhome+"\bin" )
          or
       OpenExplorerAt( getdir() )
          or
       cPath = getdir()
       OpenExplorerAt( cPath )
*/
function OpenExplorerAt( cPath )
   local sh
   sh = new oleAutoClient( "Shell.Application" )
   sh.Explore( cPath )
return


/*
    GetNetAbsoluteDrive()
    J.P. Martel
    March 25, 2005

    This routine will get the network absolute drive for 
    a drive that has been mapped, such as:
        S:
    Which might be mapped to:
        \\ServerName\NetApp

    If the drive is not mapped, and you pass a value such as
        F:
    This will return:
        F:\

    Usage: 
        ? GetNetAbsoluteDrive( "S:" )
*/
function GetNetAbsoluteDrive( cDrive )
   local oScript
   oScript = new OLEAutoClient( "scripting.FileSystemObject" )
return oScript.getAbsolutePathName( cDrive )

/*
   GetFolder()
   Posted by Rich Muller in the dBASE Newsgroups, May 12, 2005
   Modified by Ken Mayer a tad.

   This brings up the Windows BrowseForFolder dialog, which
   is resizable and allows the user to create a folder ...
   This does not select files, but selects folders ...

   Usage:
      getFolder( [<cTitle>] )
   Example:
      set proc to :dUFLP:MiscWsh additive
      cFolder = getFolder( "Select the folder to write the document to ..." )
*/
function getFolder( cTitle )
   private oShell, oFolder, cPath
   oShell = new OleAutoClient( "Shell.Application" )
   if empty( cTitle )
      cTitle = "Choose a Directory"
   endif
   oFolder = oShell.BrowseForFolder( _app.Framewin.hwnd, cTitle,;
                                     0x20+0x08+0x01, "" )
   cPath = null
   if type( "oFolder.Items().Item().path" ) == "C"
      cPath = oFolder.Items().Item().path
   endif
return cPath

/*
   GetDriveSerialNumber()
   Posted by Ivar B. Jessen in the dBASE Newsgroups, February 20, 2006
   Modified by Ken Mayer to accept a parameter for the drive, and
   correct information where possible.

   Usage:
      getDriveSerialNumber( [<cDrive>] )
   Example:
      set proc to :dUFLP:MiscWsh additive
      nNumber = getDriveSerialNumber( "C" )
   Returns:
      numeric value of serial number, rather than hexidecimal
      returns zero if drive not found ... (rather than null)
*/
function GetDriveSerialNumber( cDrive )
   private fso 
   fso = new OleAutoclient( "Scripting.FileSystemObject" )
   // only use first character of parameter
   if len( cDrive ) > 1
      cDrive = left( cDrive, 1 )
   endif
   // add colon
   cDrive = cDrive+":"

   // get serialnumber
   nSerial = fso.getDrive( cDrive ).serialNumber
   // if null, we return zero
   if nSerial == null
      nSerial := 0
   endif
   // cleanup
   fso = null
return nSerial


function MapNetDrive( cDrive, cPath )
/*
    MapNetDrive()
    Programmer: Rich Muller
    Date......: April, 2007
    Notes.....: Maps a network drive to the drive letter
                you specify. If you need a password, use
                the routine ConnectNetDrive() in MiscAPI.prg
                instead ...
    Usage.....: MapNetDrive( "F:", "\\SomeServer\SomeDrive" )
*/

   local oNet, nReturn
   oNet = new oleAutoClient("WScript.Network")
   nReturn = oNet.MapNetworkDrive( cDrive, cPath )

return nReturn

/*
   -----------------------------------------------------------------------
   End of File: MISCWSH.PRG
   ----------------------------------------------------------------------- 
*/


