/*
   ----------------------------------------------------------------
   Program...:PACKTABLES.PRG
   Programmer. Ken Mayer
   Date....... January 26, 1999

   This is a simple routine to scan through a list of .DBF tables
   in the directory or database and pack them (remove records 
   marked for deletion).

   If using a database, you need access to pathdata.cc,
   and must provide the database name:
      do packtables with "databaseName"
   otherwise:
      do packtables

   Updates ....: December, 2002
                 Rich (?) found a minor glitch, if you enter
                 a path in the BDE with a trailing backslash,
                 the code here would add a second backslash,
                 causing problems. This has been fixed.
                 November, 2006
                 Added call to _app.executeMessages() for
                 those large databases ...
     
   ** Note that this program uses Source Aliasing, specifically the
   ** dUFLP source Alias. If you do not know how to set up
   ** a Source Alias, see the instructions in WHATS.NEW at
   ** the top of the file, or in README.TXT.
   ** Any files referenced by :dUFLP:filename  must be
   ** included in an executable if you are building 
   ** one ... make sure that your project includes these
   ** files.
   **
   ** This particular program uses:
   **           Message.wfm
   **           PathData.cc

   ----------------------------------------------------------------
*/

parameter cDatabase // optional parameter
close database // just to be safe (xBase command, but ...)

// create array table
aTables = new Array()

// if we're using a database, we need the info
// a bit differently:
lDatabase = false
if argCount() > 0
   lDatabase := true
   set procedure to :dUFLP:pathdata.cc additive
   try
      dMyData = new pathDatabase()
      dMyData.databaseName := cDatabase
      dMyData.active := true
   catch (exception e)
      msgbox( "BDE Alias error: "+e.code+", "+e.message, ;
              "Problem!", 16 )
      dMyData.active := false
      return
   endtry
   // if we're here, all is well ...
   aTables.dirExt( dMyData.path+;
                   iif( right( dMyData.path, 1 ) == "\", "", "\")+;
                   "*.dbf" )
else
   // we're NOT using a database, so check the current directory:
   aTables.dirExt( "*.dbf" )
   dMyData = _app.databases[1]
endif

// check to see if we're wasting our time:
if aLen( aTables, 1 ) = 0
   msgbox( "There are no .DBF tables in this directory!",;
           "Nothing to do", 64)
   return
endif

// resize the array so it has only one column,
// that being the names of the tables:
nRows = aLen( aTables,1 )
aTables.resize( nRows, 1, 1)
// sort it ...
aTables.sort()

// open a message dialog to show user something's
// happening:
set procedure to :dUFLP:message.wfm additive
fMessage = new MessageForm()
fMessage.text         := "Processing tables"
fMessage.title.text   := "Packing Tables"
fMessage.message.text := aTables[ 1 ].toUpperCase()
fMessage.moveable     := true
fMessage.topMost      := false
fMessage.Open()

// now, do it!
for tableNumber = 1 to aLen( aTables, 1 )
    fMessage.message.text = upper( aTables[ tableNumber ] )
    if __version__ => 2.5
       _app.executeMessages()
    endif
    try
       dMyData.packTable( aTables[ tableNumber ] )
    catch( exception e )
       // if here, chances are we couldn't open the 
       // table exclusively for some reason ...
       msgbox( 'Error: '+e.code+chr(13)+e.message )
    endtry
next

// cleanup
fMessage.release()
if lDatabase
   dMyData.active := false
   release object dMyData
   dMyData = null
endif
close procedure :dUFLP:pathdata.cc
close procedure :dUFLP:message.wfm

RETURN
/*
   End of Program: PackTables.prg
*/


