/* ----------------------------------------------------------------------
    Description:
       This is a first try at creating an command interpreter that
       handles control structures without resorting to codeblocks.

       A more complete explaination can be seen in the dBASE
       Developers Bulletin article entitled: "Finally @say...get!"

       Note that you can find the dBASE Developer's Bulletin (a
       free newsletter with a lot of useful information on 
       dBASE in it) at any of the following websites:

         Canada        http://pages.infinit.net/jpmartel/bulletin.html
         France        http://www.fghoche.com/dbulletin/bulletin.html
         Germany       http://www.tnm.de/dBulletin/bulletin.html
         Italy         http://www.intrasoft.it/db2kit
         New Zealand   http://www.compkarori.com/dbase/1/bulletin.html
         South Africa  http://www.dbase.co.za/resource/dbulletin/bulletin.html
         Sweden        http://www.sakia.se/dbulletin/bulletin.html
         U.S.A.        http://www.staubassociates.com/dbase/bulletin.html

       Each line is handled one at a time, so there should never be
       a line limitation or macro expansion problem.

       Support for ElseIf is not included because the Do Case is
       equivilant, but support could be added.

       This program can allow a qualified user to add programs and
       functionality compiled programs.  This is useful in creating
       customized reports, data entry screens, and virtually any other
       functionality.  This capability is key if you want to create a
       standard .exe that is distributed to a variety of clients but you
       also want each clients application to be customized.  


    Programmers: 
 
       George Burt
       Includes Stack.cc written by Ken Mayer
       Borrows some code from Romain Strieff's NEWMacro.HOW
 
    History: 
 
       06/29/1998 -- original version test version
       08/24/1998 -- Fixed a problem with DO WHILE LOOPS
       08/01/1999 -- Added Debug flag to aid in finding problems in scripts 
       08/12/1999 -- Added @ say and Get capabilities for scripts
 
    Instantiation: 
       set procedure to terpret.prg additive
       terpret(macrofile,callingObject,event)
       if callingObject is a form, the script will immediately 
       execute.  for example:
          form1.pushbutton1.onclick = {;terpret("somescript.scp",form)}
       will execute the somescript.scp file as soon as the pushbutton is clicked.


 
    Methods:
       None

    Properties:
       None
   -------------------------------------------------------------------- 

   This program interprets "Eraseme.prg".  "Eraseme.prg"  displays 12 
   consequtive msgboxes; each with the next number in it.  As you
   display them, they should contain the next number (i.e. 1,2,3,4, etc.)

   This demonstates that all the loops and control structures are 
   working as designed.

   */

*   Program Terpret
   *-------------------------------------------------------------------
   *-- Programmer..: George Burt
   *-- Date........: 06/29/1998
   *-- Notes.......: Opens and interprets "eraseme.prg"
   *-- Written for.: Visual dBASE 7.01
   *-- Rev. History: 06/29/1998 -- Original Code
   *-- Rev. History: 09/29/1998 -- added @ Say...Get, Scripts passed via array
   *-- Calls.......: none
   *-- Usage.......: do terpret.prg
   *-- Example.....: same
   *-- Returns.....: Nothing
   *-- Parameters..: None
   *-------------------------------------------------------------------

function terpret

parameter MmacroFile,McallingObject,Mevent,Mdebug
local mcount,aexec

if type("MmacroFile")= 'A'
   MTerArray = .t.
else
   MTerArray = .f.
endif
TheGetForm =''
if type("Mdebug") = 'U' .or. empty(Mdebug)
   Mdebug = .f.
else
   Mdebug = .t.
endif
if type("Mevent") = 'U' .or. empty(Mevent)
   Mevent = ""
endif
tthis = McallingObject
if TYPE("tthis.PARENT")='U'  && THIS TEST FOR FOR TTHIS = TO A FORM
   fform = tthis
else
   fform = tthis.form
endif
if type('fform.geoContinue') = 'U'
   Mcontinue = .f.
else
   Mcontinue = fform.geoContinue
endif
if .not. Mcontinue
   Mtime= time()
   *--- The following "compiles" the script
   set proced to stack.cc addi
   TheStack = new stack()
   if MTerArray  && This means an array of commands passed to Terpret instead of a file name
   else
      nHandle=fopen(MmacroFile,"R")     &&open file with low level functions
   endif
   aExec=new array()                &&create array to store commands
   MterCount = 1
   if iif(MTerArray,.t.,nHandle>0)                     &&check valid opening of file
      MoptimizeLine = .f.
      do while iif(MterArray,MmacroFile.size>=MterCount,.not. feof(nHandle))  &&loop through file
         if MterArray
            cSTring = MmacroFile[MterCount]
            MterCount++
         else
            cString=fgets(nHandle)&&read 1 line to cString property
         endif
         if (.not. (left(ltrim(cString),1)="*" .or. left(ltrim(cstring),2)="//"));
               .and. ;
            (.not. len(trim(cString))=0)
            McString = ltrim(rtrim(cString))
            do case
               case upper(substr(McString,1,1)) = '@'
                  Mline = aExec.size+1
                  MFlowCommand = '@         '
                  MExpression = substr(McString,2,2000)
                  aExec.add("*"+tran(mline,'999999999')+MflowCommand+Mexpression)
               case upper(substr(McString,1,4)) = 'READ'
                  Mline = aExec.size+1
                  MFlowCommand = '@         '
                  aExec.add("*"+tran(mline,'999999999')+"READ")
               case upper(substr(McString,1,3)) = 'IF '
                  Mline = aExec.size+1
                  MFlowCommand = 'IF        '
                  MExpression = substr(McString,4,2000)
                  aExec.add("*"+tran(mline,'999999999')+MflowCommand+Mexpression)
                  TheStack.push(tran(mline,'9999999999')+MflowCommand)
               case upper(substr(McString,1,7)) = 'DO WHIL'
                  Mline = aExec.size+1
                  MlastDoWhileLine = Mline
                  MFlowCommand = 'DO WHILE  '
                  *--- Next two lines parse the expression
                  m32ztemp= substr(McString,4,2000)  
                  MExpression = substr(m32ztemp,at(' ',m32ztemp)+1,2000)
                  aExec.add("*"+tran(mline,'999999999')+MflowCommand+Mexpression)
                  TheStack.push(tran(mline,'9999999999')+MflowCommand)
               case upper(substr(McString,1,7)) = 'DO CASE'
                  Mline = aExec.size+1
                  MFlowCommand = 'DO CASE   '
                  *--- Next two lines parse the expression
                  m32ztemp= substr(McString,4,2000)  
                  MExpression = ""
                  aExec.add("*"+tran(mline,'999999999')+MflowCommand+Mexpression)
                  TheStack.push(tran(mline,'9999999999')+MflowCommand)
               case upper(substr(McString,1,9)) = 'OTHERWISE'
                  Mline = aExec.size+1
                  MFlowCommand = 'OTHERWISE   '
                  MStackLine = TheStack.getTop()
                  MoldLine = val(substr(MstackLine,1,10))
                  aExec[MoldLine] = "*"+tran(mLine,'999999999')+substr(aExec[MoldLine],11,2000)
                  MExpression = ""
                  aExec.add("*"+tran(mline,'999999999')+MflowCommand+Mexpression)
                  TheStack.pop()
                  TheStack.push(tran(mline,'9999999999')+MflowCommand)
               case upper(substr(McString,1,5)) = 'CASE'
                  Mline = aExec.size+1
                  MFlowCommand = 'CASE      '
                  MStackLine = TheStack.getTop()
                  MoldLine = val(substr(MstackLine,1,10))
                  aExec[MoldLine] = "*"+tran(mLine,'999999999')+substr(aExec[MoldLine],11,2000)
                  *--- Next two lines parse the expression
                  m32ztemp= substr(McString,4,2000)  
                  MExpression = substr(m32ztemp,at(' ',m32ztemp)+1,2000)
                  aExec.add("*"+tran(mline,'999999999')+MflowCommand+Mexpression)
                  TheStack.pop()
                  TheStack.push(tran(mline,'9999999999')+MflowCommand)
               case upper(substr(McString,1,5)) = 'ENDCA'
                  Mline = aExec.size+1
                  MFlowCommand = 'ENDCASE   '
                  MStackLine = TheStack.getTop()
                  MoldLine = val(substr(MstackLine,1,10))
                  *--- Changes Line number of previous flow control
                  aExec[MoldLine] = "*"+tran(mLine,'999999999')+substr(aExec[MoldLine],11,2000)
                  *--- Next two lines parse the expression
                  Mexpression = ""
                  aExec.add("*"+tran(mline,'999999999')+MflowCommand+Mexpression)
                  TheStack.pop()
               case upper(substr(McString,1,5)) = 'ENDIF'
                  Mline = aExec.size+1
                  MFlowCommand = 'ENDIF     '
                  MStackLine = TheStack.getTop()
                  MoldLine = val(substr(MstackLine,1,10))
                  *--- Changes Line number of previous flow control
                  *--- Next two lines parse the expression
                  MoldLine = val(substr(MstackLine,1,10))
                  *--- Changes Line number of previous flow control
                  aExec[MoldLine] = "*"+tran(mLine,'999999999')+substr(aExec[MoldLine],11,2000)
                  Mexpression = ""
                  aExec.add("*"+tran(mline,'999999999')+MflowCommand+Mexpression)
                  TheStack.pop()
               case upper(substr(McString,1,5)) = 'ENDDO'
                  Mline = aExec.size+1
                  MFlowCommand = 'ENDDO     '
                  MStackLine = TheStack.getTop()
                  MoldLine = val(substr(MstackLine,1,10))
                  *--- Changes Line number of previous flow control
                  aExec[MoldLine] = "*"+tran(mLine,'999999999')+substr(aExec[MoldLine],11,2000)
                  MExpression = ""
                  aExec.add("*"+tran(moldline,'999999999')+MflowCommand+Mexpression)
                  TheStack.pop()
               case upper(substr(McString,1,6)) = 'ELSEIF'
                  Mline = aExec.size+1
                  MFlowCommand = 'ELSEIF    '
                  MStackLine = TheStack.getTop()
                  MoldLine = val(substr(MstackLine,1,10))
                  *--- Next two lines parse the expression
                  m32ztemp= substr(McString,4,2000)  
                  MExpression = substr(m32ztemp,at(' ',m32ztemp)+1,2000)
                  *--- Changes Line number of previous flow control
                  aExec[MoldLine] = "*"+tran(mLine,'999999999')+substr(aExec[MoldLine],11,2000)
                  MExpression = ""
                  aExec.add("*"+tran(moldline,'999999999')+MflowCommand+Mexpression)
               case upper(substr(McString,1,4)) = 'ELSE'
                  Mline = aExec.size+1
                  MFlowCommand = 'ELSE      '
                  MStackLine = TheStack.getTop()
                  MoldLine = val(substr(MstackLine,1,10))
                  *--- Next two lines parse the expression
                  MExpression = ""
                  *--- Changes Line number of previous flow control
                  aExec[MoldLine] = "*"+tran(mLine,'999999999')+substr(aExec[MoldLine],11,2000)
                  MExpression = ""
                  aExec.add("*"+tran(moldline,'999999999')+MflowCommand+Mexpression)
                  TheStack.pop()
                  TheStack.push(tran(mline,'9999999999')+MflowCommand)
               case upper(substr(McString,1,4)) = 'LOOP'
                  Mline = aExec.size+1
                  MFlowCommand = 'LOOP      '
                  MStackLine = TheStack.getTop()
                  MoldLine = val(substr(MstackLine,1,10))
                  *--- Next two lines parse the expression
                  MExpression = ""
                  *--- Changes Line number of previous flow control
                  aExec[MoldLine] = "*"+tran(mLastDoWhileLine,'999999999')+substr(aExec[MoldLine],11,2000)
                  MExpression = ""
                  aExec.add("*"+tran(moldline,'999999999')+MflowCommand+Mexpression)
               case upper(substr(McString,1,4)) = 'EXIT'
                  Mline = aExec.size+1
                  MFlowCommand = 'EXIT      '
                  MStackLine = TheStack.getTop()
                  MoldLine = val(substr(MstackLine,1,10))
                  *--- Next two lines parse the expression
                  MExpression = ""
                  *--- Changes Line number of previous flow control
                  aExec[MoldLine] = "*"+tran(mLastDoWhileLine,'999999999')+substr(aExec[MoldLine],11,2000)
                  MExpression = ""
                  aExec.add("*"+tran(moldline,'999999999')+MflowCommand+Mexpression)
               otherwise
                  aExec.add(cString)
            endcase
         endif
      enddo
      if MterArray
      else
         fclose(nHandle)
      endif
   else
      msgbox("Could not open FILE!")
   endif
*   CLEAR
   if Mdebug
      atemp = new array()
      for i = 1 to aexec.size
         atemp.add(tran(i,'9999')+") "+aexec[i])
      next
      set proced to debug.wfm addi
      _app.debug = new debugform()
      _app.debug.listbox1.datasource = 'array atemp'
      _app.debug.open()
   endif
   
   Mtime = time()

   *--- if Mprepend is true, run original event
   m32ztemp = "tthis."+Mevent+"prepend"
   if type(m32ztemp)<>'U'
      if &m32ztemp.
         m32ztemp = "tthis."+Mevent+"prepend"
         Mappend = &m32ztemp.
         if lower(Mevent) <> 'onopen'
            m32ztemp = "tthis.orig"+Mevent+"()"
            &m32ztemp.
         endif
      endif
   endif

   *--- This runs the script
   Mcount = 1
else
   Aexec = fform.GeoAexec
   Mcount = fform.geocount
*   fform.greadcontinueButton.release()
   fform.GeoMmacroFile = null
   fform.GeoMcallingObject = null
   fform.GeoMevent = null
   fform.GeoMdebug = null
   fform.GeoAExec = null
   fform.GeoCount = null
   fform.GeoContinue = .f.
   restore from eraseme
endif
McaseFinish = .f.
do while Mcount <= aExec.size
   if left(aExec[Mcount],1) = '*'
      do case
         case substr(aExec[Mcount],11,4) = 'READ'
            set cuaenter off
            TheGetForm.GReadContinueButton = new pushbutton(TheGetForm)
            TheGetForm.GReadContinueButton.top = 25
            TheGetForm.greadcontinueButton.visible = .t.
            MgetCount = agetarray.size
            agetobj[1].setfocus()
            agetObj[MgetCount].nextObj = TheGetForm.greadcontinuebutton
            TheGetForm.GeoMmacroFile = MmacroFile
            TheGetForm.GeoMcallingObject = McallingObject
            TheGetForm.GeoMevent = Mevent
            TheGetForm.GeoMdebug = Mdebug
            TheGetForm.GeoAExec = aExec
            TheGetForm.GeoCount = Mcount+1
            TheGetForm.GeoContinue = .t.
            TheGetForm.mdi = .f.
*            save to eraseme
            TheGetForm.greadcontinueButton.ongotfocus = {;TheGetForm.close()}
            TheGetForm.readmodal()
            AsayRow = null
            AsayCol = null
            AsayArray = null
            AgetArray = null
            TheGetForm = null
            Mcount++
         case substr(aExec[Mcount],11,1) = '@'
            Mcommand = substr(aExec[Mcount],21,2000)
            m32ztemp = Mcommand
            McommaPos = at(",",m32ztemp)
            m32ztemp1 = substr(m32ztemp,1,McommaPos-1)
            Mrow = &m32ztemp1.
            MsayPos = AT(' SAY ',upper(m32ztemp))
            MsayPos = iif(MsayPos=0,999,MsayPos)
            MgetPos = AT(' GET ',upper(m32ztemp))
            MgetPos = iif(MgetPos=0,999,MgetPos)
            MtoPos  =AT(' TO ',upper(m32ztemp))
            MtoPos = iif(MtoPos=0,999,MtoPos)
            do case
               case (MsayPos < MgetPos) .and. (MsayPos <MtoPos)
                  // Reserved word SAY comes first
                  if type("asayArray") <> 'A'
                     AsayRow = new array()
                     AsayCol = new array()
                     AsayArray = new array()
                  endif
                  AsayRow.add(Mrow)
                  m32ztemp1 = substr(m32ztemp,McommaPos+1,2000)
                  Mcol = substr(m32ztemp1,1,at(' SAY ',upper(m32ztemp1))-1)
                  Mcol = &Mcol.
                  AsayCol.add(Mcol)
                  if at(' GET ',upper(m32ztemp1))>0
                     // Get present but could be in a 'say' quote
                     MgetPos = rat(" GET ",upper(m32ztemp1))
                     Msingle = rat("'",m32ztemp1)
                     Mdouble = rat('"',m32ztemp1)
                     MlastQuote = iif(Msingle>Mdouble,Msingle,Mdouble)
                     if MlastQuote > MgetPos
                        // Get is inside a quote
                        m32ztemp2 = substr(m32ztemp1,at(" SAY ",upper(m32ztemp1))+5,2000)
                        AsayArray.add(&m32ztemp2.)
                        showSayScreen()
                     else
                        // we have a live Get.  Now we can
                        // resolve the end of the say
                        m32ztemp2 = substr(m32ztemp1,at(" SAY ",upper(m32ztemp1))+5,MgetPos)
                        m32ztemp2 = substr(m32ztemp2,1,rat(" GET ",upper(m32ztemp2)))
                        
                        AsayArray.add(&m32ztemp2.)
                        showSayScreen()
                        // Now we handle the get
                        if type("agetArray") <>'A'
                           AgetRow = new array()
                           AgetCol = new array()
                           AgetArray = new array()
                           AgetObj = new array()
                        endif
                        AgetRow.add(Mrow)
                        AgetCol.add(Mcol+len(&m32ztemp2.))
                        AgetArray.add(substr(m32ztemp1,rat(" GET ",upper(m32ztemp1))+5,2000))
                        AgetObj.add("")
                        m32ztemp = new entryfield(TheGetform)
                        agetObj[agetObj.size] = m32ztemp
                        m32ztemp.name = "getfield"+ltrim(str(agetobj.size))
                        m32ztemp.fontname = "Courier New"
                        m32ztemp.fontsize = 10
                        m32ztemp.borderStyle = 3	// None
                        m32ztemp.height = .65
                        m32ztemp.top = (Mrow*.73)
                        m32ztemp.left = (agetCol[agetcol.size]*1.14)+1.14
                        m32ztemp1 = agetarray[agetarray.size]
                        ?m32ztemp1,aexec.size
                        m32ztemp.width = (len("" + &m32ztemp1.)*1.14)+1.14
                        m32ztemp.value = &m32ztemp1.
                        m32ztemp3 = "{;"+m32ztemp1+"=this.value}"
                        m32ztemp.onchange = &m32ztemp3.
                     endif
                  else
                     // No get
                     m32ztemp2 = substr(m32ztemp1,at(" SAY ",upper(m32ztemp1))+5,2000)
                     AsayArray.add(&m32ztemp2.)
                     showSayScreen()
                  endif

               case (MgetPos < MtoPos)
                  // Reserved word GET comes first and no SAY present
               case MtoPos <> 999
                  // Reserved word TO must be used
               otherwise
                  msgbox("Error: "+Mcommand)
                  Mcount++
                  loop
            endcase
            Mcount++

         case substr(aExec[Mcount],11,10) = 'IF        '
            Mcommand = substr(aExec[Mcount],21,2000)
            if &Mcommand.
               Mcount++
            else
*               ?"changed here?",Mcount,aexec[Mcount]
               Mcount = val(substr(Aexec[Mcount],2,9))+1
*               ?"jumping to line ",Mcount
            endif
         case substr(aExec[Mcount],11,10) = 'DO WHILE  '
            Mcommand = substr(aExec[Mcount],21,2000)
            if &Mcommand.
               Mcount++
            else
*               ?"changed here?",Mcount,aexec[Mcount]
               Mcount = val(substr(Aexec[Mcount],2,9))+1
*               ?"jumping to line ",Mcount
            endif
         case substr(aExec[Mcount],11,10) = 'DO CASE   '
            McaseFinish = .f.
            Mcount++
         case substr(aExec[Mcount],11,10) = 'OTHERWISE '
            if McaseFinish
               Mcount  = val(substr(Aexec[Mcount],2,9))+1
               McaseFinish = .f.
               loop
            endif
            Mcount++
         case substr(aExec[Mcount],11,10) = 'CASE      '
            if McaseFinish
               Mcount = val(substr(Aexec[Mcount],2,9))
               loop
            endif
            Mcommand = substr(aExec[Mcount],21,2000)
            if &Mcommand.
               McaseFinish = .t.
               Mcount++
            else
               McaseFinish = .f.
               Mcount = val(substr(Aexec[Mcount],2,9))
            endif
         case substr(aExec[Mcount],11,10) = 'ENDCASE   '
            McaseFinish = .f.
            Mcount++
         case substr(aExec[Mcount],11,10) = 'ENDIF     '
            Mcount++
         case substr(aExec[Mcount],11,10) = 'ENDDO     '
            Mcount = val(substr(Aexec[Mcount],2,9))
         case substr(aExec[Mcount],11,10) = 'ELSEIF    '
            msgbox("ElseIf is Not allowed")
            exit
         case substr(aExec[Mcount],11,10) = 'ELSE      '
            Mcount = val(substr(Aexec[Mcount],2,9))+1
         case substr(aExec[Mcount],11,10) = 'LOOP      '
            do while .t.
               Mcount++
               if left(aExec[Mcount],1) = '*'
                  if substr(aExec[Mcount],11,10) = 'ENDDO     '
                     Mcount =val(substr(Aexec[Mcount],2,9))
                     exit
                  endif
               endif
            enddo
         case substr(aExec[Mcount],11,10) = 'EXIT      '
            do while .t.
               Mcount++
               if left(aExec[Mcount],1) = '*'
                  if substr(aExec[Mcount],11,10) = 'ENDDO     '
                     Mcount++
                     exit
                  endif
               endif
            enddo
         otherwise
            Mcount++
      endcase
   else
      command=aExec[Mcount]
      if upper(substr(command,1,6)) = 'RETURN'
         exit
      endif
      if at(':::',command)>0  && ::: indicates macro substitution
         MgeoMacbeg = at(":::",command)
         MgeoMacend = at(";;;",command)
         MgeoPart1 = substr(command,1,MgeoMacBeg-1)
         MgeoMacro = substr(command,MgeoMacBeg+3,MgeoMacEnd-MgeoMacBeg-3)
         MgeoAnswer = &MgeoMacro.
         MgeoPart2 = substr(command,MgeoMacEnd+3,2000)
         command = MgeoPart1+MgeoAnswer+MgeoPart2
      endif
*      ?mcount,command
      try
         &command.
      catch (exception e)
         msgbox("Error: Line "+ltrim(str(Mcount))+chr(13)+e.message+chr(13)+command,"Quiting")
         return
      endtry
      Mcount++
   endif
enddo

m32ztemp = "tthis."+Mevent+"append"
if type(m32ztemp)<>'U'
   if &m32ztemp.
      m32ztemp = "tthis."+Mevent+"append"
      Mappend = &m32ztemp.
      if lower(Mevent) <> 'onopen'
         m32ztemp = "tthis.orig"+Mevent+"()"
         &m32ztemp.
      endif
   endif
endif


procedure showSayScreen


if type("TheGetForm") <> 'O'
   TheGetForm = new form()
   TheGetForm.width = 91.43
   TheGetForm.height = 18.18
   TheGetForm.autocenter = .t.
endif
if type("TheGetForm.GsayPage") = 'U'
   MGsayPage = 1
else
   MgsayPage = TheGetForm.gsaypage
endif

if type("TheGetForm.backscreen") = "O"
else
   TheGetForm.backscreen = new text(TheGetForm)
   TheGetForm.backscreen.fontName = "Courier New"
   TheGetForm.backscreen.top = 0
   TheGetForm.backscreen.left = 0
   TheGetForm.backscreen.width = 91.43
   TheGetForm.backscreen.height = 18.18
   TheGetForm.backscreen.wrap = false
   TheGetForm.backscreen.pageno = Mgsaypage
endif
*clear
Mbackscreen = space(2000)
for i = 1 to Asayarray.size
   MbegPos = ((asayrow[i])*80)+asaycol[i]
   Mvalue = AsayArray[i]
   Mbackscreen = substr(MbackScreen,1,Mbegpos)+Mvalue+substr(MbackScreen,MbegPos+len(Mvalue),2000)
next
m32ztemp = MbackScreen
MbackScreen = ''
for i = 1 to 25
   MbackScreen+="<p>"+substr(m32ztemp,((i-1)*80)+1,80)+"</p>"
next
TheGetForm.backscreen.text = Mbackscreen
return


