//--------------------------------------------------------------
//
//  VStruct.prg (and VStruct.h)
//
//  This class and header file enables one-time runtime
//  declaration of a structure, which can then be instantiated
//  as an object. The structure members are then accessed by
//  name using get and set methods.
//
//  Structure definitions are stored in a class static variable
//  that can be cleared with CLEAR MEMORY.
//
//  Examples shown in test code.
//
//  Dependencies:
//
//      VSTRUCT.H, which uses
//          STRUCTAPI.H (included with product)
//
//  Author:
//
//      Ken Chan
//
//  Peer-supported newsgroups at:
//
//      news://news.dbase.com/dbase.programming
//
//  Revision history:
//  
//  v1.00   28-Oct-97   Warning: (almost) no bounds checking!
//   1.01   15-Dec-97   Added this doc header
//                      Fixed typo in TYPE("GetClientRect") test
//                      Copied valType() function to this file
//                        instead of using copy in function lib
//                      Use -= to get signed char and short
//   1.10   19-Dec-97   Added boolean support
//                      Make string members null-terminated
//                      Added duplicate member check
//   1.11   24-Dec-97   Accurate getSize()
//   1.12   09-Mar-00   Specify ITOH() length in getHexChars()
//                      Add getBinaryChars()
//
//--------------------------------------------------------------
#include <VSTRUCT.H>

// Various test routines
VSTRUCT_DEF( "TEST", ;
   "NUM1", TYPE_SHORT, ;
   "STR" , TYPE_STRING(10), ;
   "NUM2", TYPE_DOUBLE ;
)

t = new VStruct( "TEST" )
t.setMem( "NUM1", -10000 )
? t.getMem( "NUM1" )  // should be -10000
? t.getShort( 0 )     // should be 55536
t.setMem( "NUM1", 45536 )
? t.getMem( "NUM1" )  // should be -20000
t.setMem( "STR", "dog" )
? t.getMem( "STR" )   // should be "dog"
? t.getMem( "NUM2" )  // should be zero
//return

? "RECT is", iif( VSTRUCT_ISDEF( "RECT" ), "already", "not" ), "registered"

VSTRUCT_DEF( "RECT", ;
   "LEFT"  , TYPE_LONG, ;
   "TOP"   , TYPE_LONG, ;
   "RIGHT" , TYPE_LONG, ;
   "BOTTOM", TYPE_LONG ;
)

o = new Object()
o.c = new VStruct( "RECT" )

o.c.setMem( "LEFT", 1000 )
? o.c.getMem( "LEFT" )
o.c.setString( 0, "abcdefghijklmnop" )
? o.c.getHexChars( 0, 16 )

if type( "GetClientRect" ) # "FP"
   extern clogical GetClientRect( chandle, cptr ) user32
endif
f = new form()
f.open()
? getclientrect( f.hwndclient, o.c )
? o.c.getMem( "LEFT" ), o.c.getMem( "TOP" ), o.c.getMem( "RIGHT" ), o.c.getMem( "BOTTOM" )
f.close()
//return

nDecimals = set( "DECIMALS" )
set decimals to 18
d = new VStruct( 10 )
d.setLong( 4, 0x4142AF46 )  // store double as two longs
d.setLong( 0, 0x80000000 )
v = d.getDouble( 0 )
d.setDouble( 0, v )
? d.getDouble( 0 )  // should be 2449037
d.setLong( 4, 0x3fd55555 )  // store double as two longs
d.setLong( 0, 0x55555555 )
v = d.getDouble( 0 )
d.setDouble( 0, v )
? d.getDouble( 0 )  // should be 1/3
d.setLDouble( 0, 1 )
? d.getLDouble( 0 ) // should be 1
d.setShort( 8, 0x3ffd )  // store long double as short and two longs (10 bytes)
d.setLong( 4, 0xaaaaaaaa )
d.setLong( 0, 0xaaaaaaab )
? d.getHexChars( 0, 10, true )
d.setHexChars( 0, "3ffdaaaaaaaaaaaaaaab", true )
? d.getLDouble( 0 )  // should be 1/3
d.setLDouble( 0, -0.0000123456789 )
? d.getLDouble( 0 )  // should match
d.setLDouble( 0, -23456789012345678 )
? d.getLDouble( 0 )  // these
? -23456789012345678 // two should match and show significant digits lost
set decimals to nDecimals
return

// Reference count tests for nesting objects/arrays
x = o.c.structDef; ? "refcount is", refcount( x )
d = new VStruct( "RECT" )
x = o.c.structDef; ? "refcount is", refcount( x )
release d
x = o.c.structDef; ? "refcount is", refcount( x )
VSTRUCT_DEF( "RECT", ;
   "LEFT"  , TYPE_LONG, ;
   "TOP"   , TYPE_LONG, ;
   "RIGHT" , TYPE_LONG, ;
   "BOTTOM", TYPE_LONG ;
)
? "refcount is", refcount( x )
return


class VStruct( xDef ) of String

   // Hide all String properties and methods
   protect length, string
   protect anchor, asc, big, blink, bold, charAt, chr, fixed, fontColor, fontSize
   protect indexOf, isAlpha, isLower, isUpper, italics, lastIndexOf, left, leftTrim, link
   protect replicate, right, rightTrim, small, space, strike, stuff, sub, substring, sup
   protect toLowerCase, toProperCase, toUpperCase
   protect getByte, setByte, isDBCS, toDBCS, toSBCS

   // Class static structure registry, and protected accessor codeblocks
   static vStructRegistry = new AssocArray()
   protect setStructReg, getStructReg
   this.setStructReg = {|cName,aStructDef|; vStructRegistry[ upper( cName ) ] = new VStructDef( cName, aStructDef ) }
   this.getStructReg = {|cName| iif( vStructRegistry.isKey( upper( cName ) ), vStructRegistry[ upper( cName ) ], null )}

   // Identifies instance of pre-defined structure
   this.structName = ""
   protect structDef
   this.structDef  = null

   // Hide dynamic member accessor methods (codeblocks) and table methods
   protect getMethod, setMethod, getMethodTable, setMethodTable

   if argcount() >= 1
      if valType( xDef ) == "N"
         this.setSize( xDef )
      elseif valType( xDef ) == "C"
         this.initStruct( xDef )
      endif
   endif

   function setSize( nBytes )
      if nBytes > this.length * 2
         this.string += replicate( chr( 0 ), ceiling( nBytes / 2 ) - this.length )
      elseif nBytes < this.length * 2
         this.string = this.left( ceiling( nBytes / 2 ) )
      endif

   function getSize()
      if this.structDef == null
        return this.length * 2
      else
        return this.structDef.size
      endif

   function regStruct( cName, field1, type1, field2, type2, fieldN, typeN )
      local aStructDef, n
      if argcount() >= 3
         aStructDef = new Array()
         for n = 2 to argcount()
            aStructDef.add( argvector( n ) )
         endfor
         this.setStructReg( cName, aStructDef )
      endif

   function isRegStruct( cName )
      return ( this.getStructReg( cName ) # null )

   function initStruct( cName )
      local oStructDef
      oStructDef = this.getStructReg( cName )
      if oStructDef == null
         return false
      endif
      this.structName := upper( cName )
      this.structDef  := oStructDef
      this.string     := ""
      this.setSize( this.structDef.size )
      return this.structDef.size
               
   function getMethodTable()
      local oRet
      oRet = new Object()
      oRet[ TYPE_CHAR    ] = {|o,i| o.getChar( i, true )}
      oRet[ TYPE_UCHAR   ] = {|o,i| o.getChar( i )}
      oRet[ TYPE_SHORT   ] = {|o,i| o.getShort( i, true )}
      oRet[ TYPE_USHORT  ] = {|o,i| o.getShort( i )}
      oRet[ TYPE_INT     ] = {|o,i| o.getLong( i, true )}
      oRet[ TYPE_UINT    ] = {|o,i| o.getLong( i )}
      oRet[ TYPE_LONG    ] = {|o,i| o.getLong( i, true )}
      oRet[ TYPE_ULONG   ] = {|o,i| o.getLong( i )}
      oRet[ TYPE_FLOAT   ] = {|o,i| o.getFloat( i )}
      oRet[ TYPE_DOUBLE  ] = {|o,i| o.getDouble( i )}
      oRet[ TYPE_LDOUBLE ] = {|o,i| o.getLDouble( i )}
      oRet[ TYPE_BOOL8   ] = {|o,i| o.getBoolean8( i )}
      oRet[ TYPE_BOOL16  ] = {|o,i| o.getBoolean16( i )}
      oRet[ TYPE_BOOL32  ] = {|o,i| o.getBoolean32( i )}
      return oRet

   function setMethodTable()
      local oRet
      oRet = new Object()
      oRet[ TYPE_CHAR    ] = {|o,i,v| o.setChar( i, v )}
      oRet[ TYPE_UCHAR   ] = {|o,i,v| o.setChar( i, v )}
      oRet[ TYPE_SHORT   ] = {|o,i,v| o.setShort( i, v )}
      oRet[ TYPE_USHORT  ] = {|o,i,v| o.setShort( i, v )}
      oRet[ TYPE_INT     ] = {|o,i,v| o.setLong( i, v )}
      oRet[ TYPE_UINT    ] = {|o,i,v| o.setLong( i, v )}
      oRet[ TYPE_LONG    ] = {|o,i,v| o.setLong( i, v )}
      oRet[ TYPE_ULONG   ] = {|o,i,v| o.setLong( i, v )}
      oRet[ TYPE_FLOAT   ] = {|o,i,v| o.setFloat( i, v )}
      oRet[ TYPE_DOUBLE  ] = {|o,i,v| o.setDouble( i, v )}
      oRet[ TYPE_LDOUBLE ] = {|o,i,v| o.setLDouble( i, v )}
      oRet[ TYPE_BOOL8   ] = {|o,i,v| o.setBoolean8( i, v )}
      oRet[ TYPE_BOOL16  ] = {|o,i,v| o.setBoolean16( i, v )}
      oRet[ TYPE_BOOL32  ] = {|o,i,v| o.setBoolean32( i, v )}
      return oRet

   function getMem( cName )
      static getMethodTable = this.getMethodTable()
      local nType
      nType = this.structDef.type[ upper( cName ) ]
      if nType < 0
         this.getMethod := {|i,l| this.getStringZ( i, l )}
         return this.getMethod( this.fieldOffset( cName ), this.structDef.stringLen[ upper( cName ) ] )
      else
         this.getMethod := getMethodTable[ nType ]
         // Pass this in parens to force dereference of VStruct object,
         // not sparse array returned by getMethodTable()
         return this.getMethod( (this), this.fieldOffset( cName ) )
      endif

   function setMem( cName, xValue )
      static setMethodTable = this.setMethodTable()
      local nType
      nType = this.structDef.type[ upper( cName ) ]
      if nType < 0
         this.setMethod := {|i,v,l| this.setStringZ( i, v, l )}
         this.setMethod( this.fieldOffset( cName ), xValue, this.structDef.stringLen[ upper( cName ) ] )
      else
         this.setMethod := setMethodTable[ nType ]
         // Pass this in parens to force dereference of VStruct object,
         // not sparse array returned by setMethodTable()
         this.setMethod( (this), this.fieldOffset( cName ), xValue )
      endif

   function fieldOffset( xArg )
      if valType( xArg ) == "C"
         return this.structDef.offset[ upper( xArg ) ]
      else
         return xArg
      endif

   function getChar( nIndex, bSigned )
      local nRet
      nRet = this.getByte( nIndex )
      if bSigned
         if nRet >= 0x80
            nRet -= 0x100
         endif
      endif
      return int( nRet )

   function setChar( nIndex, nValue )
      this.setByte( nIndex, bitand( nValue, 0xFF ))

   function getShort( nIndex, bSigned )
      local nRet
      nRet = this.getByte( nIndex ) + ;
             bitlshift( this.getByte( nIndex + 1 ),  8 )
      if bSigned
         if nRet >= 0x8000
            nRet -= 0x10000
         endif
      endif
      return int( nRet )

   function setShort( nIndex, nValue )
      this.setByte( nIndex, bitand( nValue, 0xFF ))
      this.setByte( nIndex + 1, bitand( bitzrshift( nValue,  8 ), 0xFF ))

   function getLong( nIndex, bSigned )
      local nRet
      nRet = this.getByte( nIndex ) + ;
             bitlshift( this.getByte( nIndex + 1 ),  8 ) + ;
             bitlshift( this.getByte( nIndex + 2 ), 16 ) + ;
             bitlshift( this.getByte( nIndex + 3 ), 24 )
      if bSigned
         if bitset( nRet, 31 )
            nRet := -bitnot( nRet ) - 1
         endif
      endif
      return int( nRet )

   function setLong( nIndex, nValue )
      this.setByte( nIndex, bitand( nValue, 0xFF ))
      this.setByte( nIndex + 1, bitand( bitzrshift( nValue,  8 ), 0xFF ))
      this.setByte( nIndex + 2, bitand( bitzrshift( nValue, 16 ), 0xFF ))
      this.setByte( nIndex + 3, bitand( bitzrshift( nValue, 24 ), 0xFF ))
      
   function getBoolean8( nIndex )
      // Test for non-zero; API may not return one for TRUE
      return ( this.getChar( nIndex ) # 0 )

   function getBoolean16( nIndex )
      return ( this.getShort( nIndex ) # 0 )

   function getBoolean32( nIndex )
      return ( this.getLong( nIndex ) # 0 )

   function setBoolean8( nIndex, bValue )
      this.setChar( nIndex, iif( bValue, 1, 0 ) )

   function setBoolean16( nIndex, bValue )
      this.setShort( nIndex, iif( bValue, 1, 0 ) )

   function setBoolean32( nIndex, bValue )
      this.setLong( nIndex, iif( bValue, 1, 0 ) )

   function getString( nIndex, nChars )
      local cRet, n, c
      cRet = ""
      for n = nIndex to nIndex + nChars - 1
         cRet += chr( this.getByte( n ) )
      endfor
      return cRet

   function getStringZ( nIndex, nChars )
      local cRet, n
      cRet = this.getString( nIndex, nChars )
      n = at( chr( 0 ), cRet )
      if n > 0
         cRet := left( cRet, n - 1 )
      endif
      return cRet

   function setString( nIndex, cValue, nChars )
      // Writes whatever portion of string will fit
      if argcount() < 3
         nChars = len( cValue )
      endif
      if nIndex + nChars > this.getSize()
         throw new VStructException( "setString: overflows structure" )
      endif
      local n
      for n = 0 to min( len( cValue ), nChars ) - 1
         this.setByte( nIndex + n, asc( cValue.charAt( n )) )
      endfor
      do while n < nChars
         this.setByte( nIndex + n++, 0 )
      enddo

   function setStringZ( nIndex, cValue, nWidth )
      // Make sure that string is at least one character smaller
      // than string member width to leave room for null-termination
      this.setString( nIndex, left( cValue, nWidth - 1 ), nWidth )

   #define DOUBLE_EBIAS      0x3ff
   #define DOUBLE_MANTBITS   52
   // Format of 64-bit double is:
   //   Bits      Description
   // --------  -------------------------------------
   //    63     Sign (1 == negative)
   //  62 - 52  Exponent (biased)
   //  51 - 0   Mantissa (with implied leading 1 bit)

   function getDouble( nIndex )
      if this.getLong( nIndex + 4 ) == 0 and this.getLong( nIndex ) == 0
         return 0.0
      endif
      local bNeg, nExp, nMant, n
      n = this.getByte( nIndex + 7 )
      bNeg = bitset( n, 7 )  // Get sign from high bit of high byte
      nExp = bitlshift( bitand( n, 0x7F ), 4 )
      n = this.getByte( nIndex + 6 )
      nExp += bitzrshift( n, 4 )
      nMant = 0x10 + bitand( n, 0x0F )  // Mantissa has implied leading 1 bit
      for n = 5 to 0 step -1
         nMant *= 256.0                 // Workaround: Must multiply by floating point number
         nMant += this.getByte( nIndex + n )  // to force promotion to double from integer
      endfor
      nMant *= 2^( nExp - DOUBLE_EBIAS - DOUBLE_MANTBITS )
      return iif( bNeg, -nMant, nMant )

   function setDouble( nIndex, nValue )
      if nValue == 0
         this.setLong( nIndex, 0 )
         this.setLong( nIndex + 4, 0 )
         return
      endif
      local nAbs, bNeg, nExp, nMant, n
      bNeg  = nValue < 0
      nAbs  = abs( nValue )
      nExp  = floor( log( nAbs ) / log( 2 ) )
      nMant = floor( nAbs / 2^( nExp - DOUBLE_MANTBITS ) )
      nExp += DOUBLE_EBIAS
      for n = 0 to 5
         this.setByte( nIndex + n, nMant % 0x100 )
         nMant /= 0x100
      endfor
      nMant := bitand( nMant, 0x0F )
      this.setByte( nIndex + 6, nMant + bitlshift( bitand( nExp, 0x0F ), 4 ) )
      nExp := bitzrshift( nExp, 4 )
      this.setByte( nIndex + 7, nExp + iif( bNeg, 0x80, 0 ) )

   #define LDOUBLE_EBIAS      0x3fff
   #define LDOUBLE_MANTBITS   64
   // Format of 80-bit long double is:
   //   Bits      Description
   // --------  -------------------------------------
   //    79     Sign (1 == negative)
   //  78 - 64  Exponent (biased)
   //  63 - 0   Mantissa (no implied leading 1 bit)

   function getLDouble( nIndex )
      if this.getLong( nIndex + 6 ) == 0 and this.getLong( nIndex + 2 ) == 0 and this.getShort( nIndex ) == 0
         return 0.0
      endif
      local bNeg, nExp, nMant, n
      n = this.getShort( nIndex + 8 )
      bNeg  = bitset( n, 15 )  // Get sign from high bit of high byte
      nExp  = bitand( n, 0x7FFF )
      nMant = 0
      for n = 7 to 0 step -1
         nMant *= 256.0                 // Workaround: Must multiply by floating point number
         nMant += this.getByte( nIndex + n )  // to force promotion to double from integer
      endfor
      nMant *= 2^( nExp - LDOUBLE_EBIAS - LDOUBLE_MANTBITS + 1 )
      return iif( bNeg, -nMant, nMant )

   function setLDouble( nIndex, nValue )
      if nValue == 0
         this.setLong( nIndex, 0 )
         this.setLong( nIndex + 4, 0 )
         this.setShort( nIndex + 8, 0 )
         return
      endif
      local nAbs, bNeg, nExp, nMant, n
      bNeg  = nValue < 0
      nAbs  = abs( nValue )
      nExp  = floor( log( nAbs ) / log( 2 ) )
      nMant = floor( nAbs / 2^( nExp - LDOUBLE_MANTBITS + 1 ) )
      nExp += LDOUBLE_EBIAS
      for n = 0 to 7
         this.setByte( nIndex + n, nMant % 0x100 )
         nMant /= 0x100
      endfor
      this.setByte( nIndex + 8, bitand( nExp, 0xFF ) )
      nExp := bitzrshift( nExp, 8 )
      this.setByte( nIndex + 9, nExp + iif( bNeg, 0x80, 0 ) )

   function getHexChars( nIndex, nBytes, bReverse )
      local cRet, n
      cRet = ""
      if bReverse    // Little-endian: AB 12 34 --> "3412AB"
         for n = 0 to nBytes - 1
            cRet := itoh( this.getByte( nIndex + n ), 2 ) + cRet
         endfor
      else
         for n = 0 to nBytes - 1
            cRet += itoh( this.getByte( nIndex + n ), 2 )
         endfor
      endif
      return cRet

   function setHexChars( nIndex, cValue, bReverse )
      local c, n
      c = space( len( cValue ) % 2 ) + cValue   // Ensure even number of hex digits
      n = 0
      if bReverse    // Little-endian: "12AB34" --> 34 AB 12
         do while not empty( c )
            this.setByte( nIndex + n++, htoi( right( c, 2 )) )
            c := left( c, len( c ) - 2 )
         enddo
      else
         do while not empty( c )
            this.setByte( nIndex + n++, htoi( left( c, 2 )) )
            c := right( c, len( c ) - 2 )
         enddo
      endif

   function getBinaryChars( nIndex, nBytes, bReverse )
      local cHex, cRet, aBin, n
      cHex = this.getHexChars( nIndex, nBytes, bReverse )
      cRet = ""
      aBin = { "0000", "0001", "0010", "0011", "0100", "0101", "0110", "0111", ;
               "1000", "1001", "1010", "1011", "1100", "1101", "1110", "1111" }
      for n = 1 to nBytes
         cRet += aBin[ htoi( substr( cHex, n * 2 - 1, 1 )) + 1 ] + ;
                 aBin[ htoi( substr( cHex, n * 2    , 1 )) + 1 ] + " "
      endfor
      return trim( cRet )

endclass     


class VStructDef( cName, aStructDef )

   this.structName = upper( cName )
   this.offset     = new AssocArray()
   this.type       = new AssocArray()
   this.stringLen  = new AssocArray()
   this.size       = this.buildStruct( aStructDef )

   function buildStruct( aStructDef )   
      static sizeTable = this.sizeTable()
      local n, nOffset, cMem, nType
      nOffset = 0
      for n = 1 to aStructDef.size / 2
         cMem = upper( aStructDef[ n * 2 - 1 ] )
         if this.offset.isKey( cMem )
            throw new VStructException( "VSTRUCT_DEF: Duplicate member: " + cMem )
         endif
         nType  = aStructDef[ n * 2 ]
         this.offset[ cMem ] = nOffset
         this.type[ cMem ]   = nType
         if nType < 0
            this.stringLen[ cMem ] = -nType
         endif
         nOffset += iif( nType < 0, -nType, sizeTable[ nType ] )
      endfor
      return nOffset

   function sizeTable()
      // TYPE_ and SIZEOF_ from STRUCTAPI.H
      local oRet
      oRet = new Object()
      oRet[ TYPE_CHAR    ] = SIZEOF_CHAR
      oRet[ TYPE_UCHAR   ] = SIZEOF_UCHAR
      oRet[ TYPE_SHORT   ] = SIZEOF_SHORT
      oRet[ TYPE_USHORT  ] = SIZEOF_USHORT
      oRet[ TYPE_INT     ] = SIZEOF_INT
      oRet[ TYPE_UINT    ] = SIZEOF_UINT
      oRet[ TYPE_LONG    ] = SIZEOF_LONG
      oRet[ TYPE_ULONG   ] = SIZEOF_ULONG
      oRet[ TYPE_FLOAT   ] = SIZEOF_FLOAT
      oRet[ TYPE_DOUBLE  ] = SIZEOF_DOUBLE
      oRet[ TYPE_LDOUBLE ] = SIZEOF_LDOUBLE
      oRet[ TYPE_BOOL8   ] = SIZEOF_BOOL8
      oRet[ TYPE_BOOL16  ] = SIZEOF_BOOL16
      oRet[ TYPE_BOOL32  ] = SIZEOF_BOOL32
      return oRet

endclass


class VStructException( cMsg ) of Exception

   this.message := cMsg

endclass


function valType
   parameter x
   return iif( x == null, "0", type( "x" ) )
